@@ -14,7 +14,11 @@
  * Files expected to exist in the playground directory.
  */
 
-#define RECORD		"record"  /* a file containing list of topscorers */
+#ifdef DEBIAN
+# define RECORD		"/var/lib/games/nethack/record"
+#else
+# define RECORD		"record"  /* a file containing list of topscorers */
+#endif
 #define HELP		"help"	  /* a file containing command descriptions */
 #define SHELP		"hh"		/* abbreviated form of the same */
 #define DEBUGHELP	"wizhelp"	/* a file containing debug mode cmds */
