@@ -9,6 +9,13 @@
 #include "config.h"
 #endif
 
+/* Some people comment out OVERLAY when uncommenting MOVERLAY */
+#if defined(MOVERLAY)
+# ifndef OVERLAY
+#  define OVERLAY
+# endif
+#endif
+
 /*	For debugging beta code.	*/
 #ifdef BETA
 #define Dpline	pline
@@ -71,7 +78,7 @@
 #include "decl.h"
 #endif
 
-extern coord bhitpos;	/* place where thrown weapon falls to the ground */
+NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground */
 
 /* types of calls to bhit() */
 #define ZAPPED_WAND	0
