@@ -12,7 +12,7 @@
 #endif
 
 /* might display need graphics code? */
-#if !defined(AMIGA) && !defined(TOS)
+#if !defined(AMIGA) && !defined(TOS) && !defined(MAC)
 # if defined(TERMLIB) || defined(OS2) || defined(MSDOS)
 #  define ASCIIGRAPH
 # endif
@@ -49,7 +49,7 @@ extern short ospeed;		/* set up in termcap.c */
 # ifdef TOS
 extern const char *hilites[MAXCOLORS];
 # else
-extern char NEARDATA *hilites[MAXCOLORS];
+extern NEARDATA char *hilites[MAXCOLORS];
 # endif
 #endif
 
