@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)attrib.c	3.1	92/10/26	*/
+/*	SCCS Id: @(#)attrib.c	3.1	93/02/17	*/
 /*	Copyright 1988, 1989, 1990, 1992, M. Stephenson		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -156,7 +156,7 @@ const struct clattr {
 		    { A_NEUTRAL, 0 },  12, 10, 14,  1,  0 };
 
 static long next_check = 600L;	/* arbitrary first setting */
-static const struct clattr NEARDATA *NDECL(clx);
+static NEARDATA const struct clattr *NDECL(clx);
 static void NDECL(init_align);
 static void NDECL(exerper);
 
@@ -449,7 +449,7 @@ exerchk()
 				break;
 		    case A_WIS: You((mod_val >0) ?
 				    "must have been very observant." :
-				    "must not have been paying attention.");
+				    "haven't been paying attention.");
 				break;
 		    case A_DEX: You((mod_val >0) ?
 				    "must have been working on your reflexes." :
@@ -457,7 +457,7 @@ exerchk()
 				break;
 		    case A_CON: You((mod_val >0) ?
 				    "must be leading a healthy life-style." :
-				    "must not have been watching your health.");
+				    "haven't been watching your health.");
 				break;
 		    }
 		}
@@ -469,6 +469,13 @@ exerchk()
 	}
 }
 
+/* next_check will otherwise have its initial 600L after a game restore */
+void
+reset_attribute_clock()
+{
+	if (moves > 600L) next_check = moves + rn1(50,800);
+}
+
 static const struct	clattr *
 clx()  {
 
@@ -677,7 +684,11 @@ int x;
 
 	if (x == A_STR) {
 		if (uarmg && uarmg->otyp == GAUNTLETS_OF_POWER) return(125);
-		else return((tmp >= 125) ? 125 : (tmp <= 3) ? 3 : tmp);
+#ifdef WIN32_BUG
+		else return(x=((tmp >= 125) ? 125 : (tmp <= 3) ? 3 : tmp));
+#else
+		else return((schar)((tmp >= 125) ? 125 : (tmp <= 3) ? 3 : tmp));
+#endif
 	} 
 #ifdef POLYSELF
 	else if(x == A_CHA) {
@@ -686,7 +697,11 @@ int x;
 		    return 18;
 	}
 #endif
-	return((tmp >= 25) ? 25 : (tmp <= 3) ? 3 : tmp);
+#ifdef WIN32_BUG
+	return(x=((tmp >= 25) ? 25 : (tmp <= 3) ? 3 : tmp));
+#else
+	return((schar)((tmp >= 25) ? 25 : (tmp <= 3) ? 3 : tmp));
+#endif
 }
 
 schar
@@ -696,9 +711,9 @@ acurrstr()
 {
 	register int str = ACURR(A_STR);
 
-	if (str <= 18) return str;
-	if (str <= 121) return (19 + str / 50); /* map to 19-21 */
-	else return str - 100;
+	if (str <= 18) return((schar)str);
+	if (str <= 121) return((schar)(19 + str / 50)); /* map to 19-21 */
+	else return((schar)(str - 100));
 }
 
 #endif /* OVL0 */
