@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)bones.c	3.1	93/01/07	*/
+/*	SCCS Id: @(#)bones.c	3.1	93/06/05	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985,1993. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -26,7 +26,7 @@ d_level *lev;
 
 	if (ledger_no(&save_dlevel)) assign_level(lev, &save_dlevel);
 
-	return (((sptr = Is_special(lev)) && !sptr->boneid)
+	return (boolean)(((sptr = Is_special(lev)) && !sptr->boneid)
 		|| !dungeons[lev->dnum].boneid
 		   /* no bones on the last or multiway branch levels */
 		   /* in any dungeon (level 1 isn't multiway).       */
@@ -83,36 +83,47 @@ boolean restore;
 			otmp->invlet = 0;
 #ifdef TUTTI_FRUTTI
 			if(otmp->otyp == SLIME_MOLD) goodfruit(otmp->spe);
+			else
 #endif
 #ifdef MAIL
 			if (otmp->otyp == SCR_MAIL) otmp->spe = 1;
+			else
 #endif
 #ifdef POLYSELF
 			if (otmp->otyp == EGG) otmp->spe = 0;
+			else
 #endif
-			if(otmp->otyp == AMULET_OF_YENDOR) {
-				/* no longer the actual amulet */
-				otmp->otyp = FAKE_AMULET_OF_YENDOR;
-				curse(otmp);
-			}
-			if(otmp->otyp == CANDELABRUM_OF_INVOCATION) {
-			    if(otmp->spe > 0) { /* leave candles, if any */
-			        otmp->otyp = WAX_CANDLE;
-				otmp->age = 50L;  /* assume used */
-				otmp->quan = (long)otmp->spe;
-				otmp->lamplit = 0;
-				otmp->spe = 0;
-			    } else obfree(otmp, (struct obj *)0);
-			}
-			if(otmp->otyp == BELL_OF_OPENING) otmp->otyp = BELL;
-			if(otmp->otyp == SPE_BOOK_OF_THE_DEAD) {
-			    otmp->otyp = SPE_MAGIC_MISSILE +
-			                    rn2(SPE_BLANK_PAPER -
-						  SPE_MAGIC_MISSILE + 1);
+			if (otmp->otyp == AMULET_OF_YENDOR) {
+			    /* no longer the real Amulet */
+			    otmp->otyp = FAKE_AMULET_OF_YENDOR;
 			    curse(otmp);
+			} else if (otmp->otyp == CANDELABRUM_OF_INVOCATION) {
+			    otmp->otyp = WAX_CANDLE;
+			    otmp->age = 50L;  /* assume used */
+			    if (otmp->spe > 0)
+				otmp->quan = (long)otmp->spe;
+			    otmp->lamplit = 0;
+			    otmp->spe = 0;
+			    otmp->owt = weight(otmp);
+			} else if (otmp->otyp == BELL_OF_OPENING) {
+			    otmp->otyp = BELL;
+			    curse(otmp);
+			} else if (otmp->otyp == SPE_BOOK_OF_THE_DEAD) {
+			    otmp->otyp = SPE_BLANK_PAPER;
+			    curse(otmp);
+#ifdef MULDGN
+			} else if (is_quest_artifact(otmp)) {
+			    /*
+			     * never leave our own quest artifact among the
+			     * bones; others (via wishing) might remain though
+			     */
+			 /* artifact_unexist(otmp); */
+			    otmp->oartifact = 0;
+			    ONAME(otmp)[0] = '\0';
+#endif
 			}
 		}
-	}			
+	}
 }
 
 static void
@@ -161,15 +172,22 @@ savebones()
 #ifdef TUTTI_FRUTTI
 	struct fruit *f;
 #endif
-	char *bonesid;
+	char c, *bonesid;
 
 	if(ledger_no(&u.uz) <= 0 || ledger_no(&u.uz) > maxledgerno()) return;
 	if(no_bones_level(&u.uz)) return; /* no bones for specific levels */
-	if(!rn2(1 + (depth(&u.uz)>>2)) /* fewer ghosts on low levels */
+	if(!Is_branchlev(&u.uz)) {
+	    /* no bones on non-branches with portals */
+	    for(ttmp = ftrap; ttmp; ttmp = ttmp->ntrap)
+		if (ttmp->ttyp == MAGIC_PORTAL) return;
+	}
+
+	if(depth(&u.uz) <= 0 ||		/* bulletproofing for endgame */
+	   (!rn2(1 + (depth(&u.uz)>>2)) /* fewer ghosts on low levels */
 #ifdef WIZARD
 		&& !wizard
 #endif
-		) return;
+		)) return;
 #ifdef EXPLORE_MODE
 	/* don't let multiple restarts generate multiple copies of objects
 	 * in bones files */
@@ -259,10 +277,10 @@ savebones()
 		if(mtmp->mtame) mtmp->mtame = mtmp->mpeaceful = 0;
 	}
 	for(ttmp = ftrap; ttmp; ttmp = ttmp->ntrap) {
-	        if(ttmp->ttyp == MAGIC_PORTAL) deltrap(ttmp);
 		ttmp->tseen = 0;
 	}
 	resetobjs(fobj,FALSE);
+	resetobjs(level.buriedobjlist, FALSE);
 
 	/* Clear all memory from the level. */
 	for(x=0; x<COLNO; x++) for(y=0; y<ROWNO; y++) {
@@ -303,7 +321,9 @@ savebones()
 	co_false();	/* make sure bonesid and savefruitchn get written */
 #endif /* MFLOPPY */
 
-	bwrite(fd, (genericptr_t) bonesid, 7);	/* DD.nnn */
+	c = (char) (strlen(bonesid) + 1);
+	bwrite(fd, (genericptr_t) &c, sizeof c);
+	bwrite(fd, (genericptr_t) bonesid, (unsigned) c);	/* DD.nnn */
 #ifdef TUTTI_FRUTTI
 	savefruitchn(fd, WRITE_SAVE | FREE_SAVE);
 #endif
@@ -317,7 +337,7 @@ getbones()
 {
 	register int fd;
 	register int ok;
-	char *bonesid, oldbonesid[7];
+	char c, *bonesid, oldbonesid[10];
 
 #ifdef EXPLORE_MODE
 	if(discover)		/* save bones files for real games */
@@ -344,7 +364,8 @@ getbones()
 		}
 #endif
 		minit();	/* ZEROCOMP */
-		mread(fd, (genericptr_t) oldbonesid, 7);	/* DD.nnn */
+		mread(fd, (genericptr_t) &c, sizeof c);	/* length incl. '\0' */
+		mread(fd, (genericptr_t) oldbonesid, (unsigned) c); /* DD.nnn */
 		if (strcmp(bonesid, oldbonesid)) {
 #ifdef WIZARD
 			if (wizard) {
@@ -363,6 +384,7 @@ getbones()
 			for(mtmp = fmon; mtmp; mtmp = mtmp->nmon)
 				resetobjs(mtmp->minvent,TRUE);
 			resetobjs(fobj,TRUE);
+			resetobjs(level.buriedobjlist,TRUE);
 		}
 	}
 	(void) close(fd);
