@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)detect.c	3.1	92/12/16	*/
+/*	SCCS Id: @(#)detect.c	3.1	93/06/15	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -30,10 +30,10 @@ char oclass;
 
     if (obj->oclass == oclass) return obj;
 
-    if (Is_container(obj)) {
+    if (Has_contents(obj)) {
 	for (otmp = obj->cobj; otmp; otmp = otmp->nobj)
 	    if (otmp->oclass == oclass) return otmp;
-	    else if (Is_container(otmp) && (temp = o_in(otmp, oclass)))
+	    else if (Has_contents(otmp) && (temp = o_in(otmp, oclass)))
 		return temp;
     }
     return (struct obj *) 0;
@@ -53,9 +53,9 @@ register char oclass;
 	if (glyph_is_object(lev->glyph)) {
 	    /* there's some object shown here */
 	    if (oclass == ALL_CLASSES) {
-		return !(level.objects[x][y] ||     /* stale if nothing here */
+		return((boolean)( !(level.objects[x][y] ||     /* stale if nothing here */
 			    ((mtmp = m_at(x,y)) != 0 &&
-				(mtmp->mgold || mtmp->minvent)));
+				(mtmp->mgold || mtmp->minvent)))));
 	    } else if (objects[glyph_to_obj(lev->glyph)].oc_class == oclass) {
 		/* the object shown here is of interest */
 		for (otmp = level.objects[x][y]; otmp; otmp = otmp->nexthere)
@@ -209,7 +209,7 @@ register struct obj	*sobj;
 	known = stale && !confused;
 	if (stale) {
 	    docrt();
-	    You("sense lack of %s nearby.", what);
+	    You("sense a lack of %s nearby.", what);
 	} else if (sobj)
 	    strange_feeling(sobj, "Your nose twitches.");
 	return !stale;
@@ -535,7 +535,7 @@ const char *
 level_distance(where)
 d_level *where;
 {
-    register xchar ll = depth(&u.uz) - depth(where);
+    register schar ll = depth(&u.uz) - depth(where);
     register boolean indun = (u.uz.dnum == where->dnum);
 
     if (ll < 0) {
@@ -587,7 +587,7 @@ struct obj *obj;
     }
     oops = (rnd(20) > ACURR(A_INT) || obj->cursed);
     if (oops && (obj->spe > 0)) {
-	switch(rnd(5)) {
+	switch (rnd(obj->oartifact ? 4 : 5)) {
 	case 1 : pline("%s is too much to comprehend!", The(bname));
 	    break;
 	case 2 : pline("%s confuses you!", The(bname));
@@ -605,6 +605,7 @@ struct obj *obj;
 	    break;
 	}
 	obj->spe--;
+	check_unpaid(obj);
 	return;
     }
 
@@ -628,6 +629,7 @@ struct obj *obj;
 		break;
 	    }
 	    obj->spe--;
+	    check_unpaid(obj);
 	}
 	return;
     }
@@ -639,7 +641,7 @@ struct obj *obj;
 	if (flags.verbose) pline("Never mind.");
 	return;
     }
-    You("peer into %s", the(bname));
+    You("peer into %s...", the(bname));
     nomul(-rnd(10));
     nomovemsg = "";
     if (obj->spe <= 0)
@@ -650,6 +652,7 @@ struct obj *obj;
 
 	makeknown(CRYSTAL_BALL);
 	obj->spe--;
+	check_unpaid(obj);
 
 	if ((class = def_char_to_objclass(ch)) != MAXOCLASSES)
 		ret = object_detect((struct obj *)0, class);
@@ -661,19 +664,20 @@ struct obj *obj;
 		    break;
 		default:
 		    {
-		    short i = rn2(SIZE(level_detects));
+		    int i = rn2(SIZE(level_detects));
 		    You("see %s, %s.",
 			level_detects[i].what,
 			level_distance(level_detects[i].where));
 		    }
-		    ret = 1;
+		    ret = 0;
 		    break;
 	}
 
-	if (!ret)
+	if (ret) {
 	    if (!rn2(100))  /* make them nervous */
 		You("see the Wizard of Yendor gazing out at you.");
-		else pline("The vision is unclear.");
+	    else pline("The vision is unclear.");
+	}
     }
     return;
 }
@@ -806,7 +810,7 @@ genericptr_t num;
 		if(levl[zx][zy].typ == SDOOR)
 		    levl[zx][zy].typ = DOOR;
 		if(levl[zx][zy].doormask & D_TRAPPED) {
-		    if(distu(zx, zy) < 3) b_trapped("door");
+		    if(distu(zx, zy) < 3) b_trapped("door", 0);
 		    else Norep("You %s an explosion!",
 				cansee(zx, zy) ? "see" :
 				   (flags.soundok ? "hear" :
