@@ -1,9 +1,9 @@
-/*	SCCS Id: @(#)dokick.c	3.1	92/10/06	*/
+/*	SCCS Id: @(#)dokick.c	3.1	93/06/15	*/
 /* Copyright (c) Izchak Miller, Mike Stephenson, Steve Linhart, 1989. */
 /* NetHack may be freely redistributed.  See license for details. */
 
-#include	"hack.h"
-#include	"eshk.h"
+#include "hack.h"
+#include "eshk.h"
 
 #ifndef POLYSELF
 # define martial()	(pl_character[0] == 'S' || pl_character[0] == 'P')
@@ -13,7 +13,7 @@
 			 || is_bigfoot(uasmon))
 #endif
 
-static struct rm NEARDATA *maploc;
+static NEARDATA struct rm *maploc;
 
 extern boolean notonhead;	/* for long worms */
 
@@ -25,7 +25,7 @@ static void FDECL(otransit_msg, (struct obj *, XCHAR_P, BOOLEAN_P, int));
 static const char *FDECL(gate_str, (XCHAR_P));
 static void FDECL(drop_to, (coord *, XCHAR_P));
 
-static struct obj NEARDATA *kickobj;
+static NEARDATA struct obj *kickobj;
 
 #define IS_SHOP(x)	(rooms[x].rtype >= SHOPBASE)
 
@@ -46,14 +46,15 @@ register boolean clumsy;
 	/* a good kick exercises your dex */
 	exercise(A_DEX, TRUE);
 
+/*	it is unchivalrous to attack the defenseless or from behind */
+	if (pl_character[0] == 'K' &&
+		u.ualign.type == A_LAWFUL && u.ualign.record > -10 &&
+		(!mon->mcanmove || mon->msleep || mon->mflee))
+		adjalign(-1);
+
 	/* squeeze some guilt feelings... */
 	if(mon->mtame) {
-#ifdef SOUNDS
-	    if (rn2(10)) yelp(mon);
-	    else growl(mon); /* give them a moment's worry */
-#endif
-	    mon->mtame--;
-	    if(!mon->mtame) newsym(mon->mx, mon->my);
+	    abuse_dog(mon);
 	    mon->mflee = mon->mtame ? 1 : 0;
 #ifdef HISX
 	    mon->mfleetim = mon->mfleetim + (dmg ? rnd(dmg) : 1);
@@ -62,7 +63,7 @@ register boolean clumsy;
 #endif
 	}
 
-	if (dmg)
+	if (dmg > 0)
 		mon->mhp -= (!martial() ? rnd(dmg) :
 			rnd(dmg)+rnd(ACURR(A_DEX)/2));
 	if(mon->mhp < 1) {
@@ -70,13 +71,15 @@ register boolean clumsy;
 		killed(mon);
 		return;
 	}
-	if(martial() && !bigmonst(mon->data) && !rn2(3) && mon->mcanmove) {
+	if(martial() && !bigmonst(mon->data) && !rn2(3) && mon->mcanmove
+	   && mon != u.ustuck) {
 		/* see if the monster has a place to move into */
 		mdx = mon->mx + u.dx;
 		mdy = mon->my + u.dy;
 		if(goodpos(mdx, mdy, mon, mon->data)) {
 			pline("%s reels from the blow.", Monnam(mon));
 			remove_monster(mon->mx, mon->my);
+			newsym(mon->mx, mon->my);
 			place_monster(mon, mdx, mdy);
 			newsym(mon->mx, mon->my);
 			set_apparxy(mon);
@@ -84,12 +87,6 @@ register boolean clumsy;
 	}
 	(void) passive(mon, FALSE, 1, TRUE);
 
-/*	it is unchivalrous to attack the defenseless or from behind */
-	if (pl_character[0] == 'K' &&
-		u.ualign.type == A_LAWFUL && u.ualign.record > -10 &&
-		(!mon->mcanmove || mon->msleep || mon->mflee))
-		adjalign(-1);
-
 }
 
 static void
@@ -231,8 +228,8 @@ register struct obj *gold;
 				robbed -= gold->quan;
 				if (robbed < 0) robbed = 0;
 				pline("The amount %scovers %s recent losses.",
-					!robbed ? "" : "partially ",
-					mtmp->female ? "her" : "his");
+				      !robbed ? "" : "partially ",
+				      his[mtmp->female]);
 				ESHK(mtmp)->robbed = robbed;
 				if(!robbed)
 					make_happy_shk(mtmp, FALSE);
@@ -250,33 +247,31 @@ register struct obj *gold;
 			    verbalize("Thank you for your contribution.");
 			else verbalize("Thanks, scum!");
 		}
-		else if(is_mercenary(mtmp->data)) {
-		    if(rn2(3)) {
-			if(mtmp->data == &mons[PM_SOLDIER]) {
-			   if(gold->quan > 100 + (u.ugold + (u.ulevel*rn2(5)))
-					    /ACURR(A_CHA))
-			    mtmp->mpeaceful = 1;
-			    }
-			if(mtmp->data == &mons[PM_SERGEANT]) {
-			   if(gold->quan > 250 + (u.ugold + (u.ulevel*rn2(5)))
-					    /ACURR(A_CHA))
-			    mtmp->mpeaceful = 1;
-			    }
-			if(mtmp->data == &mons[PM_LIEUTENANT]) {
-			   if(gold->quan > 500 + (u.ugold + (u.ulevel*rn2(5)))
-					    /ACURR(A_CHA))
-			    mtmp->mpeaceful = 1;
-			    }
-			if(mtmp->data == &mons[PM_CAPTAIN]) {
-			   if(gold->quan > 750 + (u.ugold + (u.ulevel*rn2(5)))
-					    /ACURR(A_CHA))
-			    mtmp->mpeaceful = 1;
-			    }
+#ifdef ARMY
+		else if (is_mercenary(mtmp->data)) {
+		    long goldreqd = 0L;
+
+		    if (rn2(3)) {
+			if (mtmp->data == &mons[PM_SOLDIER])
+			   goldreqd = 100L;
+			else if (mtmp->data == &mons[PM_SERGEANT])
+			   goldreqd = 250L;
+			else if (mtmp->data == &mons[PM_LIEUTENANT])
+			   goldreqd = 500L;
+			else if (mtmp->data == &mons[PM_CAPTAIN])
+			   goldreqd = 750L;
+
+			if (goldreqd) {
+			   if (gold->quan > goldreqd +
+				(u.ugold + u.ulevel*rn2(5))/ACURR(A_CHA))
+			    mtmp->mpeaceful = TRUE;
+			}
 		     }
-		     if(mtmp->mpeaceful)
+		     if (mtmp->mpeaceful)
 			    verbalize("That should do.  Now beat it!");
 		     else verbalize("That's not enough, coward!");
-		     }
+		 }
+#endif
 
 		dealloc_obj(gold);
 		return(1);
@@ -349,7 +344,7 @@ xchar x, y;
 		for(otmp = kickobj->cobj; otmp; otmp = otmp2) {
 			otmp2 = otmp->nobj;
 			if (objects[otmp->otyp].oc_material == GLASS
-								&& !rn2(3)) {
+			    && !obj_resists(otmp, 33, 100)) {
 				You("hear a muffled shatter.");
 				if(costly) loss += stolen_value(otmp, x, y, 
 					    (boolean)shkp->mpeaceful, TRUE);
@@ -441,12 +436,14 @@ xchar x, y;
 		else pline("%s come%s loose.",
 			   The(distant_name(kickobj, xname)),
 			   (kickobj->quan == 1L) ? "s" : "");
-		remove_object(kickobj);
+		freeobj(kickobj);
 		newsym(x, y);
 		if (costly && (!costly_spot(u.ux, u.uy)
 			       || !index(u.urooms, *in_rooms(x, y, SHOPBASE))))
 			addtobill(kickobj, FALSE, FALSE, FALSE);
 		if(!flooreffects(kickobj,u.ux,u.uy,"fall")) {
+		    kickobj->nobj = fobj;
+		    fobj = kickobj;
 		    place_object(kickobj, u.ux, u.uy);
 		    stackobj(kickobj);
 		    newsym(u.ux, u.uy);
@@ -512,7 +509,7 @@ xchar x, y;
 static char *
 kickstr()
 {
-	static char NEARDATA buf[BUFSZ];
+	static NEARDATA char buf[BUFSZ];
 
 	if (kickobj) Sprintf(buf, "kicking %s", doname(kickobj));
 	else {
@@ -546,45 +543,51 @@ int
 dokick()
 {
 	register int x, y;
-	register int avrg_attrib = (ACURRSTR+ACURR(A_DEX)+ACURR(A_CON))/3;
+	int avrg_attrib;
+	register struct monst *mtmp;
+	s_level *slev;
+	boolean no_kick = FALSE;
 
 #ifdef POLYSELF
-	if(nolimbs(uasmon)) {
+	if (nolimbs(uasmon)) {
 		You("have no legs to kick with.");
-		return(0);
-	}
-	if(verysmall(uasmon)) {
+		no_kick = TRUE;
+	} else if (verysmall(uasmon)) {
 		You("are too small to do any kicking.");
-		return(0);
-	}
+		no_kick = TRUE;
+	} else
 #endif
-	if(Wounded_legs) {
+	if (Wounded_legs) {
 		Your("%s %s in no shape for kicking.",
 		      ((Wounded_legs & BOTH_SIDES)==BOTH_SIDES)
 			? (const char *)makeplural(body_part(LEG)) : body_part(LEG),
 		      ((Wounded_legs & BOTH_SIDES)==BOTH_SIDES) ? "are" : "is");
-		return(0);
-	}
-
-	if(near_capacity() > SLT_ENCUMBER) {
+		no_kick = TRUE;
+	} else if (near_capacity() > SLT_ENCUMBER) {
 		Your("load is too heavy to balance yourself for a kick.");
-		return(0);
-	}
-
-	if(u.uinwater && !rn2(2)) {
+		no_kick = TRUE;
+	} else if (u.uinwater && !rn2(2)) {
 		Your("slow motion kick doesn't hit anything.");
-		return(0);
-	}
-
-	if(u.utrap) {
+		no_kick = TRUE;
+	} else if (u.utrap) {
 		switch (u.utraptype) {
 		    case TT_PIT:
-			pline("There's nothing to kick down here.");
+			pline("There's not enough room to kick down here.");
+			break;
 		    case TT_WEB:
 		    case TT_BEARTRAP:
 			You("can't move your %s!", body_part(LEG));
+			break;
+		    default:
+			break;
 		}
-		return(0);
+		no_kick = TRUE;
+	}
+
+	if (no_kick) {
+		/* discard direction typeahead, if any */
+		display_nhwindow(WIN_MESSAGE, TRUE);	/* --More-- */
+		return 0;
 	}
 
 	if(!getdir(NULL)) return(0);
@@ -592,6 +595,7 @@ dokick()
 
 	x = u.ux + u.dx;
 	y = u.uy + u.dy;
+	avrg_attrib = (ACURRSTR+ACURR(A_DEX)+ACURR(A_CON))/3;
 
 	if(u.uswallow) {
 		switch(rn2(3)) {
@@ -616,14 +620,15 @@ dokick()
 	/* non-doors, doors.			   */
 
 	if(MON_AT(x, y)) {
+		struct permonst *mdat = m_at(x,y)->data;
 		kick_monster(x, y);
 		if((Is_airlevel(&u.uz) || Levitation) && flags.move) {
 		    int range;
-		    struct monst *mon;
 
-		    mon = m_at(x,y);
-		    range = (3*(int)mon->data->cwt) /
-			((int)uasmon->cwt + (weight_cap() + inv_weight()));
+		    range = ((int)uasmon->cwt + (weight_cap() + inv_weight()));
+		    if (range < 1) range = 1; /* divide by zero avoidance */
+		    range = (3*(int)mdat->cwt) / range;
+
 		    if(range < 1) range = 1;
 		    hurtle(-u.dx, -u.dy, range);
 		}
@@ -649,8 +654,8 @@ dokick()
 			exercise(A_DEX, TRUE);
 			maploc->typ = DOOR;
 			if(maploc->doormask & D_TRAPPED) {
-			    b_trapped("door");
 			    maploc->doormask = D_NODOOR;
+			    b_trapped("door", FOOT);
 			} else
 			    maploc->doormask = D_ISOPEN;
 			if (Blind)
@@ -811,8 +816,10 @@ dumb:
 			exercise(A_STR, FALSE);
 			set_wounded_legs(RIGHT_SIDE, 5 + rnd(5));
 		}
-		if(Is_airlevel(&u.uz) || Levitation)
-		    hurtle(-u.dx, -u.dy, rn2(2));
+		if ((Is_airlevel(&u.uz) || Levitation) && rn2(2)) {
+		    hurtle(-u.dx, -u.dy, 1);
+		    return 1;		/* you moved, so use up a turn */
+		}
 		return(0);
 	}
 
@@ -824,10 +831,10 @@ dumb:
 	if(rnl(35) < avrg_attrib + (!martial() ? 0 : ACURR(A_DEX))) {
 		/* break the door */
 		if(maploc->doormask & D_TRAPPED) {
-		    pline("As you kick the door, it explodes!");
+		    if (flags.verbose) You("kick the door.");
 		    exercise(A_STR, FALSE);
-		    b_trapped("door");
 		    maploc->doormask = D_NODOOR;
+		    b_trapped("door", FOOT);
 		} else if(ACURR(A_STR) > 18 && !rn2(5) &&
 			  !*in_rooms(x, y, SHOPBASE)) {
 		    pline("As you kick the door, it shatters to pieces!");
@@ -847,10 +854,40 @@ dumb:
 		else
 		    newsym(x,y);
 		unblock_point(x,y);		/* vision */
+		if ((slev = Is_special(&u.uz)) && slev->flags.town)
+		  for(mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+		    if((mtmp->data == &mons[PM_WATCHMAN] ||
+			mtmp->data == &mons[PM_WATCH_CAPTAIN]) &&
+			couldsee(mtmp->mx, mtmp->my) &&
+			mtmp->mpeaceful) {
+			pline("%s yells:", Amonnam(mtmp));
+			verbalize("Halt, thief!  You're under arrest!");
+			(void) angry_guards(FALSE);
+			break;
+		    }
+		  }
 	} else {
 	    if (Blind) feel_location(x,y);	/* we know we hit it */
 	    exercise(A_STR, TRUE);
 	    pline("WHAMMM!!!");
+	    if ((slev = Is_special(&u.uz)) && slev->flags.town)
+	      for(mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	        if((mtmp->data == &mons[PM_WATCHMAN] ||
+		  mtmp->data == &mons[PM_WATCH_CAPTAIN]) &&
+	          couldsee(mtmp->mx, mtmp->my) &&
+	          mtmp->mpeaceful) {
+		
+		  pline("%s yells:", Amonnam(mtmp));
+		  if(levl[x][y].looted & D_WARNED) {
+			verbalize("Halt, vandal!  You're under arrest!");
+			(void) angry_guards(FALSE);
+		  } else {
+			verbalize("Hey, stop damaging that door!");
+			levl[x][y].looted |= D_WARNED;
+		  }
+		  break;
+	        }
+	      }
 	}
 	return(1);
 }
@@ -955,6 +992,7 @@ register xchar x, y, dlev;
 		if(obj == missile) continue;
 		/* number of objects in the pile */
 		oct += obj->quan;
+		if(obj == uball || obj == uchain) continue;
 		/* boulders can fall too, but rarely & never due to rocks */
 		if((isrock && obj->otyp == BOULDER) ||
 		   rn2(obj->otyp == BOULDER ? 30 : 3)) continue;
@@ -966,10 +1004,10 @@ register xchar x, y, dlev;
 				 index(u.urooms, *in_rooms(x, y, SHOPBASE))),
 				TRUE);
 		    /* set obj->no_charge to 0 */
-		    if(Is_container(obj))
-		        picked_container(obj); /* does the right thing */
-		    if(obj->otyp != GOLD_PIECE)
-		        obj->no_charge = 0;
+		    if (Has_contents(obj))
+			picked_container(obj);	/* does the right thing */
+		    if (obj->otyp != GOLD_PIECE)
+			obj->no_charge = 0;
 		}
 		obj->nobj = migrating_objs;
 		migrating_objs = obj;
@@ -988,10 +1026,16 @@ register xchar x, y, dlev;
 		pline("From the impact, %sother %s.",
 			dct == oct ? "the " : dct == 1L ? "an" : "", what);
 	    else
-		pline("%s adjacent %s %s",
-			oct == dct ? (dct > 1L ? "All the" : "The") :
-			    (dct == 1L ? "One of the" : "Some of the"),
-			what, gate_str(toloc));
+		if (oct == dct) {
+		    pline("%s adjacent %s %s",
+			    dct == 1L ? "The" : "All the",
+			    what, gate_str(toloc));
+		} else {
+		    pline("%s adjacent %s %s",
+			    dct == 1L ? "One of the" : "Some of the",
+			    dct == 1L ? "objects falls" : what,
+			    gate_str(toloc));
+		}
 	}
 
 	if(costly && shkp && price) {
@@ -1048,7 +1092,7 @@ register boolean shop_floor_obj;
 	drop_to(&cc, toloc);
 	if(!cc.y) return(FALSE);
 
-	container = Is_container(otmp);
+	container = Has_contents(otmp);
 
 	unpaid = (otmp->unpaid || (container && count_unpaid(otmp->cobj)));
 
@@ -1120,7 +1164,6 @@ obj_delivery()
 	register struct obj *otmp, *otmp0 = (struct obj *)0, *otmp2;
 
 	for(otmp = migrating_objs; otmp; otmp = otmp2) {
-
 	    otmp2 = otmp->nobj;
 
 	    if(otmp->ox == u.uz.dnum && otmp->oy == u.uz.dlevel) {
@@ -1166,6 +1209,9 @@ common:
 			    } /* else fall through */
 		    default:
 scatter:
+			    /* set dummy coordinates because there's no
+			       current position for rloco() to update */
+			    otmp->ox = otmp->oy = 0;
 			    rloco(otmp);
 			    break;
 		}
@@ -1213,6 +1259,9 @@ xchar x, y;
 {
 	register struct trap *ttmp = t_at(x, y);
 
+#ifdef MULDGN	/* this matches the player restriction in goto_level() */
+	if (on_level(&u.uz, &qstart_level) && !ok_to_quest()) return -1;
+#endif
 	if(ttmp && ttmp->ttyp == TRAPDOOR && ttmp->tseen) return 0;
 	if(xdnstair == x && ydnstair == y) return 1;
 	if(xdnladder == x && ydnladder == y) return 2;
