@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)dungeon.c	3.1	93/01/17	*/
+/*	SCCS Id: @(#)dungeon.c	3.1	93/06/26	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -8,7 +8,7 @@
 #ifdef OVL1
 
 #define	DUNGEON_FILE	"dungeon"
-#if defined(MICRO) && !defined(AMIGA)
+#if (defined(MICRO) && !defined(AMIGA)) || defined(THINK_C)
 # define RDMODE "rb"
 #else
 # define RDMODE "r"
@@ -182,9 +182,8 @@ const char	*s;
 	    if (!strcmp(dungeons[i].dname, s)) return i;
 
 	panic("Couldn't resolve dungeon number for name \"%s\".", s);
-#if defined(LINT) || defined(GCC_WARN)
+	/*NOT REACHED*/
 	return (xchar)0;
-#endif
 }
 
 s_level *
@@ -233,9 +232,8 @@ struct proto_dungeon *pd;
 		return(pdnum);
 
 	panic("parent_dnum: couldn't resolve branch.");
-#if defined(LINT) || defined(GCC_WARN)
+	/*NOT REACHED*/
 	return (xchar)0;
-#endif
 }
 
 /*
@@ -548,6 +546,44 @@ place_level(proto_index, pd)
     return FALSE;
 }
 
+
+struct level_map {
+	const char *lev_name;
+	d_level *lev_spec;
+} level_map[] = {
+	{ "air",	&air_level },
+	{ "asmodeus",	&asmodeus_level },
+	{ "astral",	&astral_level },
+	{ "baalz",	&baalzebub_level },
+	{ "bigroom",	&bigroom_level },
+	{ "castle",	&stronghold_level },
+	{ "earth",	&earth_level },
+	{ "fakewiz1",	&portal_level },
+	{ "fire",	&fire_level },
+	{ "juiblex",	&juiblex_level },
+#ifdef MULDGN
+	{ "knox",	&knox_level },
+#endif
+	{ "medusa",	&medusa_level },
+	{ "oracle",	&oracle_level },
+	{ "orcus",	&orcus_level },
+#ifdef REINCARNATION
+	{ "rogue",	&rogue_level },
+#endif
+	{ "sanctum",	&sanctum_level },
+	{ "valley",	&valley_level },
+	{ "water",	&water_level },
+	{ "wizard1",	&wiz1_level },
+	{ "wizard2",	&wiz2_level },
+	{ "wizard3",	&wiz3_level },
+#ifdef MULDGN
+	{ X_START,	&qstart_level },
+	{ X_LOCATE,	&qlocate_level },
+	{ X_GOAL,	&nemesis_level },
+#endif
+	{ "",		(d_level *)0 }
+};
+
 void
 init_dungeons()		/* initialize the "dungeon" structs */
 {
@@ -555,6 +591,7 @@ init_dungeons()		/* initialize the "dungeon" structs */
 	register int i, cl = 0, cb = 0;
 	register s_level *x;
 	struct proto_dungeon pd;
+	struct level_map *lev_map;
 
 	pd.n_levs = pd.n_brs = 0;
 
@@ -639,7 +676,7 @@ init_dungeons()		/* initialize the "dungeon" structs */
 
 	    if (i) {	/* set depth */
 		branch *br;
-		xchar from_depth;
+		schar from_depth;
 		boolean from_up;
 
 		br = add_branch(i, dungeons[i].entry_lev, &pd);
@@ -722,80 +759,35 @@ init_dungeons()		/* initialize the "dungeon" structs */
 	 * Find most of the special levels and dungeons so we can access their
 	 * locations quickly.
 	 */
-#ifdef REINCARNATION
-	if ((x = find_level("rogue")) != 0)
-	    assign_level(&rogue_level, &x->dlevel);
-#endif
-	if ((x = find_level("oracle")) != 0)
-	    assign_level(&oracle_level, &x->dlevel);
-	if ((x = find_level("bigroom")) != 0)
-	    assign_level(&bigroom_level, &x->dlevel);
-	if ((x = find_level("medusa")) != 0)
-	    assign_level(&medusa_level, &x->dlevel);
-	if ((x = find_level("castle")) != 0)
-	    assign_level(&stronghold_level, &x->dlevel);
-	if ((x = find_level("valley")) != 0)
-	    assign_level(&valley_level, &x->dlevel);
-	if ((x = find_level("wizard1")) != 0)
-	    assign_level(&wiz1_level, &x->dlevel);
-	if ((x = find_level("wizard2")) != 0)
-	    assign_level(&wiz2_level, &x->dlevel);
-	if ((x = find_level("wizard3")) != 0)
-	    assign_level(&wiz3_level, &x->dlevel);
-	if ((x = find_level("juiblex")) != 0)
-	    assign_level(&juiblex_level, &x->dlevel);
-	if ((x = find_level("orcus")) != 0)
-	    assign_level(&orcus_level, &x->dlevel);
-	if ((x = find_level("asmodeus")) != 0)
-	    assign_level(&asmodeus_level, &x->dlevel);
-	if ((x = find_level("baalz")) != 0)
-	    assign_level(&baalzebub_level, &x->dlevel);
-	if ((x = find_level("fakewiz1")) != 0)
-	    assign_level(&portal_level, &x->dlevel);
-	if ((x = find_level("sanctum")) != 0)
-	    assign_level(&sanctum_level, &x->dlevel);
-	if ((x = find_level("earth")) != 0)
-	    assign_level(&earth_level, &x->dlevel);
-	if ((x = find_level("water")) != 0)
-	    assign_level(&water_level, &x->dlevel);
-	if ((x = find_level("fire")) != 0)
-	    assign_level(&fire_level, &x->dlevel);
-	if ((x = find_level("air")) != 0)
-	    assign_level(&air_level, &x->dlevel);
-	if ((x = find_level("astral")) != 0)
-	    assign_level(&astral_level, &x->dlevel);
+	for (lev_map = level_map; lev_map->lev_name[0]; lev_map++) {
+		x = find_level(lev_map->lev_name);
+		if (x) {
+			assign_level(lev_map->lev_spec, &x->dlevel);
 #ifdef MULDGN
-	if ((x = find_level("knox")) != 0) {
-	    branch *br;
-	    assign_level(&knox_level, &x->dlevel);
-	    /*
-	     * Kludge to allow floating Knox entrance.  We specify a floating
-	     * entrance by the fact that it's entrance (end1) has a bogus dnum,
-	     * namely n_dgns.
-	     */
-	    for (br = branches; br; br = br->next)
-		if (on_level(&br->end2, &knox_level)) break;
+			if (!strncmp(lev_map->lev_name, "x-", 2)) {
+				/* This is where the name substitution on the
+				 * levels of the quest dungeon occur.
+				 */
+				x->proto[0] = pl_character[0];
+			} else if (lev_map->lev_spec == &knox_level) {
+				branch *br;
+				/*
+				 * Kludge to allow floating Knox entrance.  We
+				 * specify a floating entrance by the fact that
+				 * its entrance (end1) has a bogus dnum, namely
+				 * n_dgns.
+				 */
+				for (br = branches; br; br = br->next)
+				    if (on_level(&br->end2, &knox_level)) break;
 
-	    if (br) br->end1.dnum = n_dgns;
-	    /* adjust the branch's position on the list */
-	    insert_branch(br, TRUE);
-	}
-/*
- *	This is where the name substitution on the levels of the quest
- *	dungeon occur.
- */
-	if ((x = find_level(X_START)) != 0) {
-	    x->proto[0] = pl_character[0];
-	    assign_level(&qstart_level, &x->dlevel);
-	}
-	if ((x = find_level(X_LOCATE)) != 0) {
-	    x->proto[0] = pl_character[0];
-	    assign_level(&qlocate_level, &x->dlevel);
-	}
-	if ((x = find_level(X_GOAL)) != 0) {
-	    x->proto[0] = pl_character[0];
-	    assign_level(&nemesis_level, &x->dlevel);
+				if (br) br->end1.dnum = n_dgns;
+				/* adjust the branch's position on the list */
+				insert_branch(br, TRUE);
+			}
+#endif
+		}
 	}
+#ifdef MULDGN
 /*
  *	I hate hardwiring these names. :-(
  */
@@ -844,7 +836,7 @@ boolean noquest;
 	 */
 	register int i;
 	d_level tmp;
-	register xchar ret = 0;
+	register schar ret = 0;
 
 	for(i = 0; i < n_dgns; i++) {
 	    if((tmp.dlevel = dungeons[i].dunlev_ureached) == 0) continue;
@@ -853,7 +845,7 @@ boolean noquest;
 	    tmp.dnum = i;
 	    if(depth(&tmp) > ret) ret = depth(&tmp);
 	}
-	return(ret);
+	return((xchar) ret);
 }
 
 /* return a bookkeeping level number for purpose of comparisons and
@@ -862,7 +854,7 @@ xchar
 ledger_no(lev)
 d_level	*lev;
 {
-	return(lev->dlevel + dungeons[lev->dnum].ledger_start);
+	return((xchar)(lev->dlevel + dungeons[lev->dnum].ledger_start));
 }
 
 /*
@@ -887,12 +879,17 @@ xchar
 ledger_to_dnum(ledgerno)
 xchar	ledgerno;
 {
-	xchar	i;
+	register int i;
 
-	for(i = 0; i < n_dgns; i++)
-	    if(dungeons[i].ledger_start >= ledgerno) return(i-1);
+	/* find i such that (i->base + 1) <= ledgerno <= (i->base + i->count) */
+	for (i = 0; i < n_dgns; i++)
+	    if (dungeons[i].ledger_start < ledgerno &&
+		ledgerno <= dungeons[i].ledger_start + dungeons[i].num_dunlevs)
+		return (xchar)i;
 
-	return(MAXDUNGEON);
+	panic("level number out of range [ledger_to_dnum(%d)]", (int)ledgerno);
+	/*NOT REACHED*/
+	return (xchar)0;
 }
 
 /* return the level of the dungeon this ledgerno exists in */
@@ -900,7 +897,7 @@ xchar
 ledger_to_dlev(ledgerno)
 xchar	ledgerno;
 {
-	return(ledgerno - dungeons[ledger_to_dnum(ledgerno)].ledger_start);
+	return((xchar)(ledgerno - dungeons[ledger_to_dnum(ledgerno)].ledger_start));
 }
 
 #endif /* OVL1 */
@@ -908,18 +905,18 @@ xchar	ledgerno;
 
 /* returns the depth of a level, in floors below the surface	*/
 /* (note levels in different dungeons can have the same depth).	*/
-xchar
+schar
 depth(lev)
 d_level	*lev;
 {
-	return dungeons[lev->dnum].depth_start + lev->dlevel - 1;
+	return((schar)( dungeons[lev->dnum].depth_start + lev->dlevel - 1));
 }
 
 boolean
 on_level(lev1, lev2)	/* are "lev1" and "lev2" actually the same? */
 d_level	*lev1, *lev2;
 {
-	return((lev1->dnum == lev2->dnum) && (lev1->dlevel == lev2->dlevel));
+	return((boolean)((lev1->dnum == lev2->dnum) && (lev1->dlevel == lev2->dlevel)));
 }
 
 #endif /* OVL0 */
@@ -1040,26 +1037,26 @@ boolean
 On_stairs(x, y)
 xchar x, y;
 {
-	return((x == xupstair && y == yupstair) ||
+	return((boolean)((x == xupstair && y == yupstair) ||
 	       (x == xdnstair && y == ydnstair) ||
 	       (x == xdnladder && y == ydnladder) ||
 	       (x == xupladder && y == yupladder) ||
-	       (x == sstairs.sx && y == sstairs.sy));
+	       (x == sstairs.sx && y == sstairs.sy)));
 }
 
 boolean
 Is_botlevel(lev)
 d_level *lev;
 {
-	return lev->dlevel == dungeons[lev->dnum].num_dunlevs;
+	return((boolean)(lev->dlevel == dungeons[lev->dnum].num_dunlevs));
 }
 
 boolean
 Can_dig_down(lev)
 d_level *lev;
 {
-	return !level.flags.hardfloor
-	    && !Is_botlevel(lev) && !Invocation_lev(lev);
+	return((boolean)(!level.flags.hardfloor
+	    && !Is_botlevel(lev) && !Invocation_lev(lev)));
 }
 
 /*
@@ -1071,7 +1068,7 @@ boolean
 Can_fall_thru(lev)
 d_level *lev;
 {
-	return Can_dig_down(lev) || Is_stronghold(lev);
+	return((boolean)(Can_dig_down(lev) || Is_stronghold(lev)));
 }
 
 /*
@@ -1084,10 +1081,10 @@ boolean
 Can_rise_up(lev)
 d_level *lev;
 {
-    return !In_endgame(lev) &&
+    return((boolean)(!In_endgame(lev) &&
 	(lev->dlevel > 1 ||
 	 (dungeons[lev->dnum].entry_lev == 1 && ledger_no(lev) != 1 &&
-	  sstairs.sx && sstairs.up));
+	  sstairs.sx && sstairs.up))));
 }
 
 /*
@@ -1110,7 +1107,7 @@ int levnum;
 	xchar dgn = u.uz.dnum;
 
 	if (levnum <= 0) {
-	    impossible("get_level:  levnum = %d\n", levnum);
+	    /* can only currently happen in endgame */
 	    levnum = u.uz.dlevel;
 	} else if (levnum > dungeons[dgn].depth_start
 			    + dungeons[dgn].num_dunlevs - 1) {
@@ -1157,7 +1154,7 @@ boolean
 In_quest(lev)	/* are you in the quest dungeon? */
 d_level *lev;
 {
-	return(lev->dnum == quest_dnum);
+	return((boolean)(lev->dnum == quest_dnum));
 }
 #endif /* MULDGN */
 
@@ -1169,7 +1166,7 @@ boolean
 In_mines(lev)	/* are you in the mines dungeon? */
 d_level	*lev;
 {
-	return(lev->dnum == mines_dnum);
+	return((boolean)(lev->dnum == mines_dnum));
 }
 
 /*
@@ -1213,7 +1210,7 @@ at_dgn_entrance(s)
     branch *br;
 
     br = dungeon_branch(s);
-    return on_level(&u.uz, &br->end1) ? TRUE : FALSE;
+    return((boolean)(on_level(&u.uz, &br->end1) ? TRUE : FALSE));
 }
 #endif /* MULDGN */
 
@@ -1221,7 +1218,7 @@ boolean
 In_tower(lev)	/* are you inside the tower? */
 d_level	*lev;
 {
-	return(lev->dnum == tower_dnum);
+	return((boolean)(lev->dnum == tower_dnum));
 }
 
 #endif /* OVL1 */
@@ -1231,7 +1228,7 @@ boolean
 In_hell(lev)	/* are you in one of the Hell levels? */
 d_level	*lev;
 {
-	return(dungeons[lev->dnum].flags.hellish);
+	return((boolean)(dungeons[lev->dnum].flags.hellish));
 }
 
 #endif /* OVL0 */
@@ -1297,8 +1294,8 @@ boolean
 Invocation_lev(lev)
 d_level *lev;
 {
-	return(In_hell(lev) &&
-		lev->dlevel == (dungeons[lev->dnum].num_dunlevs - 1));
+	return((boolean)(In_hell(lev) &&
+		lev->dlevel == (dungeons[lev->dnum].num_dunlevs - 1)));
 }
 
 /* use instead of depth() wherever a degree of difficulty is made
@@ -1308,12 +1305,12 @@ xchar
 level_difficulty()
 {
 	if (In_endgame(&u.uz))
-		return(depth(&sanctum_level) + u.ulevel/2);
+		return((xchar)(depth(&sanctum_level) + u.ulevel/2));
 	else
 		if (u.uhave.amulet)
 			return(deepest_lev_reached(FALSE));
 		else
-			return(depth(&u.uz));
+			return((xchar) depth(&u.uz));
 }
 
 
