@@ -1,8 +1,8 @@
-/*	SCCS Id: @(#)eat.c	3.1	92/12/06	*/
+/*	SCCS Id: @(#)eat.c	3.1	93/05/19	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
-#include	"hack.h"
+#include "hack.h"
 /*#define DEBUG		/* uncomment to enable new eat code debugging */
 
 #ifdef DEBUG
@@ -74,18 +74,18 @@ const char *hu_stat[] = {
 
 #ifndef OVLB
 
-STATIC_DCL const char NEARDATA comestibles[];
+STATIC_DCL NEARDATA const char comestibles[];
 #ifdef POLYSELF
-STATIC_OVL const char NEARDATA allobj[];
+STATIC_OVL NEARDATA const char allobj[];
 #endif /* POLYSELF */
 
 #else
 
-STATIC_OVL const char NEARDATA comestibles[] = { FOOD_CLASS, 0 };
+STATIC_OVL NEARDATA const char comestibles[] = { FOOD_CLASS, 0 };
 
 #ifdef POLYSELF
 /* Gold must come first for getobj(). */
-STATIC_OVL const char NEARDATA allobj[] = {
+STATIC_OVL NEARDATA const char allobj[] = {
 	GOLD_CLASS, WEAPON_CLASS, ARMOR_CLASS, POTION_CLASS, SCROLL_CLASS,
 	WAND_CLASS, RING_CLASS, AMULET_CLASS, FOOD_CLASS, TOOL_CLASS,
 	GEM_CLASS, ROCK_CLASS, BALL_CLASS, CHAIN_CLASS, SPBOOK_CLASS, 0 };
@@ -103,14 +103,15 @@ register struct obj *obj;
 		return TRUE;
 	if (u.umonnum == PM_GELATINOUS_CUBE && is_organic(obj))
 		return TRUE;
-	return !!index(comestibles, obj->oclass);
+	return((boolean)(!!index(comestibles, obj->oclass)));
 }
 # endif /* POLYSELF */
 #endif /* OVL1 */
 #ifdef OVLB
 
 void
-init_uhunger(){
+init_uhunger()
+{
 	u.uhunger = 900;
 	u.uhs = NOT_HUNGRY;
 }
@@ -126,12 +127,12 @@ static const struct { const char *txt; int nut; } tintxts[] = {
 };
 #define	TTSZ	SIZE(tintxts)
 
-static struct {
+static NEARDATA struct {
 	struct	obj *tin;
 	int	usedtime, reqtime;
-} NEARDATA tin;
+} tin;
 
-static struct {
+static NEARDATA struct {
 	struct	obj *piece;	/* the thing being eaten, or last thing that
 				 * was partially eaten, unless that thing was
 				 * a tin, which uses the tin structure above */
@@ -142,11 +143,12 @@ static struct {
 	Bitfield(fullwarn,1);	/* have warned about being full */
 	Bitfield(eating,1);	/* victual currently being eaten */
 	Bitfield(doreset,1);	/* stop eating at end of turn */
-} NEARDATA victual;
+} victual;
 
 STATIC_PTR
 int
-eatmdone() {		/* called after mimicing is over */
+eatmdone()		/* called after mimicing is over */
+{
 	u.usym =
 #ifdef POLYSELF
 		u.mtimedone ? uasmon->mlet :
@@ -172,7 +174,9 @@ choke(food)	/* To a full belly all food is bad. (It.) */
 	if (pl_character[0] == 'K' && u.ualign.type == A_LAWFUL)
 		u.ualign.record--;	/* gluttony is unchivalrous */
 
-	if (!rn2(20)) {
+	exercise(A_CON, FALSE);
+
+	if (!rn2(20) || Breathless) {
 		You("stuff yourself and then vomit voluminously.");
 		morehungry(1000);	/* you just got *very* sick! */
 		vomit();
@@ -206,7 +210,8 @@ choke(food)	/* To a full belly all food is bad. (It.) */
 }
 
 static void
-recalc_wt() {	/* modify object wt. depending on time spent consuming it */
+recalc_wt()	/* modify object wt. depending on time spent consuming it */
+{
 	register struct obj *piece = victual.piece;
 
 #ifdef DEBUG
@@ -223,8 +228,8 @@ recalc_wt() {	/* modify object wt. depending on time spent consuming it */
 }
 
 void
-reset_eat() {		/* called when eating interrupted by an event */
-
+reset_eat()		/* called when eating interrupted by an event */
+{
     /* we only set a flag here - the actual reset process is done after
      * the round is spent eating.
      */
@@ -276,12 +281,13 @@ register struct obj *otmp;
 
 	if (!otmp->oeaten) {
 	    if(((!carried(otmp) && costly_spot(otmp->ox, otmp->oy) &&
-		 saleable(rooms[*u.ushops-ROOMOFFSET].rtype-SHOPBASE, otmp))
+		 !otmp->no_charge)
 		 || otmp->unpaid) &&
 		 (otmp->otyp == CORPSE || objects[otmp->otyp].oc_delay > 1)) {
 		/* create a dummy duplicate to put on bill */
-		You("bite it, you bought it!");
+		verbalize("You bit it, you bought it!");
 		bill_dummy_object(otmp);
+		otmp->no_charge = 1;	/* you now own this */
 	    }
 	    otmp->oeaten = (otmp->otyp == CORPSE ?
 				(int)mons[otmp->corpsenm].cnutrit :
@@ -331,7 +337,8 @@ do_reset_eat() {
 
 STATIC_PTR
 int
-eatfood() {		/* called each move during eating process */
+eatfood()		/* called each move during eating process */
+{
 	if(!carried(victual.piece) && !obj_here(victual.piece, u.ux, u.uy)) {
 		/* maybe it was stolen? */
 		do_reset_eat();
@@ -339,7 +346,7 @@ eatfood() {		/* called each move during eating process */
 	}
 	if(!victual.eating) return(0);
 
-	if(++victual.usedtime < victual.reqtime) {
+	if(++victual.usedtime <= victual.reqtime) {
 	    if(bite()) return(0);
 	    return(1);	/* still busy */
 	} else {	/* done */
@@ -377,6 +384,11 @@ cprefx(pm)
 register int pm;
 {
 	if ((pl_character[0]=='E') ? is_elf(&mons[pm]) : is_human(&mons[pm])) {
+#ifdef POLYSELF
+		if (uasmon != &playermon) {
+			You("have a bad feeling deep inside.");
+		}
+#endif /* POLYSELF */
 		You("cannibal!  You will regret this!");
 		Aggravate_monster |= FROMOUTSIDE;
 	}
@@ -589,7 +601,8 @@ register struct permonst *ptr;
 		debugpline("Trying to give fire resistance");
 #endif
 		if(!(HFire_resistance & FROMOUTSIDE)) {
-			You("feel a momentary chill.");
+			You(Hallucination ? "be chillin'." :
+			    "feel a momentary chill.");
 			HFire_resistance |= FROMOUTSIDE;
 		}
 		break;
@@ -616,7 +629,9 @@ register struct permonst *ptr;
 		debugpline("Trying to give disintegration resistance");
 #endif
 		if(!(HDisint_resistance & FROMOUTSIDE)) {
-			You("feel very firm.");
+			You(Hallucination ?
+			    "feel totally together, man." :
+			    "feel very firm.");
 			HDisint_resistance |= FROMOUTSIDE;
 		}
 		break;
@@ -625,7 +640,10 @@ register struct permonst *ptr;
 		debugpline("Trying to give shock resistance");
 #endif
 		if(!(HShock_resistance & FROMOUTSIDE)) {
-			Your("health currently feels amplified!");
+			if (Hallucination)
+				You("feel grounded in reality.");
+			else
+				Your("health currently feels amplified!");
 			HShock_resistance |= FROMOUTSIDE;
 		}
 		break;
@@ -643,7 +661,8 @@ register struct permonst *ptr;
 		debugpline("Trying to give teleport");
 #endif
 		if(!(HTeleportation & FROMOUTSIDE)) {
-			You("feel very jumpy.");
+			You(Hallucination ? "feel diffuse." :
+			    "feel very jumpy.");
 			HTeleportation |= FROMOUTSIDE;
 		}
 		break;
@@ -652,7 +671,9 @@ register struct permonst *ptr;
 		debugpline("Trying to give teleport control");
 #endif
 		if(!(HTeleport_control & FROMOUTSIDE)) {
-			You("feel in control of yourself.");
+			You(Hallucination ?
+			    "feel centered in your personal space." :
+			    "feel in control of yourself.");
 			HTeleport_control |= FROMOUTSIDE;
 		}
 		break;
@@ -661,8 +682,9 @@ register struct permonst *ptr;
 		debugpline("Trying to give telepathy");
 #endif
 		if(!(HTelepat & FROMOUTSIDE)) {
-			You("feel a %s mental acuity.",
-				Hallucination ? "normal" : "strange");
+			You(Hallucination ?
+			    "feel in touch with the cosmos." :
+			    "feel a strange mental acuity.");
 			HTelepat |= FROMOUTSIDE;
 			/* If blind, make sure monsters show up. */
 			if (Blind) see_monsters();
@@ -732,13 +754,13 @@ register int pm;
 	    case PM_SMALL_MIMIC:
 		tmp += 20;
 		if(u.usym == S_HUMAN) {
-		    You("cannot resist the temptation to mimic a pile of gold.");
+		    You("can't resist the temptation to mimic a pile of gold.");
 		    nomul(-tmp);
 		    afternmv = eatmdone;
 		    if (pl_character[0]=='E')
-			nomovemsg = "You now again prefer mimicking an elf.";
+			nomovemsg = "You now prefer mimicking an elf again.";
 		    else
-			nomovemsg = "You now again prefer mimicking a human.";
+			nomovemsg = "You now prefer mimicking a human again.";
 		    u.usym = 0; /* hack! no monster sym 0; use for gold */
 		    newsym(u.ux,u.uy);
 		}
@@ -855,8 +877,9 @@ opentin()		/* called during each move whilst opening a tin */
 	}
 	if(tin.usedtime < tin.reqtime)
 		return(1);		/* still busy */
-	if(tin.tin->cursed && tin.tin->spe != -1 && !rn2(8)) {
-		b_trapped("tin");
+	if(tin.tin->otrapped || 
+	   (tin.tin->cursed && tin.tin->spe != -1 && !rn2(8))) {
+		b_trapped("tin", 0);
 		goto use_me;
 	}
 	You("succeed in opening the tin.");
@@ -869,6 +892,8 @@ opentin()		/* called during each move whilst opening a tin */
 	    r = tin.tin->cursed ? 4 :		/* Always rotten if cursed */
 		    (tin.tin->spe == -1) ? 5 :	/* "homemade" if player made */
 			rn2(TTSZ-1);		/* else take your pick */
+	    if (tin.tin->spe == -1 && !tin.tin->blessed && !rn2(7))
+		r = 4;				/* some homemade tins go bad */
 	    pline("It smells like %s.", makeplural(
 		  Hallucination ? rndmonnam() : mons[tin.tin->corpsenm].mname));
 	    if (yn("Eat it?") == 'n') {
@@ -876,6 +901,10 @@ opentin()		/* called during each move whilst opening a tin */
 		if (flags.verbose) You("discard the open tin.");
 		goto use_me;
 	    }
+	    /* in case stop_occupation() was called on previous meal */
+	    victual.piece = (struct obj *)0;
+	    victual.fullwarn = victual.eating = victual.doreset = FALSE;
+
 	    You("consume %s %s.", tintxts[r].txt,
 			mons[tin.tin->corpsenm].mname);
 	    tin.tin->dknown = tin.tin->known = TRUE;
@@ -886,9 +915,10 @@ opentin()		/* called during each move whilst opening a tin */
 	    else lesshungry(tintxts[r].nut);
 
 	    if(r == 0) {			/* Deep Fried */
-		Glib = rnd(15);
+	        /* Assume !Glib, because you can't open tins when Glib. */
+		Glib += rnd(15);
 		pline("Eating deep fried food made your %s very slippery.",
-			makeplural(body_part(FINGER)));
+		      makeplural(body_part(FINGER)));
 	    }
 	} else {
 	    if (tin.tin->cursed)
@@ -926,8 +956,11 @@ start_tin(otmp)		/* called when starting to open a tin */
 
 #ifdef POLYSELF
 	if (metallivorous(uasmon)) {
-		You("bite right into the metal tin....");
+		You("bite right into the metal tin...");
 		tmp = 1;
+	} else if (nolimbs(uasmon)) {
+		You("cannot handle the tin properly to open it.");
+		return;
 	} else
 #endif
 	if (otmp->blessed) {
@@ -958,7 +991,8 @@ start_tin(otmp)		/* called when starting to open a tin */
 no_opener:
 		pline("It is not so easy to open this tin.");
 		if(Glib) {
-			pline("The tin slips out of your hands.");
+			pline("The tin slips from your %s.",
+			      makeplural(body_part(FINGER)));
 			if(otmp->quan > 1L) {
 				register struct obj *obj;
 				obj = splitobj(otmp, 1L);
@@ -978,7 +1012,8 @@ no_opener:
 }
 
 int
-Hear_again() {		/* called when waking up after fainting */
+Hear_again()		/* called when waking up after fainting */
+{
 	flags.soundok = 1;
 	return 0;
 }
@@ -1004,10 +1039,15 @@ rottenfood()
 		pline("Everything suddenly goes dark.");
 		make_blinded((long)d(2,10),FALSE);
 	} else if(!rn2(3)) {
-		if(Blind)
-		  pline("The world spins and you slap against the floor.");
+		const char *what, *where;
+		if (!Blind)
+		    what = "goes",  where = "dark";
+		else if (Levitation || Is_airlevel(&u.uz) ||
+			 Is_waterlevel(&u.uz))
+		    what = "you lose control of",  where = "yourself";
 		else
-		  pline("The world spins and goes dark.");
+		    what = "you slap against the",  where = surface(u.ux,u.uy);
+		pline("The world spins and %s %s.", what, where);
 		flags.soundok = 0;
 		nomul(-rnd(10));
 		nomovemsg = "You are conscious again.";
@@ -1081,10 +1121,11 @@ eatcorpse(otmp)		/* called when a corpse is selected as food */
 	}
 	if(!tp && otmp->corpsenm != PM_LIZARD && (otmp->orotten || !rn2(7))) {
 #ifdef POLYSELF
-	    if(rottenfood(otmp)) {
+	    if(rottenfood(otmp))
 #else
-	    if(rottenfood()) {
+	    if(rottenfood())
 #endif
+	    {
 		otmp->orotten = TRUE;
 		(void)touchfood(otmp);
 		return(1);
@@ -1183,7 +1224,9 @@ fprefx(otmp)		/* called on "first bite" of (non-corpse) food */
 #ifdef TUTTI_FRUTTI
 		if (otmp->otyp==SLIME_MOLD && !otmp->cursed
 			&& otmp->spe == current_fruit)
-		    pline("My, that was a yummy %s!", singular(otmp, xname));
+		    pline("My, that was a %s %s!",
+			  Hallucination ? "primo" : "yummy",
+			  singular(otmp, xname));
 		else
 #endif
 #ifdef UNIX
@@ -1200,10 +1243,10 @@ fprefx(otmp)		/* called on "first bite" of (non-corpse) food */
 		    }
 		} else
 #endif
-		    pline("This %s is %s!", singular(otmp, xname),
-		      otmp->cursed ? (Hallucination ? "grody" : "terrible"):
-		      Hallucination ? "gnarly" :
-			(otmp->otyp==CRAM_RATION ? "bland" : "delicious"));
+		    pline("This %s is %s", singular(otmp, xname),
+		      otmp->cursed ? (Hallucination ? "grody!" : "terrible!") :
+		      otmp->otyp == CRAM_RATION ? "bland." :
+		      Hallucination ? "gnarly!" : "delicious!");
 		break;
 	}
 }
@@ -1403,7 +1446,7 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	if (!(otmp = floorfood("eat", 0))) return 0;
 	if (check_capacity(NULL)) return 0;
 #ifdef POLYSELF
-	/* We have to make non-foods take no time to eat, unless we want to
+	/* We have to make non-foods take 1 move to eat, unless we want to
 	 * do ridiculous amounts of coding to deal with partly eaten plate
 	 * mails, players who polymorph back to human in the middle of their
 	 * metallic meal, etc....
@@ -1423,6 +1466,9 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	    if (otmp->oclass == GOLD_CLASS)
 		basenutrit = ((otmp->quan > 200000L) ? 2000
 			: (int)(otmp->quan/100L));
+	    else if(otmp->oclass == BALL_CLASS || otmp->oclass == CHAIN_CLASS)
+		basenutrit = weight(otmp);
+	    /* oc_nutrition is usually weight anyway */
 	    else basenutrit = objects[otmp->otyp].oc_nutrition;
 	    victual.nmod = basenutrit;
 	    victual.eating = TRUE; /* needed for lesshungry() */
@@ -1488,10 +1534,11 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 		(otmp->orotten || !rn2(7))))) {
 
 #ifdef POLYSELF
-		if(rottenfood(otmp)) {
+		if(rottenfood(otmp))
 #else
-		if(rottenfood()) {
+		if(rottenfood())
 #endif
+		{
 		    otmp->orotten = TRUE;
 		    return(1);
 		}
@@ -1578,7 +1625,7 @@ gethungry()	/* as time goes by - called by moveloop() and domove() */
 	    /* Conflict uses up food too */
 	    if ((Conflict & (~W_ARTI))) u.uhunger--;
 	    /* +0 charged rings don't do anything, so don't affect hunger */
-	    switch (moves % 20) {	/* note: use even cases only */
+	    switch ((int)(moves % 20)) {	/* note: use even cases only */
 	     case  4: if (uleft &&
 			  (uleft->spe || !objects[uleft->otyp].oc_charged))
 			    u.uhunger--;
@@ -1642,7 +1689,7 @@ register int num;
 		    victual.fullwarn = TRUE;
 		    if (victual.canchoke &&
 			/* a one-gulp food will not survive a stop */
-				objects[victual.piece->otyp].oc_delay > 1) {
+				victual.reqtime > 1) {
 			if(yn("Stop eating?") == 'y')
 			{
 				reset_eat();
@@ -1658,7 +1705,8 @@ register int num;
 
 STATIC_PTR
 int
-unfaint() {
+unfaint()
+{
 	(void) Hear_again();
 	if(u.uhs > FAINTING)
 		u.uhs = FAINTING;
@@ -1671,18 +1719,21 @@ unfaint() {
 #ifdef OVL0
 
 boolean
-is_fainted() {
-	return(u.uhs == FAINTED);
+is_fainted()
+{
+	return((boolean)(u.uhs == FAINTED));
 }
 
 void
-reset_faint() {	/* call when a faint must be prematurely terminated */
+reset_faint()	/* call when a faint must be prematurely terminated */
+{
 	if(is_fainted()) nomul(0);
 }
 
 #if 0
 void
-sync_hunger() {
+sync_hunger()
+{
 
 	if(is_fainted()) {
 
@@ -1784,9 +1835,9 @@ newuhs(incr)		/* compute and comment on your (new?) hunger status */
  * in inventory.
  */
 struct obj *
-floorfood(verb,corpseonly)	/* get food from floor or pack */
+floorfood(verb,corpsecheck)	/* get food from floor or pack */
 	const char *verb;
-	boolean corpseonly;
+	int corpsecheck; /* 0, no check, 1, corpses, 2, tinnable corpses */
 {
 	register struct obj *otmp;
 	char qbuf[QBUFSZ];
@@ -1811,7 +1862,8 @@ floorfood(verb,corpseonly)	/* get food from floor or pack */
 	if (!(Levitation && !Is_airlevel(&u.uz)  && !Is_waterlevel(&u.uz))
 	    && !u.uswallow) {
 	    for(otmp = level.objects[u.ux][u.uy]; otmp; otmp = otmp->nexthere) {
-		if(corpseonly ? otmp->otyp==CORPSE :
+		if(corpsecheck ?
+		(otmp->otyp==CORPSE && (corpsecheck == 1 || tinnable(otmp))) :
 #ifdef POLYSELF
 		    feeding ? (otmp->oclass != GOLD_CLASS && is_edible(otmp)) :
 #endif
@@ -1842,7 +1894,8 @@ floorfood(verb,corpseonly)	/* get food from floor or pack */
 /* added nomul (MRS) - it makes sense, you're too busy being sick! */
 /* TO DO: regurgitate swallowed monsters when poly'd */
 void
-vomit() {		/* A good idea from David Neves */
+vomit()		/* A good idea from David Neves */
+{
 	make_sick(0L,TRUE);
 	nomul(-2);
 }
