@@ -32,11 +32,16 @@ extern int errno;
 char bones[FILENAME];		/* pathname of bones files */
 char lock[FILENAME];		/* pathname of level files */
 #else
-static char bones[] = "bonesnn.xxxx";
-# ifdef VMS
-char lock[PL_NSIZ+17] = "1lock"; /* long enough for _uid+name+.99;1 */
+# ifdef DEBIAN
+static char bones[] = "/var/lib/games/nethack/bonesnn.xxxx";
+char lock[256] = "/var/lib/games/nethack/1lock";
 # else
+static char bones[] = "bonesnn.xxxx";
+#   ifdef VMS
+char lock[PL_NSIZ+17] = "1lock"; /* long enough for _uid+name+.99;1 */
+#   else
 char lock[PL_NSIZ+14] = "1lock"; /* long enough for uid+name+.99 */
+#   endif
 # endif
 #endif
 
@@ -341,8 +346,13 @@ set_savefile_name()
 	}
 	Strcat(SAVEF, ".sav");
 # else
+#   ifdef DEBIAN
+	Sprintf(SAVEF, "/var/lib/games/nethack/save/%d%s", (int)getuid(), plname);
+	regularize(SAVEF+28);
+#   else  /* NOT DEBIAN */
 	Sprintf(SAVEF, "save/%d%s", (int)getuid(), plname);
 	regularize(SAVEF+5);	/* avoid . or / in name */
+#   endif /* NOT DEBIAN */
 # endif	/* MICRO */
 #endif	/* VMS */
 }
