@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)hack.c	3.1	92/12/04	*/
+/*	SCCS Id: @(#)hack.c	3.1	93/06/15	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -115,6 +115,10 @@ moverock()
 		case SPIKED_PIT:
 		case PIT:
 		    freeobj(otmp);
+		    /* vision kludge to get messages right;
+		       the pit will temporarily be seen even
+		       if this is one among multiple boulders */
+		    if (!Blind) viz_array[ry][rx] |= IN_SIGHT;
 		    if (!flooreffects(otmp, rx, ry, "fall")) {
 			place_object(otmp, rx, ry);
 			otmp->nobj = fobj;
@@ -122,11 +126,11 @@ moverock()
 		    }
 		    continue;
 		case TRAPDOOR:
-		    pline("%s falls into and plugs a hole in the ground!",
-			  The(xname(otmp)));
+		    pline("%s falls into and plugs a hole in the %s!",
+			  The(xname(otmp)), surface(rx, ry));
 		    deltrap(ttmp);
 		    delobj(otmp);
-		    delallobj(rx, ry);
+		    bury_objs(rx, ry);
 		    if (cansee(rx,ry)) newsym(rx,ry);
 		    continue;
 		case LEVEL_TELEP:
@@ -162,7 +166,7 @@ moverock()
 		long lastmovetime;
 		lastmovetime = 0;
 #else
-		static long NEARDATA lastmovetime;
+		static NEARDATA long lastmovetime;
 #endif
 		/* note: this var contains garbage initially and
 		   after a restore */
@@ -224,12 +228,15 @@ still_chewing(x,y)
 {
     struct rm *lev      = &(levl[x][y]);
     struct obj *boulder = sobj_at(BOULDER,x,y);
+    register boolean shopedge = *in_rooms(x, y, SHOPBASE);
+    register const char *digtxt = (const char*) 0, *dmgtxt = (const char*) 0;
 
     if (dig_pos.x != x || dig_pos.y != y ||
 				!on_level(&dig_level, &u.uz) || dig_down) {
-	if (!boulder && (lev->diggable & W_NONDIGGABLE))
+	if (!boulder && (lev->diggable & W_NONDIGGABLE)) {
 	    You("hurt your teeth on the hard stone.");
-	else {
+	    nomul(0);
+	} else {
 	    dig_down = FALSE;
 	    dig_pos.x = x;
 	    dig_pos.y = y;
@@ -250,8 +257,8 @@ still_chewing(x,y)
     }
 
     if (boulder) {
-	You("eat the boulder.");	/* yum */
 	delobj(boulder);		/* boulder goes bye-bye */
+	You("eat the boulder.");	/* yum */
 
 	/*
 	 *  The location could still block because of
@@ -267,7 +274,11 @@ still_chewing(x,y)
 	}
 	
     } else if (IS_WALL(lev->typ)) {
-	You("chew a hole in the wall.");
+	if(shopedge) {
+	    add_damage(x, y, 10L * ACURRSTR);
+	    dmgtxt = "damage";
+	}
+	digtxt = "chew a hole in the wall.";
 	if (level.flags.is_maze_lev) {
 	    lev->typ = ROOM;
 	} else if (level.flags.is_cavernous_lev) {
@@ -278,30 +289,37 @@ still_chewing(x,y)
 	}
     } else if (lev->typ == SDOOR) {
 	if (lev->doormask & D_TRAPPED) {
-	    b_trapped("secret door");
 	    lev->doormask = D_NODOOR;
+	    b_trapped("secret door", 0);
 	} else {
-	    You("chew through the secret door.");
+	    digtxt = "chew through the secret door.";
 	    lev->doormask = D_BROKEN;
 	}
 	lev->typ = DOOR;
 
     } else if (IS_DOOR(lev->typ)) {
+	if(shopedge) {
+	    add_damage(x, y, 400L);
+	    dmgtxt = "break";
+	}
 	if (lev->doormask & D_TRAPPED) {
-	    b_trapped("door");
 	    lev->doormask = D_NODOOR;
+	    b_trapped("door", 0);
 	} else {
-	    You("chew through the door.");
+	    digtxt = "chew through the door.";
 	    lev->doormask = D_BROKEN;
 	}
 
     } else { /* STONE or SCORR */
-	You("chew a passage through the rock.");
+	digtxt = "chew a passage through the rock.";
 	lev->typ = CORR;
     }
 
     unblock_point(x, y);	/* vision */
     newsym(x, y);
+    if (digtxt) You(digtxt);	/* after newsym */
+    if (dmgtxt)
+	pay_for_damage(dmgtxt);
     dig_level.dnum = 0;
     dig_level.dlevel = -1;
     return 0;
@@ -376,7 +394,14 @@ may_dig(x,y)
 register xchar x,y;
 /* intended to be called only on ROCKs */
 {
-return (!(IS_STWALL(levl[x][y].typ) && (levl[x][y].diggable & W_NONDIGGABLE)));
+return((boolean)(!(IS_STWALL(levl[x][y].typ) && (levl[x][y].diggable & W_NONDIGGABLE))));
+}
+
+boolean
+may_passwall(x,y)
+register xchar x,y;
+{
+return((boolean)(!(IS_STWALL(levl[x][y].typ) && (levl[x][y].diggable & W_NONPASSWALL))));
 }
 
 #endif /* OVLB */
@@ -386,19 +411,19 @@ STATIC_OVL boolean
 bad_rock(x,y)
 register xchar x,y;
 {
-	return(IS_ROCK(levl[x][y].typ)
+	return((boolean)(IS_ROCK(levl[x][y].typ)
 #ifdef POLYSELF
-		    && !passes_walls(uasmon)
+		    && !(passes_walls(uasmon) && may_passwall(x,y))
 		    && (!tunnels(uasmon) || needspick(uasmon) || !may_dig(x,y))
 #endif
-	);
+	));
 }
 
 boolean
 invocation_pos(x, y)
 xchar x, y;
 {
-        return(Invocation_lev(&u.uz) && x == inv_pos.x && y == inv_pos.y);
+        return((boolean)(Invocation_lev(&u.uz) && x == inv_pos.x && y == inv_pos.y));
 }
 
 #endif /* OVL1 */
@@ -492,6 +517,16 @@ domove()
 				y = u.uy + u.dy;
 			} while(!isok(x, y) || bad_rock(x, y));
 		}
+		/* turbulence might alter your actual destination */
+		if (u.uinwater) {
+			water_friction();
+			if (!u.dx && !u.dy) {
+				nomul(0);
+				return;
+			}
+			x = u.ux + u.dx;
+			y = u.uy + u.dy;
+		}
 		if(!isok(x, y)) {
 			nomul(0);
 			return;
@@ -643,7 +678,7 @@ domove()
 	if (IS_ROCK(tmpr->typ)) {
 	    if (Blind) feel_location(x,y);
 #ifdef POLYSELF
-	    if (passes_walls(uasmon)) {
+	    if (passes_walls(uasmon) && may_passwall(x,y)) {
 		;	/* do nothing */
 	    } else if (tunnels(uasmon) && !needspick(uasmon)) {
 		/* Eat the rock. */
@@ -762,7 +797,6 @@ domove()
 
 	/* now move the hero */
 	mtmp = m_at(x, y);
-	if (u.uinwater) water_friction();
 	u.ux += u.dx;
 	u.uy += u.dy;
 	/* if safepet at destination then move the pet to the hero's
@@ -781,11 +815,21 @@ domove()
 		int swap_result;
 
 		/* if trapped, there's a chance the pet goes wild */
-		if (mtmp->mtrapped && !rn2(4)) {
-		    pline ("%s suddenly goes wild!",
-			   mtmp->mnamelth ? NAME(mtmp) : Monnam(mtmp));
-		    mtmp->mtame = mtmp->mpeaceful = mtmp->msleep = 0;
-		}
+		if (mtmp->mtrapped) {
+		    if (!rn2(mtmp->mtame)) {
+			mtmp->mtame = mtmp->mpeaceful = mtmp->msleep = 0;
+#ifndef SOUNDS
+			pline ("%s suddenly goes wild!",
+			       mtmp->mnamelth ? NAME(mtmp) : Monnam(mtmp));
+#else
+		        growl(mtmp);
+		    } else {
+		        yelp(mtmp);
+#endif
+		    }
+	        }
+
+		if(!mtmp->mtame) newsym(mtmp->mx, mtmp->my);
 
 		mtmp->mtrapped = 0;
 		mtmp->mundetected = 0;
@@ -807,11 +851,11 @@ domove()
 		case 1:	/* trapped */
 		case 3: /* changed levels */
 		    /* there's already been a trap message, reinforce it */
+		    abuse_dog(mtmp);
+#ifndef SOUNDS	/* else complaint from abuse_dog() */
 		    pline("Trapping your pet was a selfish move.");
-		    if (!rn2(4)) {
-			pline("You'll pay!");
-			adjalign(-5);
-		    }
+#endif
+		    adjalign(-3);
 		    break;
 		case 2:
 		    /* it may have drowned or died.  that's no way to
@@ -820,7 +864,7 @@ domove()
 		    if (rn2(4)) {
 			pline ("%s complains in a booming voice:", u_gname());
 			verbalize("Losing your pet like this was a mistake!");
-			u.ugangr++ ;
+			u.ugangr++;
 			adjalign(-15);
 		    }
 		    break;
@@ -910,15 +954,20 @@ spoteffects()
 		if (!is_pool(u.ux,u.uy)) {
 			if (Is_waterlevel(&u.uz))
 				You("pop into an air bubble.");
+			else if (is_lava(u.ux, u.uy))
+				You("leave the water...");	/* oops! */
 			else
-				You("are on solid ground again.");
+				You("are on solid %s again.",
+				    is_ice(u.ux, u.uy) ? "ice" : "land");
 		}
 		else if (Is_waterlevel(&u.uz))
 			goto stillinwater;
-		else if (Levitation || is_floater(uasmon))
+		else if (Levitation)
 			You("pop out of the water like a cork!");
+#ifdef POLYSELF
 		else if (is_flyer(uasmon))
 			You("fly out of the water.");
+#endif
 		else if (Wwalking)
 			You("slowly rise above the surface.");
 		else
@@ -947,11 +996,8 @@ stillinwater:;
 	if(IS_SINK(levl[u.ux][u.uy].typ) && Levitation)
 		dosinkfall();
 #endif
-	if(!flags.nopick && OBJ_AT(u.ux, u.uy) &&
-	   (!is_pool(u.ux,u.uy) || Underwater))
-		pickup(1);
-	else read_engr_at(u.ux,u.uy);
-	if(trap = t_at(u.ux,u.uy))
+	pickup(1);
+	if ((trap = t_at(u.ux,u.uy)) != 0)
 		dotrap(trap);	/* fall into pit, arrow trap, etc. */
 	if((mtmp = m_at(u.ux, u.uy)) && !u.uswallow) {
 		mtmp->mundetected = 0;
@@ -1038,7 +1084,7 @@ register int typewanted;
 
 	min_x = x - 1;
 	max_x = x + 1;
-	if (x < 0)
+	if (x < 1)
 		min_x += step;
 	else 
 	if (x >= COLNO)
@@ -1129,7 +1175,7 @@ register boolean newlev;
 	if (*u.ushops0)
 	    u_left_shop(u.ushops_left, newlev);
 
-	if (!*u.uentered && !*u.ushops_entered) 
+	if (!*u.uentered && !*u.ushops_entered) 	/* implied by newlev */
 	    return;		/* no entrance messages necessary */
 
 	/* Did we just enter a shop? */
@@ -1143,53 +1189,55 @@ register boolean newlev;
 	    /* vault.c insists that a vault remain a VAULT,
 	     * and temples should remain TEMPLEs,
 	     * but everything else gives a message only the first time */
-	    if(!newlev) 
-	   	switch (rt) {
-		    case ZOO:
-		   	pline("Welcome to David's treasure zoo!");
-		    	break;
-		    case SWAMP:
-		    	pline("It %s rather %s down here.",
-	    		      Blind ? "feels" : "looks",
-	    		      Blind ? "humid" : "muddy");
-		    	break;
-		    case COURT:
-			You("enter an opulent throne room!");
-		    	break;
-		    case MORGUE:
-		    	if(midnight())
-			    pline("Run away!  Run away!");
-		    	else
-			    You("have an uncanny feeling...");
-		    	break;
-		    case BEEHIVE:
-		    	You("enter a giant beehive!");
-		    	break;
-#ifdef ARMY
-		    case BARRACKS:
-		    	if(monstinroom(&mons[PM_SOLDIER], roomno) ||
-			    monstinroom(&mons[PM_SERGEANT], roomno) ||
-			    monstinroom(&mons[PM_LIEUTENANT], roomno) ||
-			    monstinroom(&mons[PM_CAPTAIN], roomno))
-		    	    You("enter a military barracks!");
-		    	else 
-			    You("enter an abandoned barracks.");
-		    	break;
+	    switch (rt) {
+		case ZOO:
+		    pline("Welcome to David's treasure zoo!");
+		    break;
+		case SWAMP:
+		    pline("It %s rather %s down here.",
+			  Blind ? "feels" : "looks",
+			  Blind ? "humid" : "muddy");
+		    break;
+		case COURT:
+		    You("enter an opulent throne room!");
+		    break;
+		case MORGUE:
+		    if(midnight()) {
+#ifdef POLYSELF
+			const char *run = locomotion(uasmon, "Run");
+			pline("%s away!  %s away!", run, run);
+#else
+			pline("Run away!  Run away!");
 #endif
-		    case DELPHI:
-		    	if(monstinroom(&mons[PM_ORACLE], roomno))
-			    verbalize("Hello, %s, welcome to Delphi!", plname);
-		    	break;
-		    case TEMPLE:
-			intemple(roomno + ROOMOFFSET);
-		    	/* fall through */
-		    default:
-		    	rt = 0;
-	    	} 
-	    else 
-		rt = 0;
+		    } else
+			You("have an uncanny feeling...");
+		    break;
+		case BEEHIVE:
+		    You("enter a giant beehive!");
+		    break;
+#ifdef ARMY
+		case BARRACKS:
+		    if(monstinroom(&mons[PM_SOLDIER], roomno) ||
+			monstinroom(&mons[PM_SERGEANT], roomno) ||
+			monstinroom(&mons[PM_LIEUTENANT], roomno) ||
+			monstinroom(&mons[PM_CAPTAIN], roomno))
+			You("enter a military barracks!");
+		    else 
+			You("enter an abandoned barracks.");
+		    break;
+#endif
+		case DELPHI:
+		    if(monstinroom(&mons[PM_ORACLE], roomno))
+			verbalize("Hello, %s, welcome to Delphi!", plname);
+		    break;
+		case TEMPLE:
+		    intemple(roomno + ROOMOFFSET);
+		    /* fall through */
+		default:
+		    rt = 0;
+	    } 
 
-	    if(rt != 0) {
+	    if (rt != 0) {
 		rooms[roomno].rtype = OROOM;
 		if (!search_special(rt)) {
 			/* No more room of that type */
@@ -1250,14 +1298,6 @@ dopickup()
 			  Blind ? "feel" : "see");
 		return(1);
 	}
-	if(!OBJ_AT(u.ux, u.uy)) {
-		pline("There is nothing here to pick up.");
-		return(0);
-	}
-	if(Levitation && !Is_airlevel(&u.uz) && !Is_waterlevel(&u.uz)) {
-		You("cannot reach the floor.");
-		return(1);
-	}
 	if(is_pool(u.ux, u.uy)) {
 		if(Wwalking
 #ifdef POLYSELF
@@ -1272,6 +1312,14 @@ dopickup()
 			return(1);
 		}
 	}
+	if(!OBJ_AT(u.ux, u.uy)) {
+		pline("There is nothing here to pick up.");
+		return(0);
+	}
+	if(Levitation && !Is_airlevel(&u.uz) && !Is_waterlevel(&u.uz)) {
+		You("cannot reach the %s.", surface(u.ux,u.uy));
+		return(1);
+	}
 	pickup(-count);
 	return(1);
 }
@@ -1521,8 +1569,14 @@ weight_cap()
 		carrcap = MAX_CARR_CAP;
 	else {
 		if(carrcap > MAX_CARR_CAP) carrcap = MAX_CARR_CAP;
-		if(Wounded_legs & LEFT_SIDE) carrcap -= 100;
-		if(Wounded_legs & RIGHT_SIDE) carrcap -= 100;
+#ifdef POLYSELF
+		if (!is_flyer(uasmon))
+#endif
+					{
+			if(Wounded_legs & LEFT_SIDE) carrcap -= 100;
+			if(Wounded_legs & RIGHT_SIDE) carrcap -= 100;
+		}
+		if (carrcap < 0) carrcap = 0;
 	}
 	return((int) carrcap);
 }
@@ -1555,10 +1609,11 @@ inv_weight()
 int
 near_capacity()
 {
-    int cap, wt = inv_weight();
+    int cap, wt = inv_weight(), wc = weight_cap();
 
-    if (wt < 0) return UNENCUMBERED;
-    cap = (wt / (weight_cap()/2)) + 1;
+    if (wt <= 0) return UNENCUMBERED;
+    if (wc <= 1) return OVERLOADED;
+    cap = (wt*2 / wc) + 1;
     return min(cap, OVERLOADED);
 }
 
