@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)makemon.c	3.1	92/11/01	*/
+/*	SCCS Id: @(#)makemon.c	3.1	93/06/26	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -9,7 +9,7 @@
 # include <ctype.h>
 #endif
 
-STATIC_VAR struct monst NEARDATA zeromonst;
+STATIC_VAR NEARDATA struct monst zeromonst;
 
 #define uncommon(ptr) \
 	(((ptr)->geno & (G_GENOD | G_EXTINCT | G_NOGEN | G_UNIQ)) || \
@@ -19,7 +19,6 @@ STATIC_VAR struct monst NEARDATA zeromonst;
 static boolean NDECL(cmavail);
 static int FDECL(align_shift, (struct permonst *));
 #endif /* OVL0 */
-STATIC_DCL boolean FDECL(is_home_elemental,(struct permonst *));
 STATIC_DCL boolean FDECL(wrong_elem_type, (struct permonst *));
 STATIC_DCL void FDECL(m_initgrp,(struct monst *,int,int,int));
 STATIC_DCL void FDECL(m_initthrow,(struct monst *,int,int));
@@ -36,19 +35,18 @@ extern int monstr[];
 #define tooweak(monindx, lev)	(monstr[monindx] < lev)
 
 #ifdef OVLB
-STATIC_OVL boolean
+boolean
 is_home_elemental(ptr)
 register struct permonst *ptr;
 {
-	if (ptr->mlet != S_ELEMENTAL) return FALSE;
-	if (!In_endgame(&u.uz)) return FALSE;
-	switch(monsndx(ptr)) {
+	if (ptr->mlet == S_ELEMENTAL)
+	    switch (monsndx(ptr)) {
 		case PM_AIR_ELEMENTAL: return Is_airlevel(&u.uz);
 		case PM_FIRE_ELEMENTAL: return Is_firelevel(&u.uz);
 		case PM_EARTH_ELEMENTAL: return Is_earthlevel(&u.uz);
 		case PM_WATER_ELEMENTAL: return Is_waterlevel(&u.uz);
-	}
-	return FALSE;	/* shouldn't be reached */
+	    }
+	return FALSE;
 }
 
 /*
@@ -58,7 +56,9 @@ STATIC_OVL boolean
 wrong_elem_type(ptr)
     register struct permonst *ptr;
 {
-    if (Is_earthlevel(&u.uz)) {
+    if (ptr->mlet == S_ELEMENTAL) {
+	return((boolean)(!is_home_elemental(ptr)));
+    } else if (Is_earthlevel(&u.uz)) {
 	/* no restrictions? */
     } else if (Is_waterlevel(&u.uz)) {
 	/* just monsters that can swim */
@@ -333,6 +333,11 @@ register struct monst *mtmp;
 		(void)mongets(mtmp, KNIFE);
 		(void)mongets(mtmp, LONG_SWORD);
 		break;
+	    case S_ZOMBIE:
+		if (!rn2(4)) (void)mongets(mtmp, LEATHER_ARMOR);
+		if (!rn2(4))
+			(void)mongets(mtmp, (rn2(3) ? KNIFE : SHORT_SWORD));
+		break;
 	    case S_DEMON:
 		switch (mm) {
 		    case PM_BALROG:
@@ -364,35 +369,49 @@ register struct monst *mtmp;
 		if (!is_demon(ptr)) break;
 		/* fall thru */
 /*
- *	Now the general case, ~40% chance of getting some type
- *	of weapon. TODO: Add more weapons types (use bigmonst());
+ *	Now the general case, Some chance of getting some type
+ *	of weapon for "normal" monsters.  Certain special types
+ *	of monsters will get a bonus chance or different selections.
  */
 	    default:
-		switch(rnd(12)) {
+	      {
+		int bias;
+		
+		bias = is_lord(ptr) + is_prince(ptr) * 2 + extra_nasty(ptr);
+		switch(rnd(14 - (2 * bias))) {
 		    case 1:
-			m_initthrow(mtmp, DART, 12);
+			if(strongmonst(ptr)) (void) mongets(mtmp, BATTLE_AXE);
+			else m_initthrow(mtmp, DART, 12);
 			break;
 		    case 2:
-			(void) mongets(mtmp, CROSSBOW);
-			m_initthrow(mtmp, CROSSBOW_BOLT, 12);
+			if(strongmonst(ptr))
+			    (void) mongets(mtmp, TWO_HANDED_SWORD);
+			else {
+			    (void) mongets(mtmp, CROSSBOW);
+			    m_initthrow(mtmp, CROSSBOW_BOLT, 12);
+			}
 			break;
 		    case 3:
 			(void) mongets(mtmp, BOW);
 			m_initthrow(mtmp, ARROW, 12);
 			break;
 		    case 4:
-			m_initthrow(mtmp, DAGGER, 3);
+			if(strongmonst(ptr)) (void) mongets(mtmp, LONG_SWORD);
+			else m_initthrow(mtmp, DAGGER, 3);
 			break;
 		    case 5:
-			(void) mongets(mtmp, AKLYS);
+			if(strongmonst(ptr))
+			    (void) mongets(mtmp, LUCERN_HAMMER);
+			else (void) mongets(mtmp, AKLYS);
 			break;
 		    default:
 			break;
 		}
-		break;
+	      }
+	      break;
 	}
 #ifdef MUSE
-	if ((int) mtmp->m_lev > rn2(70))
+	if ((int) mtmp->m_lev > rn2(75))
 		(void) mongets(mtmp, rnd_offensive_item(mtmp));
 #endif
 }
@@ -546,7 +565,7 @@ register struct	monst	*mtmp;
 	if (ptr == &mons[PM_SOLDIER] && rn2(13)) return;
 #endif
 #ifdef MUSE
-	if ((int) mtmp->m_lev > rn2(30))
+	if ((int) mtmp->m_lev > rn2(50))
 		(void) mongets(mtmp, rnd_defensive_item(mtmp));
 	if ((int) mtmp->m_lev > rn2(100))
 		(void) mongets(mtmp, rnd_misc_item(mtmp));
@@ -851,7 +870,7 @@ struct permonst *mdat;
 	if (mdat) {
 	    if (IS_POOL(levl[x][y].typ))
 		if (mdat == &playermon &&
-		    (HLevitation || Wwalking || Magical_breathing))
+		    (HLevitation || Wwalking || Amphibious))
 			return 1;
 		else	return (is_flyer(mdat) || is_swimmer(mdat));
 	    if (levl[x][y].typ == LAVAPOOL)
@@ -859,7 +878,7 @@ struct permonst *mdat;
 			return 1;
 		else return
 			(is_flyer(mdat) || (mdat == &mons[PM_FIRE_ELEMENTAL]));
-	    if (passes_walls(mdat)) return 1;
+	    if (passes_walls(mdat) && may_passwall(x,y)) return 1;
 	}
 	if (!ACCESSIBLE(levl[x][y].typ)) return 0;
 	if (closed_door(x, y) && (!mdat || !amorphous(mdat)))
@@ -1020,8 +1039,8 @@ static int
 align_shift(ptr)
 register struct permonst *ptr;
 {
-    static long NEARDATA oldmoves = 0L;	/* != 1, starting value of moves */
-    static s_level NEARDATA *lev;
+    static NEARDATA long oldmoves = 0L;	/* != 1, starting value of moves */
+    static NEARDATA s_level *lev;
     register int alshift;
 
     if(oldmoves != moves) {
@@ -1048,12 +1067,12 @@ rndmonst()		/* select a random monster */
 	register struct permonst *ptr;
 	register int i, ct;
 	register int zlevel;
-	static int NEARDATA minmlev, NEARDATA maxmlev, NEARDATA accept;
-	static long NEARDATA oldmoves = 0L;	/* != 1, starting value of moves */
+	static NEARDATA int minmlev, maxmlev, accept;
+	static NEARDATA long oldmoves = 0L;	/* != 1, starting value of moves */
 #ifdef REINCARNATION
-	static boolean NEARDATA upper;
+	static NEARDATA boolean upper;
 #endif
-	static boolean NEARDATA elemlevel;
+	static NEARDATA boolean elemlevel;
 
 #ifdef MULDGN
 	if(u.uz.dnum == quest_dnum && (ptr = qt_montype())) return(ptr);
@@ -1084,7 +1103,7 @@ rndmonst()		/* select a random monster */
 		if(tooweak(ct, minmlev) || toostrong(ct, maxmlev))
 		    continue;
 #ifdef REINCARNATION
-		if(upper && !isupper(def_monsyms[ptr->mlet])) continue;
+		if(upper && !isupper(def_monsyms[(int)ptr->mlet])) continue;
 #endif
 		if(elemlevel && wrong_elem_type(ptr)) continue;
 		if(uncommon(ptr)) continue;
@@ -1107,7 +1126,7 @@ rndmonst()		/* select a random monster */
 		if(tooweak(i, minmlev) || toostrong(i, maxmlev))
 		    continue;
 #ifdef REINCARNATION
-		if(upper & !isupper(def_monsyms[ptr->mlet])) continue;
+		if(upper & !isupper(def_monsyms[(int)ptr->mlet])) continue;
 #endif
 		if(elemlevel && wrong_elem_type(ptr)) continue;
 		if(uncommon(ptr)) continue;
@@ -1156,7 +1175,8 @@ int	spc;
 	for(last = first; last < NUMMONS && mons[last].mlet == class; last++)
 	    if(!(mons[last].geno & mask)) {
 		/* consider it */
-		if(num && toostrong(last, maxmlev) && rn2(2)) break;
+		if(num && toostrong(last, maxmlev) &&
+		   monstr[last] != monstr[last-1] && rn2(2)) break;
 		num += mons[last].geno & G_FREQ;
 	    }
 
@@ -1168,8 +1188,12 @@ int	spc;
 	for(num = rnd(num); num > 0; first++)
 	    if(!(mons[first].geno & mask)) {
 		/* skew towards lower value monsters at lower exp. levels */
-		if(adj_lev(&mons[first]) > (u.ulevel*2)) num--;
 		num -= mons[first].geno & G_FREQ;
+		if (num && adj_lev(&mons[first]) > (u.ulevel*2)) {
+		    /* but not when multiple monsters are same level */
+		    if (mons[first].mlevel != mons[first+1].mlevel)
+			num--;
+		}
 	    }
 	first--; /* correct an off-by-one error */
 
@@ -1205,13 +1229,13 @@ register struct monst *victim;
 	register int newtype;
 	register struct permonst *ptr = mtmp->data;
 
-	if (ptr->mlevel >= 50 || is_golem(ptr) || is_home_elemental(ptr)
-		|| is_mplayer(ptr))
-	    /* doesn't grow up, has strange hp calculation so might be
-	     * weakened by tests below */
-	    return ptr;
-
 	if (victim) {
+	    if (ptr->mlevel >= 50 || is_golem(ptr) || is_home_elemental(ptr)
+		    || is_mplayer(ptr))
+		/* doesn't grow up, has strange hp calculation so might be
+		 * weakened by tests below */
+		return ptr;
+
 	    mtmp->mhpmax = mtmp->mhpmax + (1 + rn2((int)victim->m_lev+1));
 	    if (mtmp->mhpmax <= (8 * (int)mtmp->m_lev)
 			|| (mtmp->m_lev == 0 && mtmp->mhpmax <= 4))
@@ -1240,6 +1264,7 @@ register struct monst *victim;
 			return (struct permonst *)0;
 		}
 		mtmp->data = &mons[newtype];
+		newsym(mtmp->mx, mtmp->my);	/* color may change */
 		mtmp->m_lev = mons[newtype].mlevel;
 	}
 	if (newtype == monsndx(ptr) && victim &&
@@ -1278,12 +1303,19 @@ register int otyp;
 		otmp->oerodeproof = TRUE;
 	    } else if(is_mplayer(mtmp->data) && is_sword(otmp))
 			otmp->spe = (3 + rn2(4));
+
 	    if(otmp->otyp == CANDELABRUM_OF_INVOCATION) {
 		otmp->spe = 0;
 		otmp->age = 0L;
 		otmp->lamplit = FALSE;
 		otmp->blessed = otmp->cursed = FALSE;
+	    } else if (otmp->otyp == BELL_OF_OPENING) {
+		otmp->blessed = otmp->cursed = FALSE;
+	    } else if (otmp->otyp == SPE_BOOK_OF_THE_DEAD) {
+		otmp->blessed = FALSE;
+		otmp->cursed = TRUE;
 	    }
+
 	    mpickobj(mtmp, otmp);
 	    return(otmp->spe);
 	} else return(0);
@@ -1338,14 +1370,14 @@ register struct permonst *ptr;
 	if (mal < A_NEUTRAL && u.uhave.amulet) return FALSE;
 
 	/* minions are hostile to players that have strayed at all */
-	if (is_minion(ptr)) return(u.ualign.record >= 0);
+	if (is_minion(ptr)) return((boolean)(u.ualign.record >= 0));
 
 	/* Last case:  a chance of a co-aligned monster being
 	 * hostile.  This chance is greater if the player has strayed
 	 * (u.ualign.record negative) or the monster is not strongly aligned.
 	 */
-	return !!rn2(16 + (u.ualign.record < -15 ? -15 : u.ualign.record)) &&
-		!!rn2(2 + abs(mal));
+	return((boolean)(!!rn2(16 + (u.ualign.record < -15 ? -15 : u.ualign.record)) &&
+		!!rn2(2 + abs(mal))));
 }
 
 /* Set malign to have the proper effect on player alignment if monster is
@@ -1389,20 +1421,23 @@ struct monst *mtmp;
 		else
 			mtmp->malign = 20;	/* really hostile */
 	} else if (always_peaceful(mtmp->data)) {
+		int absmal = abs(mal);
 		if (mtmp->mpeaceful)
-			mtmp->malign = -3*max(5,abs(mal));
+			mtmp->malign = -3*max(5,absmal);
 		else
-			mtmp->malign = 3*max(5,abs(mal)); /* renegade */
+			mtmp->malign = 3*max(5,absmal); /* renegade */
 	} else if (always_hostile(mtmp->data)) {
+		int absmal = abs(mal);
 		if (coaligned)
 			mtmp->malign = 0;
 		else
-			mtmp->malign = max(5,abs(mal));
+			mtmp->malign = max(5,absmal);
 	} else if (coaligned) {
+		int absmal = abs(mal);
 		if (mtmp->mpeaceful)
-			mtmp->malign = -3*max(3,abs(mal));
+			mtmp->malign = -3*max(3,absmal);
 		else	/* renegade */
-			mtmp->malign = max(3,abs(mal));
+			mtmp->malign = max(3,absmal);
 	} else	/* not coaligned and therefore hostile */
 		mtmp->malign = abs(mal);
 }
@@ -1410,7 +1445,7 @@ struct monst *mtmp;
 #endif /* OVL1 */
 #ifdef OVLB
 
-static char NEARDATA syms[] = {
+static NEARDATA char syms[] = {
 	MAXOCLASSES, MAXOCLASSES+1, RING_CLASS, WAND_CLASS, WEAPON_CLASS,
 	FOOD_CLASS, GOLD_CLASS, SCROLL_CLASS, POTION_CLASS, ARMOR_CLASS,
 	AMULET_CLASS, TOOL_CLASS, ROCK_CLASS, GEM_CLASS, SPBOOK_CLASS,
