@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)mkobj.c	3.1	93/01/17	*/
+/*	SCCS Id: @(#)mkobj.c	3.1	93/05/01	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -66,8 +66,6 @@ const struct icp hellprobs[] = {
 { 4, AMULET_CLASS}
 };
 
-static int NEARDATA mksx=0, NEARDATA mksy=0;
-
 struct obj *
 mkobj_at(let,x,y, artif)
 char let;
@@ -76,16 +74,10 @@ boolean artif;
 {
 	register struct obj *otmp;
 
-	mksx = x; mksy = y;
-	/* We need to know the X, Y coordinates while creating the object,
-	 * to insure shop boxes are empty.
-	 * Yes, this is a horrible kludge...
-	 */
 	otmp = mkobj(let,artif);
 	otmp->nobj = fobj;
 	fobj = otmp;
 	place_object(otmp, x, y);
-	mksx = mksy = 0;
 	return(otmp);
 }
 
@@ -96,12 +88,11 @@ boolean init;
 {
 	register struct obj *otmp;
 
-	mksx = x; mksy = y;
 	otmp = mksobj(otyp,init,TRUE);
 	otmp->nobj = fobj;
+	fobj = otmp;
 	place_object(otmp, x, y);
-	mksx = mksy = 0;
-	return((fobj = otmp));
+	return(otmp);
 }
 
 struct obj *
@@ -598,7 +589,9 @@ register struct obj *obj;
 	if (obj->otyp == CORPSE && obj->corpsenm > -1)
 		return (int)obj->quan * mons[obj->corpsenm].cwt;
 	else if (obj->otyp == GOLD_PIECE)
-		return (int)((obj->quan + 500L) / 1000L);
+		return (int)((obj->quan + 50L) / 100L);
+	else if (obj->otyp == HEAVY_IRON_BALL && obj->owt != 0)
+		return((int)(obj->owt));	/* kludge for "very" heavy iron ball */
 	return(wt ? wt*(int)obj->quan : ((int)obj->quan + 1)>>1);
 }
 
@@ -676,7 +669,6 @@ register int lth;
 		otmp = oname(otmp, nm, FALSE);
 		fobj = otmp;
 		level.objects[x][y] = otmp;
-		if (is_pool(x,y)) water_damage(otmp,TRUE);
 	}
 	return(otmp);
 }
@@ -689,9 +681,8 @@ register struct obj *otmp;
 
 	if (objects[otyp].oc_oprop == FIRE_RES) return FALSE;
 
-	return((objects[otyp].oc_material == WOOD ||
-			objects[otyp].oc_material == 0));
-
+	return((boolean)(objects[otyp].oc_material <= WOOD &&
+			objects[otyp].oc_material != LIQUID));
 }
 
 #endif /* OVLB */
@@ -719,7 +710,6 @@ int x, y;
 	otmp->nexthere = otmp2;
 	level.objects[x][y] = otmp;
     }
-    if (is_pool(x,y)) water_damage(otmp,TRUE);
 
     /* set the new object's location */
     otmp->ox = x;
