@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)mon.c	3.1	93/01/19	*/
+/*	SCCS Id: @(#)mon.c	3.1	93/06/12	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -19,8 +19,10 @@ STATIC_DCL void NDECL(dmonsfree);
 #define warnDelay 10
 long lastwarntime;
 int lastwarnlev;
+
 const char *warnings[] = {
-	"white", "pink", "red", "ruby", "purple", "black" };
+	"white", "pink", "red", "ruby", "purple", "black"
+};
 
 static void NDECL(warn_effects);
 
@@ -213,7 +215,7 @@ warn_effects()
 		break;
 	    default:
 		if (Hallucination)
-		    Your("spider-sense is tingling....");
+		    Your("spider-sense is tingling...");
 		else
 		    You("feel apprehensive as you sense a %s flash.",
 			warnings[warnlevel]);
@@ -230,7 +232,8 @@ register struct monst *mtmp;
 {
     boolean inpool, infountain;
 
-    inpool = is_pool(mtmp->mx,mtmp->my);
+    inpool = is_pool(mtmp->mx,mtmp->my) &&
+	     !is_flyer(mtmp->data) && !is_floater(mtmp->data);
     infountain = IS_FOUNTAIN(levl[mtmp->mx][mtmp->my].typ);
 
     /* Gremlin multiplying won't go on forever since the hit points
@@ -246,15 +249,26 @@ register struct monst *mtmp;
 		pline("%s multiplies.", Monnam(mtmp));
 	    dryup(mtmp->mx,mtmp->my);
 	}
+	if (inpool) water_damage(mtmp->minvent, FALSE, FALSE);
 	return (0);
     }
     if (inpool) {
-	/* most monsters drown in pools */
-	if (!is_flyer(mtmp->data) && !is_clinger(mtmp->data)
-	    && !is_swimmer(mtmp->data) && !magic_breathing(mtmp->data)) {
+	/* Most monsters drown in pools.  flooreffects() will take care of
+	 * water damage to dead monsters' inventory, but survivors need to
+	 * be handled here.  Swimmers are able to protect their stuff...
+	 */
+	if (!is_clinger(mtmp->data)
+	    && !is_swimmer(mtmp->data) && !amphibious(mtmp->data)) {
 	    if (cansee(mtmp->mx,mtmp->my))
 		pline("%s drowns.", Monnam(mtmp));
 	    mondead(mtmp);
+#ifdef MUSE
+	    if (mtmp->mhp > 0) {
+		rloc(mtmp);
+		water_damage(mtmp->minvent, FALSE, FALSE);
+		return 0;
+	    }
+#endif
 	    return (1);
 	}
     } else {
@@ -365,7 +379,8 @@ meatgold(mtmp)
 	/* Eats topmost metal object if it is there */
 	for (otmp = level.objects[mtmp->mx][mtmp->my];
 						    otmp; otmp = otmp->nexthere)
-	    if (is_metallic(otmp) && touch_artifact(otmp,mtmp)) {
+	    if (is_metallic(otmp) && !obj_resists(otmp, 5, 95) &&
+		touch_artifact(otmp,mtmp)) {
 		    if (cansee(mtmp->mx,mtmp->my) && flags.verbose)
 			pline("%s eats %s!", Monnam(mtmp),
 				distant_name(otmp,doname));
@@ -404,7 +419,8 @@ meatobj(mtmp)		/* for gelatinous cubes */
 	/* Engulfs others, except huge rocks and metal attached to player */
 	for (otmp = level.objects[mtmp->mx][mtmp->my]; otmp; otmp = otmp2) {
 	    otmp2 = otmp->nexthere;
-	    if(is_organic(otmp) && touch_artifact(otmp,mtmp)) {
+	    if (is_organic(otmp) && !obj_resists(otmp, 5, 95) &&
+		    touch_artifact(otmp,mtmp)) {
 		if (otmp->otyp == CORPSE && otmp->corpsenm == PM_COCKATRICE
 						&& !resists_ston(mtmp->data))
 		    continue;
@@ -483,21 +499,27 @@ mpickstuff(mtmp, str)
 	register struct monst *mtmp;
 	register const char *str;
 {
-	register struct obj *otmp;
+	register struct obj *otmp, *otmp2;
 
 /*	prevent shopkeepers from leaving the door of their shop */
 	if(mtmp->isshk && inhishop(mtmp)) return;
 
-	for(otmp = level.objects[mtmp->mx][mtmp->my]; otmp; otmp=otmp->nexthere)
+	for(otmp = level.objects[mtmp->mx][mtmp->my]; otmp; otmp = otmp2) {
+	    otmp2 = otmp->nexthere;
 /*	Nymphs take everything.  Most monsters don't pick up corpses. */
 	    if (
 #ifdef MUSE
 		!str ? searches_for_item(mtmp,otmp) :
 #endif
-		  (index(str, otmp->oclass)
-		   && (otmp->otyp != CORPSE || mtmp->data->mlet == S_NYMPH))) {
-		if (!touch_artifact(otmp,mtmp)) return;
-		if (!can_carry(mtmp,otmp)) return;
+		  !!(index(str, otmp->oclass))) {
+		if (otmp->otyp == CORPSE && mtmp->data->mlet != S_NYMPH
+#ifdef MUSE
+    && (otmp->corpsenm != PM_COCKATRICE || !(mtmp->misc_worn_check & W_ARMG))
+#endif
+									)
+			continue;
+		if (!touch_artifact(otmp,mtmp)) continue;
+		if (!can_carry(mtmp,otmp)) continue;
 		if (cansee(mtmp->mx,mtmp->my) && flags.verbose)
 			pline("%s picks up %s.", Monnam(mtmp), doname(otmp));
 		freeobj(otmp);
@@ -508,6 +530,7 @@ mpickstuff(mtmp, str)
 		newsym(mtmp->mx, mtmp->my);
 		return;			/* pick only one object */
 	    }
+	}
 }
 
 #endif /* OVL2 */
@@ -565,6 +588,9 @@ struct obj *otmp;
 	register int newload = otmp->owt;
 
 	if (otmp->otyp == CORPSE && otmp->corpsenm == PM_COCKATRICE
+#ifdef MUSE
+			&& !(mtmp->misc_worn_check & W_ARMG)
+#endif
 						&& !resists_ston(mtmp->data))
 		return(FALSE);
 	if (mtmp->isshk) return(TRUE); /* no limit */
@@ -579,7 +605,7 @@ struct obj *otmp;
 
 	/* nymphs deal in stolen merchandise, but not boulders or statues */
 	if (mtmp->data->mlet == S_NYMPH)
-		return !(otmp->oclass == ROCK_CLASS);
+		return((boolean)(!(otmp->oclass == ROCK_CLASS)));
 
 	if(curr_mon_load(mtmp) + newload > max_mon_load(mtmp)) return(FALSE);
 
@@ -624,8 +650,9 @@ nexttry:	/* eels prefer the water, but if there is no water nearby,
 	for(nx = max(1,x-1); nx <= maxx; nx++)
 	  for(ny = max(0,y-1); ny <= maxy; ny++) {
 	    if(nx == x && ny == y) continue;
-	    if(IS_ROCK(ntyp = levl[nx][ny].typ) && !(flag & ALLOW_WALL) &&
-		!((flag & ALLOW_DIG) && may_dig(nx,ny))) continue;
+	    if(IS_ROCK(ntyp = levl[nx][ny].typ) &&
+	       !((flag & ALLOW_WALL) && may_passwall(nx,ny)) &&
+	       !((flag & ALLOW_DIG) && may_dig(nx,ny))) continue;
 	    if(IS_DOOR(ntyp) && !amorphous(mon->data) &&
 	       ((levl[nx][ny].doormask & D_CLOSED && !(flag & OPENDOOR)) ||
 		(levl[nx][ny].doormask & D_LOCKED && !(flag & UNLOCKDOOR))
@@ -753,7 +780,7 @@ register int x,y;
 {
 	register int distance = dist2(mon->mx, mon->my, x, y);
 	if (distance==2 && mon->data==&mons[PM_GRID_BUG]) return 0;
-	return (distance < 3);
+	return((boolean)(distance < 3));
 }
 
 #endif /* OVL1 */
@@ -837,6 +864,46 @@ register struct monst *mtmp;
 	if(mtmp->wormno) wormgone(mtmp);
 }
 
+#ifdef MUSE
+static void FDECL(lifesaved_monster, (struct monst *));
+
+static void
+lifesaved_monster(mtmp)
+struct monst *mtmp;
+{
+	struct obj *lifesave;
+
+	if ((lifesave = which_armor(mtmp, W_AMUL))
+			&& lifesave->otyp == AMULET_OF_LIFE_SAVING) {
+		pline("But wait...");
+		if (canseemon(mtmp)) {
+			pline("%s's medallion begins to glow!",
+				Monnam(mtmp));
+			makeknown(AMULET_OF_LIFE_SAVING);
+			pline("%s looks much better!", Monnam(mtmp));
+			pline("The medallion crumbles to dust!");
+		} else
+			pline("Maybe not...");
+		m_useup(mtmp, lifesave);
+		if (mtmp->mhpmax <= 0) mtmp->mhpmax = 10;
+		mtmp->mhp = mtmp->mhpmax;
+		mtmp->mcanmove = 1;
+		mtmp->mfrozen = 0;
+		if (mtmp->mtame && !mtmp->isminion) {
+			struct edog *edog = EDOG(mtmp);
+			if (edog->hungrytime < moves+500)
+				edog->hungrytime = moves+500;
+		}
+		if (mtmp->data->geno & G_GENOD)
+			pline("Unfortunately %s is still genocided...",
+				mon_nam(mtmp));
+		else
+			return;
+	}
+	mtmp->mhp = 0;
+}
+#endif
+
 void
 mondead(mtmp)
 register struct monst *mtmp;
@@ -848,6 +915,10 @@ register struct monst *mtmp;
 		 * the m_detach or there will be relmon problems later */
 		if(!grddead(mtmp)) return;
 	}
+#ifdef MUSE
+	lifesaved_monster(mtmp);
+	if (mtmp->mhp > 0) return;
+#endif
 
 	/* restore chameleon, lycanthropes to true form at death */
 	if(mtmp->cham) mtmp->data = &mons[PM_CHAMELEON];
@@ -914,11 +985,14 @@ mondied(mdef)
 register struct monst *mdef;
 {
 	mondead(mdef);
+#ifdef MUSE
+	if(mdef->mhp > 0) return; /* lifesaved */
+#endif
 	if(rn2(3)
 #ifdef REINCARNATION
 	   && !Is_rogue_level(&u.uz)
 #endif
-					)
+	   && !(level.flags.graveyard && is_undead(mdef->data) && rn2(3)))
 		(void) make_corpse(mdef);
 }
 
@@ -945,19 +1019,44 @@ void
 monstone(mdef)
 register struct monst *mdef;
 {
-	struct obj *otmp, *contents;
+	struct obj *otmp, *obj, *nxt, *contents;
 	xchar x = mdef->mx, y = mdef->my;
 
+#ifdef MUSE
+	/* we have to make the statue before calling mondead, to be able to
+	 * put inventory in it, and we have to check for lifesaving before
+	 * making the statue....
+	 */
+	lifesaved_monster(mdef);
+	if (mdef->mhp > 0) return;
+#endif
+	mdef->mtrapped = 0;	/* (see m_detach) */
+
 	if((int)mdef->data->msize > MZ_TINY ||
 	   !rn2(2 + ((mdef->data->geno & G_FREQ) > 2))) {
 		otmp = mk_named_object(STATUE, mdef->data, x, y,
-			NAME(mdef), (int)mdef->mnamelth);
-		contents = otmp->cobj = mdef->minvent;
-		while(contents) {
-			contents->owornmask = 0L;
-			contents = contents->nobj;
+					NAME(mdef), (int)mdef->mnamelth);
+		/* some objects may end up outside the statue */
+		contents = 0;
+		for (obj = mdef->minvent; obj; obj = nxt) {
+		    nxt = obj->nobj;
+		    obj->owornmask = 0L;
+		    if (obj->otyp == BOULDER ||
+#if 0				/* monsters don't carry statues */
+     (obj->otyp == STATUE && mons[obj->corpsenm].msize >= mdef->data->msize) ||
+#endif
+				obj_resists(obj, 0, 0)) {
+			if (flooreffects(obj, x, y, "fall")) continue;
+			place_object(obj, x, y);
+			obj->nobj = fobj;
+			fobj = obj;
+		    } else {
+			obj->nobj = contents;
+			contents = obj;
+		    }
 		}
-		mdef->minvent = (struct obj *)0;
+		otmp->cobj = contents;
+		mdef->minvent = 0;
 		if (mdef->mgold) {
 			struct obj *au;
 			au = mksobj(GOLD_PIECE, FALSE, FALSE);
@@ -971,10 +1070,9 @@ register struct monst *mdef;
 	} else
 		otmp = mksobj_at(ROCK, x, y, TRUE);
 
-	mondead(mdef);
-
 	stackobj(otmp);
 	if (cansee(x, y)) newsym(x,y);
+	mondead(mdef);
 }
 
 /* another monster has killed the monster mdef */
@@ -982,7 +1080,7 @@ void
 monkilled(mdef, fltxt, how)
 register struct monst *mdef;
 const char *fltxt;
-uchar how;
+int how;
 {
 	if (cansee(mdef->mx, mdef->my) && fltxt)
 	    pline("%s is %s%s%s!", Monnam(mdef),
@@ -1044,7 +1142,8 @@ xkilled(mtmp, dest)
 	boolean wasinside = u.uswallow && (u.ustuck == mtmp);
 
 	if (dest & 1) {
-	    if(!canseemon(mtmp) && !sensemon(mtmp)) You("destroy it!");
+	    if(!wasinside && !canseemon(mtmp) && !sensemon(mtmp))
+		You("destroy it!");
 	    else {
 		You("destroy %s!",
 			mtmp->mtame ? x_monnam(mtmp, 0, "poor", 0)
@@ -1066,6 +1165,10 @@ xkilled(mtmp, dest)
 	if(stoned) monstone(mtmp);
 	else mondead(mtmp);
 
+#ifdef MUSE
+	if (mtmp->mhp > 0) return; /* monster lifesaved */
+#endif
+
 	mdat = mtmp->data; /* note: mondead can change mtmp->data */
 
 	if (stoned) {
@@ -1077,7 +1180,7 @@ xkilled(mtmp, dest)
 #ifdef REINCARNATION
 		 || Is_rogue_level(&u.uz)
 #endif
-	   || (mdat == &mons[PM_WRAITH] && Is_valley(&u.uz) && rn2(5)))
+	   || (level.flags.graveyard && is_undead(mdat) && rn2(3)))
 		goto cleanup;
 
 #ifdef MAIL
@@ -1136,7 +1239,7 @@ xkilled(mtmp, dest)
 	if(redisp) newsym(x,y);
 cleanup:
 	/* punish bad behaviour */
-	if(is_human(mdat) && !always_hostile(mdat) &&
+	if(is_human(mdat) && (!always_hostile(mdat) && mtmp->malign <= 0) &&
 	   (monsndx(mdat) < PM_ARCHEOLOGIST || monsndx(mdat) > PM_WIZARD) &&
 	   u.ualign.type != A_CHAOTIC) {
 		HTelepat &= ~INTRINSIC;
@@ -1321,8 +1424,23 @@ register struct monst *mtmp;
     if(mtmp->data->msound == MS_SHRIEK) {
 	if(flags.soundok)
 	    pline("%s shrieks.", Monnam(mtmp));
+	if (!rn2(10)) {
+	    if (!rn2(13))
+		(void) makemon(&mons[PM_PURPLE_WORM], 0, 0);
+	    else
+		(void) makemon((struct permonst *)0, 0, 0);
+
+	}
 	aggravate();
     }
+    if(mtmp->data == &mons[PM_MEDUSA] && !mtmp->mcan) {
+	register int i;
+	for(i = 0; i < NATTK; i++)
+	     if(mtmp->data->mattk[i].aatyp == AT_GAZE) {
+		 (void) gazemu(mtmp, &mtmp->data->mattk[i]);
+		 break;
+	     }
+    }
 }
 
 #endif /* OVLB */
@@ -1481,6 +1599,20 @@ register struct permonst *mdat;
 		if(!rn2(10)) mtmp->female = !mtmp->female;
 	}
 
+	if (In_endgame(&u.uz) && is_mplayer(olddata)) {
+		/* mplayers start out as "Foo the Bar", but some of the
+		 * titles are inappropriate when polymorphed, particularly
+		 * into the opposite sex.  players don't use ranks when
+		 * polymorphed, so dropping the rank for mplayers seems
+		 * reasonable.
+		 */
+		char *p = index(NAME(mtmp), ' ');
+		if (p) {
+			*p = '\0';
+			mtmp->mnamelth = p - NAME(mtmp) + 1;
+		}
+	}
+
 	if(mdat == mtmp->data) return(0);	/* still the same monster */
 
 	if(mtmp->wormno) {			/* throw tail away */
@@ -1552,6 +1684,10 @@ register struct permonst *mdat;
 	newsym(mtmp->mx,mtmp->my);
 #ifdef MUSE
 	mon_break_armor(mtmp);
+	/* Unfortunately, by now we forgot who did the polymorph, so we don't
+	 * have any way to give the player credit if this was a polymorph wand.
+	 */
+	mselftouch(mtmp, "No longer petrify-resistant, ", FALSE);
 	possibly_unwield(mtmp);
 #endif
 	return(1);
