@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)music.c	3.1	92/11/26	*/
+/*	SCCS Id: @(#)music.c	3.1	93/05/25	*/
 /* 	Copyright (c) 1989 by Jean-Christophe Collet */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -22,7 +22,7 @@
  *			level.
  * (leather) drum	Will awaken monsters like the horn.
  * drum of earthquake	Will initiate an earthquake whose intensity depends
- *			on player level.  That is, it creates ramdom pits
+ *			on player level.  That is, it creates random pits
  *			called here chasms.
  */
 
@@ -36,6 +36,21 @@ static void FDECL(charm_monsters,(int));
 static void FDECL(do_earthquake,(int));
 static int FDECL(do_improvisation,(struct obj *));
 
+#ifdef UNIX386MUSIC
+static int NDECL(atconsole);
+static void FDECL(speaker,(struct obj *,char *));
+#endif
+#ifdef VPIX_MUSIC
+extern int sco_flag_console;	/* will need changing if not _M_UNIX */
+static void NDECL(playinit);
+static void FDECL(playstring, (char *,size_t));
+static void FDECL(speaker,(struct obj *,char *));
+#endif
+
+#ifdef AMIGA
+void FDECL( amii_speaker, ( struct obj *, char *, int ) );
+#endif
+
 /*
  * Wake every monster in range...
  */
@@ -45,16 +60,18 @@ awaken_monsters(distance)
 int distance;
 {
 	register struct monst *mtmp = fmon;
+	register int distm;
 
 	while(mtmp) {
-		if (distu(mtmp->mx, mtmp->my) < distance/3) {
-			/* May scare some monsters */
-			if (!resist(mtmp, SCROLL_CLASS, 0, NOTELL))
-			  mtmp->mflee = 1;
-		} else if (distu(mtmp->mx, mtmp->my) < distance) {
-			mtmp->msleep = 0;
-			mtmp->mcanmove = 1;
-			mtmp->mfrozen = 0;
+		distm = distu(mtmp->mx, mtmp->my);
+		if (distm < distance) {
+		    mtmp->msleep = 0;
+		    mtmp->mcanmove = 1;
+		    mtmp->mfrozen = 0;
+		    /* May scare some monsters */
+		    if (distm < distance/3 &&
+			    !resist(mtmp, SCROLL_CLASS, 0, NOTELL))
+			mtmp->mflee = 1;
 		}
 		mtmp = mtmp->nmon;
 	}
@@ -71,9 +88,15 @@ int distance;
 	register struct monst *mtmp = fmon;
 
 	while(mtmp) {
-		  if (distu(mtmp->mx, mtmp->my) < distance)
-		    if(mtmp->mcanmove && !resist(mtmp, WAND_CLASS, 0, NOTELL))
-		      mtmp->mcanmove = mtmp->mfrozen = 0;
+		if (distu(mtmp->mx, mtmp->my) < distance &&
+			!resist(mtmp, WAND_CLASS, 0, NOTELL)) {
+		    register int min_sleep = d(10,10);
+		    /* 10d10 turns + wake_nearby() to rouse */
+		    mtmp->msleep = 1;
+		    mtmp->mcanmove = 0;
+		    if ((int)mtmp->mfrozen < min_sleep)
+			mtmp->mfrozen = min_sleep;
+		}
 		mtmp = mtmp->nmon;
 	}
 }
@@ -125,18 +148,19 @@ int distance;
 /* Awake only soldiers of the level. */
 
 void
-awaken_soldiers() {
+awaken_soldiers()
+{
 #ifdef ARMY
 	register struct monst *mtmp = fmon;
 
 	while(mtmp) {
 	    if (is_mercenary(mtmp->data) && mtmp->data != &mons[PM_GUARD]) {
-		mtmp->mpeaceful = mtmp->msleep = 0;
+		mtmp->mpeaceful = mtmp->msleep = mtmp->mfrozen = 0;
 		mtmp->mcanmove = 1;
 		if (canseemon(mtmp))
 		    pline("%s is now ready for battle!", Monnam(mtmp));
 		else
-		    Norep("You hear the sound of battle gear being readied.");
+		    Norep("You hear the rattle of battle gear being readied.");
 	    }
 	    mtmp = mtmp->nmon;
 	}
@@ -184,7 +208,8 @@ int force;
 	if (end_x >= COLNO) end_x = COLNO - 1;
 	if (end_y >= ROWNO) end_y = ROWNO - 1;
 	for (x=start_x; x<=end_x; x++) for (y=start_y; y<=end_y; y++) {
-	    if (mtmp = m_at(x,y)) {
+	    if ((mtmp = m_at(x,y)) != 0) {
+		wakeup(mtmp);	/* peaceful monster will become hostile */
 		if (mtmp->mundetected && is_hider(mtmp->data)) {
 		    mtmp->mundetected = 0;
 		    if (cansee(x,y))
@@ -220,13 +245,14 @@ int force;
 		  case ROOM :
 		  case CORR : /* Try to make a pit */
 do_pit:		    chasm = maketrap(x,y,PIT);
+		    if (!chasm) break;	/* no pit if portal at that location */
 		    chasm->tseen = 1;
 
 		    levl[x][y].doormask = 0;
 
 		    mtmp = m_at(x,y);
 
-		    if (otmp = sobj_at(BOULDER, x, y)) {
+		    if ((otmp = sobj_at(BOULDER, x, y)) != 0) {
 			if (cansee(x, y))
 			   pline("KADOOM! The boulder falls into a chasm%s!",
 			      ((x == u.ux) && (y == u.uy)) ? " below you" : "");
@@ -235,7 +261,7 @@ do_pit:		    chasm = maketrap(x,y,PIT);
 			freeobj(otmp);
 			(void) flooreffects(otmp, x, y, "");
 			break;
-		    }	
+		    }
 
 		    /* We have to check whether monsters or player
 		       falls in a chasm... */
@@ -247,6 +273,10 @@ do_pit:		    chasm = maketrap(x,y,PIT);
 				pline("%s falls into a chasm!", Monnam(mtmp));
 			    else if (flags.soundok && humanoid(mtmp->data))
 				You("hear a scream!");
+#ifdef MUSE
+			    mselftouch(mtmp, "Falling, ", TRUE);
+			    if (mtmp->mhp > 0)
+#endif
 			    if ((mtmp->mhp -= rnd(6)) <= 0) {
 				if(!cansee(x,y))
 				    pline("It is destroyed!");
@@ -300,7 +330,13 @@ struct obj *instr;
 #ifdef MAC
 	mac_speaker ( instr , "C" ) ;
 #endif
-
+#ifdef AMIGA
+	amii_speaker ( instr , "Cw", AMII_OKAY_VOLUME ) ;
+#endif
+#ifdef VPIX_MUSIC
+	if (sco_flag_console)
+		speaker(instr, "C");
+#endif
 	if (Confusion)
 	  pline("What you produce is quite far from music...");
 	else
@@ -314,6 +350,7 @@ struct obj *instr;
 	      case MAGIC_FLUTE: /* Make monster fall asleep */
 		if (instr->spe > 0) {
 			instr->spe--;
+			check_unpaid(instr);
 			You("produce soft music.");
 			put_monsters_to_sleep((int)u.ulevel*5);
 		}
@@ -328,6 +365,7 @@ struct obj *instr;
 	      case FIRE_HORN:	/* Idem wand of fire */
 		if (instr->spe > 0) {
 			instr->spe--;
+			check_unpaid(instr);
 			if (!getdir(NULL)) {
 				if (!Blind)
 				    pline("%s glows then fades.",
@@ -359,9 +397,10 @@ struct obj *instr;
 		break;
 	      case MAGIC_HARP:	/* Charm monsters */
 		if (instr->spe > 0) {
+			instr->spe--;
+			check_unpaid(instr);
 			pline("%s produces very attractive music.",
 			      The(xname(instr)));
-			instr->spe--;
 			charm_monsters(((int)u.ulevel - 1) / 3 + 1);
 		}
 		exercise(A_DEX, TRUE);
@@ -373,10 +412,13 @@ struct obj *instr;
 		break;
 	      case DRUM_OF_EARTHQUAKE:	/* create several pits */
 		if (instr->spe > 0) {
+			instr->spe--;
+			check_unpaid(instr);
 			You("produce a heavy, thunderous rolling!");
 			pline("The entire dungeon is shaking around you!");
-			instr->spe--;
 			do_earthquake(((int)u.ulevel - 1) / 3 + 1);
+			/* shake up monsters in a much larger radius... */
+			awaken_monsters(ROWNO * COLNO);
 			makeknown(DRUM_OF_EARTHQUAKE);
 		}
 		break;
@@ -387,70 +429,6 @@ struct obj *instr;
 	return (2);		/* That takes time */
 }
 
-#ifdef SYSV386MUSIC
-/*
- * Play audible music on the machine's speaker if appropriate.
- */
-
-static int
-atconsole()
-{
-    /*
-     * Kluge alert: This code assumes that your [34]86 has no X terminals
-     * attached and that the console tty type is AT386 (this is always true
-     * under AT&T UNIX for these boxen). The theory here is that your remote
-     * ttys will have terminal type `ansi' or something else other than
-     * `AT386' or `xterm'. We'd like to do better than this, but testing
-     * to see if we're running on the console physical terminal is quite
-     * difficult given the presence of virtual consoles and other modern
-     * UNIX impedimenta...
-     */
-    char	*termtype = getenv("TERM");
-
-     return(!strcmp(termtype, "AT386") || !strcmp(termtype, "xterm"));
-}
-
-static void
-speaker(instr, buf)
-struct obj *instr;
-char	*buf;
-{
-    /*
-     * For this to work, you need to have installed the PD speaker-control
-     * driver for PC-compatible UNIX boxes that I (eric@snark.uu.net)
-     * posted to comp.sources.unix in Feb 1990. A copy may be included
-     * with your nethack distribution.
-     */
-    int	fd;
-
-    if ((fd = open("/dev/speaker", 1)) != -1)
-    {
-	/* emit a prefix to modify instrumental `timbre' */
-	switch (instr->otyp)
-	{
-	case WOODEN_FLUTE:
-	case MAGIC_FLUTE:
-	    (void) write(fd, ">ol", 1); /* up one octave & lock */
-	    break;
-	case TOOLED_HORN:
-	case FROST_HORN:
-	case FIRE_HORN:
-	    (void) write(fd, "<<ol", 2); /* drop two octaves & lock */
-	    break;
-	case BUGLE:
-	    (void) write(fd, "ol", 2); /* octave lock */
-	    break;
-	case WOODEN_HARP:
-	case MAGIC_HARP:
-	    (void) write(fd, "l8mlol", 4); /* fast, legato, octave lock */
-	    break;
-	}
-	(void) write(fd, buf, strlen(buf));
-	(void) close(fd);
-    }
-}
-#endif /* SYSV386MUSIC */
-
 /*
  * So you want music...
  */
@@ -459,7 +437,10 @@ int
 do_play_instrument(instr)
 struct obj *instr;
 {
-    char buf[BUFSZ], *s, c = 'y';
+    char buf[BUFSZ], c = 'y';
+#ifndef	AMIGA
+	char *s;
+#endif
     int x,y;
     boolean ok;
 
@@ -471,16 +452,39 @@ struct obj *instr;
 	c = yn("Improvise?");
     }
     if (c == 'n') {
-	getlin("What tune are you playing? [what 5 notes]", buf);
+	if (u.uevent.uheard_tune == 2 && yn("Play the passtune?") == 'y')
+		Strcpy(buf, tune);
+	else
+		getlin("What tune are you playing? [what 5 notes]", buf);
+#ifndef	AMIGA
+	/* The AMIGA supports two octaves of notes */
 	for (s=buf; *s; s++) *s = highc(*s);
+#endif
 	You("extract a strange sound from %s!", the(xname(instr)));
-#ifdef SYSV386MUSIC 
+#ifdef UNIX386MUSIC
 	/* if user is at the console, play through the console speaker */
 	if (atconsole())
 	    speaker(instr, buf);
-#endif /* SYSV386MUSIC */
+#endif
+#ifdef VPIX_MUSIC
+	if (sco_flag_console)
+	    speaker(instr, buf);
+#endif
 #ifdef MAC
 	mac_speaker ( instr , buf ) ;
+#endif
+#ifdef AMIGA
+	{
+		char nbuf[ 20 ];
+		int i;
+		for( i = 0; buf[i] && i < 5; ++i )
+		{
+			nbuf[ i*2 ] = buf[ i ];
+			nbuf[ (i*2)+1 ] = 'h';
+		}
+		nbuf[ i*2 ] = 0;
+		amii_speaker ( instr , nbuf, AMII_OKAY_VOLUME ) ;
+	}
 #endif
 	/* Check if there was the Stronghold drawbridge near
 	 * and if the tune conforms to what we're waiting for.
@@ -553,4 +557,147 @@ struct obj *instr;
 	    return do_improvisation(instr);
 }
 
+#ifdef UNIX386MUSIC
+/*
+ * Play audible music on the machine's speaker if appropriate.
+ */
+
+static int
+atconsole()
+{
+    /*
+     * Kluge alert: This code assumes that your [34]86 has no X terminals
+     * attached and that the console tty type is AT386 (this is always true
+     * under AT&T UNIX for these boxen). The theory here is that your remote
+     * ttys will have terminal type `ansi' or something else other than
+     * `AT386' or `xterm'. We'd like to do better than this, but testing
+     * to see if we're running on the console physical terminal is quite
+     * difficult given the presence of virtual consoles and other modern
+     * UNIX impedimenta...
+     */
+    char	*termtype = getenv("TERM");
+
+     return(!strcmp(termtype, "AT386") || !strcmp(termtype, "xterm"));
+}
+
+static void
+speaker(instr, buf)
+struct obj *instr;
+char	*buf;
+{
+    /*
+     * For this to work, you need to have installed the PD speaker-control
+     * driver for PC-compatible UNIX boxes that I (esr@snark.thyrsus.com)
+     * posted to comp.sources.unix in Feb 1990.  A copy should be included
+     * with your nethack distribution.
+     */
+    int	fd;
+
+    if ((fd = open("/dev/speaker", 1)) != -1)
+    {
+	/* send a prefix to modify instrumental `timbre' */
+	switch (instr->otyp)
+	{
+	case WOODEN_FLUTE:
+	case MAGIC_FLUTE:
+	    (void) write(fd, ">ol", 1); /* up one octave & lock */
+	    break;
+	case TOOLED_HORN:
+	case FROST_HORN:
+	case FIRE_HORN:
+	    (void) write(fd, "<<ol", 2); /* drop two octaves & lock */
+	    break;
+	case BUGLE:
+	    (void) write(fd, "ol", 2); /* octave lock */
+	    break;
+	case WOODEN_HARP:
+	case MAGIC_HARP:
+	    (void) write(fd, "l8mlol", 4); /* fast, legato, octave lock */
+	    break;
+	}
+	(void) write(fd, buf, strlen(buf));
+	(void) close(fd);
+    }
+}
+#endif /* UNIX386MUSIC */
+
+#ifdef VPIX_MUSIC
+
+# if 0
+#include <sys/types.h>
+#include <sys/console.h>
+#include <sys/vtkd.h>
+# else
+#define KIOC ('K' << 8)
+#define KDMKTONE (KIOC | 8)
+# endif
+
+#define noDEBUG
+
+#include "interp.c"	/* from snd86unx.shr */
+
+static void tone(hz, ticks)
+/* emit tone of frequency hz for given number of ticks */
+unsigned int hz, ticks;
+{
+    ioctl(0,KDMKTONE,hz|((ticks*10)<<16));
+# ifdef DEBUG
+    printf("TONE: %6d %6d\n",hz,ticks * 10);
+# endif
+    nap(ticks * 10);
+}
+
+
+static void rest(ticks)
+/* rest for given number of ticks */
+int	ticks;
+{
+    nap(ticks * 10);
+# ifdef DEBUG
+    printf("REST:        %6d\n",ticks * 10);
+# endif
+}
+
+
+static void
+speaker(instr, buf)
+struct obj *instr;
+char	*buf;
+{
+    /* emit a prefix to modify instrumental `timbre' */
+    playinit();
+    switch (instr->otyp)
+    {
+	case WOODEN_FLUTE:
+	case MAGIC_FLUTE:
+	    playstring(">ol", 1); /* up one octave & lock */
+	    break;
+	case TOOLED_HORN:
+	case FROST_HORN:
+	case FIRE_HORN:
+	    playstring("<<ol", 2); /* drop two octaves & lock */
+	    break;
+	case BUGLE:
+	    playstring("ol", 2); /* octave lock */
+	    break;
+	case WOODEN_HARP:
+	case MAGIC_HARP:
+	    playstring("l8mlol", 4); /* fast, legato, octave lock */
+	    break;
+    }
+    playstring( buf, strlen(buf));
+}
+
+# ifdef DEBUG
+main(argc,argv)
+char *argv[];
+{
+    if (argc == 2) {
+	playinit();
+	playstring(argv[1], strlen(argv[1]));
+    }
+}
+# endif
+#endif	/* VPIX_MUSIC */
+
 /*music.c*/
