@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)polyself.c 3.1	92/11/24
+/*	SCCS Id: @(#)polyself.c 3.1	93/06/24	*/
 /*	Copyright (C) 1987, 1988, 1989 by Ken Arromdee */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -96,7 +96,7 @@ newman()
 	redist_attr();
 	u.uhunger = rn1(500,500);
 	newuhs(FALSE);
-	Sick = 0;
+	if (Sick) make_sick(0L, FALSE);
 	Stoned = 0;
 	if (u.uhp <= 0 || u.uhpmax <= 0) {
 #ifdef POLYSELF
@@ -118,7 +118,8 @@ newman()
 #ifdef POLYSELF
 	polyman();
 #endif
-	You("feel like a new %sman!", flags.female ? "wo" : "");
+	You("feel like a new %s!",
+	    pl_character[0] == 'E' ? "elf" : flags.female ? "woman" : "man");
 	flags.botl = 1;
 	(void) encumber_msg();
 }
@@ -244,7 +245,7 @@ int	mntmp;
 		if(flags.female) dochange = TRUE;
 	} else if (is_female(&mons[mntmp])) {
 		if(!flags.female) dochange = TRUE;
-	} else if (!is_neuter(&mons[mntmp])) {
+	} else if (!is_neuter(&mons[mntmp]) && mntmp != u.ulycn) {
 		if(!rn2(10)) dochange = TRUE;
 	}
 	if (dochange) {
@@ -274,7 +275,7 @@ int	mntmp;
 		You("no longer seem to be petrifying.");
 	}
 	if (u.usym == S_FUNGUS && Sick) {
-		Sick = 0;
+		make_sick(0L, FALSE);
 		You("no longer feel sick.");
 	}
 
@@ -345,7 +346,7 @@ int	mntmp;
 	    if (u.usym == S_UNICORN)
 		pline(use_thec,monsterc,"use your horn");
 	    if (u.umonnum == PM_MIND_FLAYER)
-		pline(use_thec,monsterc,"for a mental blast");
+		pline(use_thec,monsterc,"emit a mental blast");
 	    if (uasmon->msound == MS_SHRIEK) /* worthless, actually */
 		pline(use_thec,monsterc,"shriek");
 	    if ((lays_eggs(uasmon) || u.umonnum==PM_QUEEN_BEE) && flags.female)
@@ -360,6 +361,10 @@ int	mntmp;
 	    u.utrap = 0;
 	    pline("The rock seems to no longer trap you.");
 	}
+	if ((amorphous(uasmon) || is_whirly(uasmon)) && Punished) {
+	    You("slip out of the iron chain.");
+	    unpunish();
+	}
 	flags.botl = 1;
 	vision_full_recalc = 1;
 	exercise(A_CON, FALSE);
@@ -371,17 +376,17 @@ int	mntmp;
 static void
 break_armor()
 {
-     struct obj *otmp;
+    register struct obj *otmp;
 
-     if (breakarm(uasmon)) {
-	if (otmp = uarm) {
+    if (breakarm(uasmon)) {
+	if ((otmp = uarm) != 0) {
 		if (donning(otmp)) cancel_don();
 		You("break out of your armor!");
 		exercise(A_STR, FALSE);
 		(void) Armor_gone();
 		useup(otmp);
 	}
-	if (otmp = uarmc) {
+	if ((otmp = uarmc) != 0) {
 	    if(otmp->oartifact) {
 		Your("cloak falls off!");
 		(void) Cloak_off();
@@ -398,14 +403,14 @@ break_armor()
 		useup(uarmu);
 	}
 #endif
-     } else if (sliparm(uasmon)) {
-	if (otmp = uarm) {
+    } else if (sliparm(uasmon)) {
+	if ((otmp = uarm) != 0) {
 		if (donning(otmp)) cancel_don();
 		Your("armor falls around you!");
 		(void) Armor_gone();
 		dropx(otmp);
 	}
-	if (otmp = uarmc) {
+	if ((otmp = uarmc) != 0) {
 		if (is_whirly(uasmon))
 			Your("cloak falls, unsupported!");
 		else You("shrink out of your cloak!");
@@ -413,7 +418,7 @@ break_armor()
 		dropx(otmp);
 	}
 #ifdef TOURIST
-	if (otmp = uarmu) {
+	if ((otmp = uarmu) != 0) {
 		if (is_whirly(uasmon))
 			You("seep right through your shirt!");
 		else You("become much too small for your shirt!");
@@ -421,37 +426,40 @@ break_armor()
 		dropx(otmp);
 	}
 #endif
-     }
-     if (nohands(uasmon) || verysmall(uasmon)) {
-	  if (otmp = uarmg) {
-	       if (donning(otmp)) cancel_don();
-	       /* Drop weapon along with gloves */
-	       You("drop your gloves%s!", uwep ? " and weapon" : "");
-	       drop_weapon(0);
-	       (void) Gloves_off();
-	       dropx(otmp);
-	  }
-	  if (otmp = uarms) {
-	       You("can no longer hold your shield!");
-	       (void) Shield_off();
-	       dropx(otmp);
-	  }
-	  if (otmp = uarmh) {
-	       if (donning(otmp)) cancel_don();
-	       Your("helmet falls to the floor!");
-	       (void) Helmet_off();
-	       dropx(otmp);
-	  }
-	  if (otmp = uarmf) {
-	       if (donning(otmp)) cancel_don();
-	       if (is_whirly(uasmon))
-		   Your("boots fall away!");
-	       else Your("boots %s off your feet!",
+    }
+    if (nohands(uasmon) || verysmall(uasmon)) {
+	if ((otmp = uarmg) != 0) {
+	    if (donning(otmp)) cancel_don();
+	    /* Drop weapon along with gloves */
+	    You("drop your gloves%s!", uwep ? " and weapon" : "");
+	    drop_weapon(0);
+	    (void) Gloves_off();
+	    dropx(otmp);
+	}
+	if ((otmp = uarms) != 0) {
+	    You("can no longer hold your shield!");
+	    (void) Shield_off();
+	    dropx(otmp);
+	}
+	if ((otmp = uarmh) != 0) {
+	    if (donning(otmp)) cancel_don();
+	    Your("helmet falls to the %s!", surface(u.ux, u.uy));
+	    (void) Helmet_off();
+	    dropx(otmp);
+	}
+    }
+    if (nohands(uasmon) || verysmall(uasmon) || slithy(uasmon) || 
+		u.usym == S_CENTAUR) {
+	if ((otmp = uarmf) != 0) {
+	    if (donning(otmp)) cancel_don();
+	    if (is_whirly(uasmon))
+		Your("boots fall away!");
+	    else Your("boots %s off your feet!",
 			verysmall(uasmon) ? "slide" : "are pushed");
-	       (void) Boots_off();
-	       dropx(otmp);
-	  }
-     }
+	    (void) Boots_off();
+	    dropx(otmp);
+	}
+    }
 }
 
 static void
@@ -459,7 +467,7 @@ drop_weapon(alone)
 int alone;
 {
      struct obj *otmp;
-     if (otmp = uwep) {
+     if ((otmp = uwep) != 0) {
 	  /* !alone check below is currently superfluous but in the
 	   * future it might not be so if there are monsters which cannot
 	   * wear gloves but can wield weapons
@@ -587,7 +595,7 @@ dospinweb()
 		case PIT:
 		case SPIKED_PIT: You("spin a web, covering up the pit.");
 			deltrap(ttmp);
-			delallobj(u.ux, u.uy);
+			bury_objs(u.ux, u.uy);
 			if (Invisible) newsym(u.ux, u.uy);
 			return(1);
 		case SQKY_BOARD: pline("The squeaky board is muffled.");
@@ -596,15 +604,16 @@ dospinweb()
 			return(1);
 		case TELEP_TRAP:
 		case LEVEL_TELEP:
+		case MAGIC_PORTAL:
 			Your("webbing vanishes!");
 			return(0);
 		case WEB: You("make the web thicker.");
 			return(1);
 		case TRAPDOOR:
-		    You("web over the trap door.");
-		    deltrap(ttmp);
-		    if (Invisible) newsym(u.ux, u.uy);
-		    return 1;
+			You("web over the trap door.");
+			deltrap(ttmp);
+			if (Invisible) newsym(u.ux, u.uy);
+			return 1;
 		case ARROW_TRAP:
 		case DART_TRAP:
 		case BEAR_TRAP:
@@ -622,7 +631,7 @@ dospinweb()
 			return(0);
 	}
 	ttmp = maketrap(u.ux, u.uy, WEB);
-	ttmp->tseen = 1;
+	if (ttmp) ttmp->tseen = 1;
 	if (Invisible) newsym(u.ux, u.uy);
 	return(1);
 }
@@ -675,6 +684,9 @@ doconfuse()
 		    if (!mtmp->mcanmove || mtmp->mstun || mtmp->msleep ||
 					!mtmp->mcansee || !haseyes(mtmp->data))
 			continue;
+#ifdef MUSE
+		    if (!mon_reflects(mtmp, "Your gaze is reflected by %s %s."))
+#endif
 		    if (!mtmp->mconf)
 			Your("gaze confuses %s!", mon_nam(mtmp));
 		    else
@@ -777,33 +789,33 @@ int part;
 	 * plus the trailing null, after pluralizing (since sometimes a
 	 * buffer is made a fixed size and must be able to hold it)
 	 */
-	static const char NEARDATA *humanoid_parts[] = { "arm", "eye", "face", "finger",
+	static NEARDATA const char *humanoid_parts[] = { "arm", "eye", "face", "finger",
 		"fingertip", "foot", "hand", "handed", "head", "leg",
 		"light headed", "neck", "spine", "toe" };
 #ifdef POLYSELF
-	static const char NEARDATA *jelly_parts[] = { "pseudopod", "dark spot", "front",
+	static NEARDATA const char *jelly_parts[] = { "pseudopod", "dark spot", "front",
 		"pseudopod extension", "pseudopod extremity",
 		"pseudopod root", "grasp", "grasped", "cerebral area",
 		"lower pseudopod", "viscous", "middle", "surface",
 		"pseudopod extremity" },
-	NEARDATA *animal_parts[] = { "forelimb", "eye", "face", "foreclaw", "claw tip",
+	*animal_parts[] = { "forelimb", "eye", "face", "foreclaw", "claw tip",
 		"rear claw", "foreclaw", "clawed", "head", "rear limb",
 		"light headed", "neck", "spine", "rear claw tip" },
-	NEARDATA *horse_parts[] = { "forelimb", "eye", "face", "forehoof", "hoof tip",
+	*horse_parts[] = { "forelimb", "eye", "face", "forehoof", "hoof tip",
 		"rear hoof", "foreclaw", "hooved", "head", "rear limb",
 		"light headed", "neck", "backbone", "rear hoof tip" },
-	NEARDATA *sphere_parts[] = { "appendage", "optic nerve", "body", "tentacle",
+	*sphere_parts[] = { "appendage", "optic nerve", "body", "tentacle",
 		"tentacle tip", "lower appendage", "tentacle", "tentacled",
 		"body", "lower tentacle", "rotational", "equator", "body",
 		"lower tentacle tip" },
-	NEARDATA *fungus_parts[] = { "mycelium", "visual area", "front", "hypha",
+	*fungus_parts[] = { "mycelium", "visual area", "front", "hypha",
 		"hypha", "root", "strand", "stranded", "cap area",
 		"rhizome", "sporulated", "stalk", "root", "rhizome tip" },
-	NEARDATA *vortex_parts[] = { "region", "eye", "front", "minor current",
+	*vortex_parts[] = { "region", "eye", "front", "minor current",
 		"minor current", "lower current", "swirl", "swirled",
 		"central core", "lower current", "addled", "center",
 		"currents", "edge" },
-	NEARDATA *snake_parts[] = { "vestigial limb", "eye", "face", "large scale",
+	*snake_parts[] = { "vestigial limb", "eye", "face", "large scale",
 		"large scale tip", "rear region", "scale gap", "scale gapped",
 		"head", "rear region", "light headed", "neck", "length",
 		"rear scale" };
