@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)potion.c	3.1	92/12/10		  */
+/*	SCCS Id: @(#)potion.c	3.1	93/07/07	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -8,7 +8,7 @@
 static void NDECL(ghost_from_bottle);
 static boolean FDECL(neutralizes, (struct obj *,struct obj *));
 
-static int NEARDATA nothing, NEARDATA unkn;
+static NEARDATA int nothing, unkn;
 #endif /* OVLB */
 
 extern boolean notonhead;	/* for long worms */
@@ -18,7 +18,7 @@ boolean notonhead = FALSE;
 
 #ifdef OVLB
 
-static const char NEARDATA beverages[] = { POTION_CLASS, 0 };
+static NEARDATA const char beverages[] = { POTION_CLASS, 0 };
 
 void
 make_confused(xtime,talk)
@@ -55,7 +55,7 @@ boolean talk;
 	}
 	if (xtime && !old) {
 		if (talk)
-			You("stagger....");
+			You("stagger...");
 		flags.botl = 1;
 	}
 	HStun = xtime;
@@ -310,8 +310,8 @@ peffects(otmp)
 	case POT_WATER:
 		if(!otmp->blessed && !otmp->cursed) {
 			pline("This tastes like %swater.",
-			      otmp->spe == -1 ? "impure " : "");
-			lesshungry(rnd(otmp->spe == -1 ? 3 : 10));
+			      otmp->odiluted ? "impure " : "");
+			lesshungry(rnd(otmp->odiluted ? 3 : 10));
 			break;
 		}
 		unkn++;
@@ -365,11 +365,13 @@ peffects(otmp)
 		break;
 	case POT_BOOZE:
 		unkn++;
-		pline("Ooph!  This tastes like %s!",
+		pline("Ooph!  This tastes like %s%s!",
+		      otmp->odiluted ? "watered down " : "",
 		      Hallucination ? "furniture polish" : "liquid fire");
 		if (!otmp->blessed) make_confused(HConfusion + d(3,8),FALSE);
 		/* the whiskey makes us feel better */
-		if(u.uhp < u.uhpmax) losehp(-1, "", 0); /* can't kill you */
+		if (u.uhp < u.uhpmax && !otmp->odiluted)
+			losehp(-1, "", 0); /* can't kill you */
 		lesshungry(10 * (2 + bcsign(otmp)));
 		exercise(A_WIS, FALSE);
 		if(otmp->cursed) {
@@ -429,14 +431,16 @@ peffects(otmp)
 		else pline (Hallucination ?
 #ifdef TUTTI_FRUTTI
 		   "This tastes like 10%% real %s juice all-natural beverage." :
-		   "This tastes like %s juice.", pl_fruit
+		   "This tastes like %s%s juice.",
+		   otmp->odiluted ? "reconstituted " : "", pl_fruit
 #else
 		   "This tastes like 10%% real fruit juice all-natural beverage." :
-		   "This tastes like fruit juice."
+		   "This tastes like %sfruit juice.",
+		   otmp->odiluted ? "reconstituted " : ""
 #endif
 			    );
 		if (otmp->otyp == POT_FRUIT_JUICE) {
-			lesshungry(10 * (2 + bcsign(otmp)));
+			lesshungry((otmp->odiluted ? 5 : 10) * (2 + bcsign(otmp)));
 			break;
 		}
 		if (!otmp->cursed) {
@@ -454,11 +458,11 @@ peffects(otmp)
 		newsym(u.ux,u.uy); /* see yourself! */
 		break;
 	case POT_PARALYSIS:
-		if(Levitation || Is_waterlevel(&u.uz))
-			You("are motionlessly suspended.");
+		if (Levitation || Is_airlevel(&u.uz) || Is_waterlevel(&u.uz))
+		    You("are motionlessly suspended.");
 		else
-			Your("%s are frozen to the floor!",
-				makeplural(body_part(FOOT)));
+		    Your("%s are frozen to the %s!",
+			 makeplural(body_part(FOOT)), surface(u.ux, u.uy));
 		nomul(-(rn1(10, 25 - 12*bcsign(otmp))));
 		exercise(A_DEX, FALSE);
 		break;
@@ -639,7 +643,6 @@ peffects(otmp)
 			} while (lmoves < 1 || lmoves > 300);
 			HLevitation += lmoves;
 		} else HLevitation += rnd(150);
-		u.uprops[LEVITATION].p_tofn = float_down;
 		break;
 	case POT_GAIN_ENERGY:			/* M. Stephenson */
 		{	register int	 num;
@@ -852,7 +855,7 @@ void
 potionbreathe(obj)
 register struct obj *obj;
 {
-	register int i, ii, isdone;
+	register int i, ii, isdone, kn = 0;
 
 	switch(obj->otyp) {
 	case POT_RESTORE_ABILITY:
@@ -886,7 +889,7 @@ register struct obj *obj;
 		}
 		break;
 	case POT_HALLUCINATION:
-		You("have a vision for a moment.");
+		You("have a momentary vision.");
 		break;
 	case POT_CONFUSION:
 	case POT_BOOZE:
@@ -899,24 +902,28 @@ register struct obj *obj;
 			pline("For an instant you could see through yourself!");
 		break;
 	case POT_PARALYSIS:
+		kn++;
 		pline("Something seems to be holding you.");
 		nomul(-rnd(5));
 		exercise(A_DEX, FALSE);
 		break;
 	case POT_SPEED:
+		if (!Fast) Your("knees seem more flexible now.");
 		Fast += rnd(5);
-		Your("knees seem more flexible now.");
 		exercise(A_DEX, TRUE);
 		break;
 	case POT_BLINDNESS:
-		if (!Blind && !u.usleep) pline("It suddenly gets dark.");
-		make_blinded(Blinded + rnd(5),FALSE);
+		if (!Blind && !u.usleep) {
+		    kn++;
+		    pline("It suddenly gets dark.");
+		}
+		make_blinded(Blinded + rnd(5), FALSE);
 		break;
 	case POT_WATER:
 #ifdef POLYSELF
 		if(u.umonnum == PM_GREMLIN) {
 		    struct monst *mtmp;
-		    if(mtmp = cloneu()) {
+		    if ((mtmp = cloneu()) != 0) {
 			mtmp->mhpmax = (u.mhmax /= 2);
 			You("multiply.");
 		    }
@@ -932,7 +939,10 @@ register struct obj *obj;
 		break;
 	}
 	/* note: no obfree() */
-	if (obj->dknown && !objects[obj->otyp].oc_name_known &&
+	if (obj->dknown)
+	    if (kn)
+		makeknown(obj->otyp);
+	    else if (!objects[obj->otyp].oc_name_known &&
 						!objects[obj->otyp].oc_uname)
 		docall(obj);
 }
@@ -985,11 +995,11 @@ register struct obj *obj;
 	    case POTION_CLASS:
 		if (obj->otyp == POT_WATER) return FALSE;
 		Your("%s.", aobjnam(obj,"dilute"));
-		if (obj->spe == -1) {
-			obj->spe = 0;
+		if (obj->odiluted) {
+			obj->odiluted = 0;
 			obj->blessed = obj->cursed = FALSE;
 			obj->otyp = POT_WATER;
-		} else obj->spe--;
+		} else obj->odiluted++;
 		return TRUE;
 	    case SCROLL_CLASS:
 		if (obj->otyp != SCR_BLANK_PAPER
@@ -1139,7 +1149,7 @@ dodip()
 		if (Blind) obj->dknown = 0;
 
 		switch (neutralizes(obj, potion) ||
-			obj->spe == -1 /* diluted */ ? 1 : rnd(8)) {
+			obj->odiluted ? 1 : rnd(8)) {
 			case 1:
 				obj->otyp = POT_WATER;
 				obj->blessed = obj->cursed = 0;
@@ -1165,11 +1175,11 @@ dodip()
 		}
 
 		if (obj->otyp == POT_WATER) {
-			obj->spe = 0; /* in case it was diluted before */
+			obj->odiluted = 0; /* in case it was diluted before */
 			pline("The mixture bubbles violently%s.",
 				Blind ? "" : ", then clears");
 		} else {
-			obj->spe--; /* diluted */
+			obj->odiluted++;
 			if (!Blind) {
 				pline("The mixture looks %s.",
 					OBJ_DESCR(objects[obj->otyp]));
@@ -1208,7 +1218,7 @@ dodip()
 		potion->otyp = POT_WATER;
 		potion->blessed = 0;
 		potion->cursed = 0;
-		potion->spe = 0;
+		potion->odiluted = 0;
 		return(1);
 	}
 
