@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)priest.c	3.1	92/01/05
+/*	SCCS Id: @(#)priest.c	3.1	93/05/15	*/
 /* Copyright (c) Izchak Miller, Steve Linhart, 1989. 		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -132,8 +132,8 @@ histemple_at(priest, x, y)
 register struct monst *priest;
 register xchar x, y;
 {
-	return((EPRI(priest)->shroom == *in_rooms(x, y, TEMPLE)) &&
-	       on_level(&(EPRI(priest)->shrlevel), &u.uz));
+	return((boolean)((EPRI(priest)->shroom == *in_rooms(x, y, TEMPLE)) &&
+	       on_level(&(EPRI(priest)->shrlevel), &u.uz)));
 }
 
 /*
@@ -196,8 +196,9 @@ boolean sanctum;   /* is it the seat of the high priest? */
 	if(MON_AT(sx+1, sy))
 		rloc(m_at(sx+1, sy)); /* insurance */
 
-	if(priest = (sanctum ? makemon(&mons[PM_HIGH_PRIEST], sx+1, sy)
-			     : makemon(&mons[PM_ALIGNED_PRIEST], sx+1, sy))) {
+	priest = (sanctum ? makemon(&mons[PM_HIGH_PRIEST], sx+1, sy)
+			  : makemon(&mons[PM_ALIGNED_PRIEST], sx+1, sy));
+	if (priest) {
 		EPRI(priest)->shroom = (sroom - rooms) + ROOMOFFSET;
 		EPRI(priest)->shralign = Amask2align(levl[sx][sy].altarmask);
 		EPRI(priest)->shrpos.x = sx;
@@ -265,7 +266,7 @@ char *
 priestname(mon)
 register struct monst *mon;
 {
-	static char NEARDATA pname[PL_NSIZ];
+	static NEARDATA char pname[PL_NSIZ];
 
 	Strcpy(pname, "the ");
 	if (mon->minvis) Strcat(pname, "invisible ");
@@ -304,7 +305,7 @@ boolean
 p_coaligned(priest)
 struct monst *priest;
 {
-	return(u.ualign.type == ((int)EPRI(priest)->shralign));
+	return((boolean)(u.ualign.type == ((int)EPRI(priest)->shralign)));
 }
 
 static boolean
@@ -318,7 +319,7 @@ struct monst *pri;
 	lev = &levl[EPRI(pri)->shrpos.x][EPRI(pri)->shrpos.y];
 	if (!IS_ALTAR(lev->typ) || !(lev->altarmask & AM_SHRINE))
 		return(FALSE);
-	return(EPRI(pri)->shralign == Amask2align(lev->altarmask & ~AM_SHRINE));
+	return((boolean)(EPRI(pri)->shralign == Amask2align(lev->altarmask & ~AM_SHRINE)));
 }
 
 struct monst *
@@ -377,7 +378,8 @@ register int roomno;
 		switch(rn2(3)) {
 		  case 0: You("have an eerie feeling..."); break;
 		  case 1: You("feel like you are being watched."); break;
-		  default: pline("A shiver runs down your spine."); break;
+		  default: pline("A shiver runs down your %s.",
+			body_part(SPINE)); break;
 		}
 		if(!rn2(5)) {
 		    struct monst *mtmp;
@@ -414,10 +416,7 @@ register struct monst *priest;
 		 !priest->mpeaceful || !priest->mcanmove || priest->msleep) {
 	    if(!priest->mcanmove || priest->msleep) {
 		pline("%s breaks out of %s reverie!",
-			humanoid(priest->data)
-				? (priest->female ? "her" : "his")
-				: "its",
-			Monnam(priest));
+		      Monnam(priest), his[pronoun_gender(priest)]);
 		priest->mfrozen = priest->msleep = 0;
 		priest->mcanmove = 1;
 	    }
@@ -430,7 +429,7 @@ register struct monst *priest;
 		   verbalize("Talk?  Here is what I have to say!");
 		   break;
 		default:
-		   verbalize("Pilgrim, I have lost mine desire to talk.");
+		   verbalize("Pilgrim, I would speak no longer with thee.");
 		   break;
 	    }
 	    return;
@@ -484,7 +483,9 @@ register struct monst *priest;
 		    verbalize("I bestow upon thee a blessing.");
 		    Clairvoyant += rn1(500,500);
 		}
-	    } else if(offer < (u.ulevel * 600)) {
+	    } else if(offer < (u.ulevel * 600) &&
+		      u.ublessed < 20 &&
+		      (u.ublessed < 9 || !rn2(u.ublessed))) {
 		verbalize("Thy devotion has been rewarded.");
 		if (!(Protection & INTRINSIC))  {
 			Protection |= FROMOUTSIDE;
@@ -494,7 +495,7 @@ register struct monst *priest;
 		verbalize("Thy selfless generosity is deeply appreciated.");
 		if(u.ugold < (offer * 2L) && coaligned) {
 		    if(strayed && (moves - u.ucleansed) > 5000L) {
-			u.ualign.record = 0; /* cleanse him */
+			u.ualign.record = 0; /* cleanse thee */
 			u.ucleansed = moves;
 		    } else {
 			u.ualign.record += 2;
@@ -562,7 +563,7 @@ xchar x, y;
 	    (roomno != *in_rooms(x, y, TEMPLE)) ||
 	    !(priest = findpriest(roomno)))
 		return(FALSE);
-	return(has_shrine(priest) && p_coaligned(priest) && priest->mpeaceful);
+	return((boolean)(has_shrine(priest) && p_coaligned(priest) && priest->mpeaceful));
 }
 
 void
@@ -630,10 +631,38 @@ angry_priest()
 {
 	register struct monst *priest;
 
-	if(priest = findpriest(temple_occupied(u.urooms)))
+	if ((priest = findpriest(temple_occupied(u.urooms))) != 0)
 		wakeup(priest);
 }
 
+/*
+ * When saving bones, find priests that aren't on their shrine level,
+ * and remove them.   This avoids big problems when restoring bones.
+ */
+void
+clearpriests()
+{
+    register struct monst *mtmp, *mtmp2;
+
+    for(mtmp = fmon; mtmp; mtmp = mtmp2) {
+	mtmp2 = mtmp->nmon;
+	if (mtmp->ispriest && !on_level(&(EPRI(mtmp)->shrlevel), &u.uz))
+	    mongone(mtmp);
+    }
+}
+
+/* munge priest-specific structure when restoring -dlc */
+void
+restpriest(mtmp, ghostly)
+register struct monst *mtmp;
+boolean ghostly;
+{
+    if(u.uz.dlevel) {
+	if (ghostly)
+	    assign_level(&(EPRI(mtmp)->shrlevel), &u.uz);
+    }
+}
+
 #endif /* OVLB */
 
 /*priest.c*/
