@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)restore.c	3.1	93/01/23	*/
+/*	SCCS Id: @(#)restore.c	3.1	93/04/06	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -25,15 +25,19 @@ static void FDECL(restgenoinfo, (int));
 static boolean FDECL(restgamestate, (int, unsigned int *));
 static int FDECL(restlevelfile, (int,XCHAR_P));
 
+#ifdef AMII_GRAPHICS
+void NDECL( amii_setpens );		/* use colors from save file */
+#endif
+
 #ifdef MULDGN
 #include "quest.h"
 #endif
 
 boolean restoring = FALSE;
 #ifdef TUTTI_FRUTTI
-static struct fruit NEARDATA *oldfruit;
+static NEARDATA struct fruit *oldfruit;
 #endif
-static long NEARDATA omoves;
+static NEARDATA long omoves;
 
 /* Recalculate level.objects[x][y], since this info was not saved. */
 static void
@@ -162,17 +166,13 @@ restobjchn(fd, ghostly)
 register int fd;
 boolean ghostly;
 {
-	register struct obj *otmp, *otmp2;
+	register struct obj *otmp, *otmp2 = 0;
 	register struct obj *first = (struct obj *)0;
 #ifdef TUTTI_FRUTTI
 	register struct fruit *oldf;
 #endif
 	int xl;
 
-#if defined(LINT) || defined(GCC_WARN)
-	/* suppress "used before set" warning from lint */
-	otmp2 = 0;
-#endif
 	while(1) {
 		mread(fd, (genericptr_t) &xl, sizeof(xl));
 		if(xl == -1) break;
@@ -200,8 +200,8 @@ boolean ghostly;
 				&& !Is_candle(otmp))
 			otmp->age = monstermoves-omoves+otmp->age;
 
-		/* get contents of the container */
-		if (Is_container(otmp) || otmp->otyp == STATUE)
+		/* get contents of a container or statue */
+		if (Has_contents(otmp))
 		    otmp->cobj = restobjchn(fd,ghostly);
 
 		otmp2 = otmp;
@@ -219,7 +219,7 @@ restmonchn(fd, ghostly)
 register int fd;
 boolean ghostly;
 {
-	register struct monst *mtmp, *mtmp2;
+	register struct monst *mtmp, *mtmp2 = 0;
 	register struct monst *first = (struct monst *)0;
 	int xl;
 	struct permonst *monbegin;
@@ -229,10 +229,6 @@ boolean ghostly;
 	mread(fd, (genericptr_t)&monbegin, sizeof(monbegin));
 	moved = (monbegin != mons);
 
-#if defined(LINT) || defined(GCC_WARN)
-	/* suppress "used before set" warning from lint */
-	mtmp2 = 0;
-#endif
 	while(1) {
 		mread(fd, (genericptr_t) &xl, sizeof(xl));
 		if(xl == -1) break;
@@ -249,9 +245,20 @@ boolean ghostly;
 		if(mtmp->minvent)
 			mtmp->minvent = restobjchn(fd, ghostly);
 #ifdef MUSE
-		if (mtmp->mw) mtmp->mw = mtmp->minvent;	/* wield 1st obj in inventory */
+		if (mtmp->mw) {
+			struct obj *obj;
+
+			for(obj = mtmp->minvent; obj; obj = obj->nobj)
+				if (obj->owornmask & W_WEP) break;
+			if (obj) mtmp->mw = obj;
+			else {
+				MON_NOWEP(mtmp);
+				impossible("bad monster weapon restore");
+			}
+		}
 #endif
-		if (mtmp->isshk) restshk(mtmp);
+		if (mtmp->isshk) restshk(mtmp, ghostly);
+		if (mtmp->ispriest) restpriest(mtmp, ghostly);
 
 		mtmp2 = mtmp;
 	}
@@ -330,7 +337,12 @@ unsigned int *mid;
 	flags.msg_history = oldflags.msg_history;
 	flags.echo = oldflags.echo;
 	flags.cbreak = oldflags.cbreak;
-
+#ifdef NEWS
+	flags.news = oldflags.news;
+#endif
+#ifdef AMII_GRAPHICS
+	amii_setpens();		/* use colors from save file */
+#endif
 	mread(fd, (genericptr_t) &u, sizeof(struct you));
 	if(u.uhp <= 0) {
 	    You("were not healthy enough to survive restoration.");
@@ -386,6 +398,9 @@ static int
 restlevelfile(fd, ltmp)
 register int fd;
 xchar ltmp;
+#ifdef applec
+# pragma unused(fd)
+#endif
 {
 	register int nfd;
 
@@ -452,6 +467,9 @@ register int fd;
 		restoring = FALSE;
 		return(0);
 	}
+#ifdef MULDGN
+	quest_init();
+#endif
 #ifdef INSURANCE
 	savestateinlock();
 #endif
@@ -459,19 +477,22 @@ register int fd;
 	if (rtmp < 2) return(rtmp);  /* dorecover called recursively */
 
 #ifdef MICRO
-# ifdef AMIGA
+# ifdef AMII_GRAPHICS
 	{
-	extern winid WIN_BASE;
-	clear_nhwindow(WIN_BASE);	/* hack until there's a hook for this */
+	extern struct window_procs amii_procs;
+	if(windowprocs.win_init_nhwindows== amii_procs.win_init_nhwindows){
+	    extern winid WIN_BASE;
+	    clear_nhwindow(WIN_BASE);	/* hack until there's a hook for this */
+	}
 	}
 # else
 	clear_nhwindow(WIN_MAP);
 # endif
 	clear_nhwindow(WIN_MESSAGE);
-	You("got as far as level %d in %s%s.",
+	You("return to level %d in %s%s.",
 		depth(&u.uz), dungeons[u.uz.dnum].dname,
-		flags.debug ? " while in WIZARD mode" :
-		flags.explore ? " while in discovery mode" : "");
+		flags.debug ? " while in debug mode" :
+		flags.explore ? " while in explore mode" : "");
 	curs(WIN_MAP, 1, 1);
 	dotcnt = 0;
 	putstr(WIN_MAP, 0, "Restoring:");
@@ -485,8 +506,9 @@ register int fd;
 			break;
 		getlev(fd, 0, ltmp, FALSE);
 #ifdef MICRO
-		curs(WIN_MAP, 11 + dotcnt++, 1);
+		curs(WIN_MAP, 1+dotcnt++, 2);
 		putstr(WIN_MAP, 0, ".");
+		mark_synch();
 #endif
 		rtmp = restlevelfile(fd, ltmp);
 		if (rtmp < 2) return(rtmp);  /* dorecover called recursively */
@@ -549,6 +571,7 @@ register int fd;
 #ifdef MULDGN
 	load_qtlist();	/* re-load the quest text info */
 #endif
+	reset_attribute_clock();
 	/* Set up the vision internals, after levl[] data is loaded */
 	/* but before docrt().					    */
 	vision_reset();
@@ -632,6 +655,9 @@ boolean ghostly;
 		uchar	len;
 		struct rm r;
 		
+#if defined(MAC)
+		memset ( & r , 0 , sizeof ( r ) ) ; /* Suppress warning about used before set */
+#endif
 		i = 0; j = 0; len = 0;
 		while(i < ROWNO) {
 		    while(j < COLNO) {
@@ -750,6 +776,7 @@ boolean ghostly;
 	dealloc_trap(trap);
 	fobj = restobjchn(fd, ghostly);
 	find_lev_obj();
+	level.buriedobjlist = restobjchn(fd, ghostly);
 	billobjs = restobjchn(fd, ghostly);
 	rest_engravings(fd);
 	rest_rooms(fd);		/* No joke :-) */
@@ -814,6 +841,14 @@ boolean ghostly;
 		}
 		break;
 	    }
+	} else if (ghostly && !Is_branchlev(&u.uz)) {
+	    /* Make sure there are no dangling portals.  If so, remove them */
+	    register struct trap *ttmp;
+	    for(ttmp = ftrap; ttmp; ttmp = ttmp->ntrap)
+		if (ttmp->ttyp == MAGIC_PORTAL) {
+		    deltrap(ttmp);
+		    break; /* max of 1 portal/level */
+		}
 	}
 }
 
@@ -823,11 +858,11 @@ boolean ghostly;
 #ifndef ZEROCOMP_BUFSIZ
 #define ZEROCOMP_BUFSIZ BUFSZ
 #endif
-static unsigned char NEARDATA inbuf[ZEROCOMP_BUFSIZ];
-static unsigned short NEARDATA inbufp = 0;
-static unsigned short NEARDATA inbufsz = 0;
-static short NEARDATA inrunlength = -1;
-static int NEARDATA mreadfd;
+static NEARDATA unsigned char inbuf[ZEROCOMP_BUFSIZ];
+static NEARDATA unsigned short inbufp = 0;
+static NEARDATA unsigned short inbufsz = 0;
+static NEARDATA short inrunlength = -1;
+static NEARDATA int mreadfd;
 
 static int
 mgetc()
