@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)save.c	3.1	93/01/07	*/
+/*	SCCS Id: @(#)save.c	3.1	93/06/27	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -13,6 +13,9 @@
 #endif /* EXPLORE_MODE */
 
 boolean hu;		/* set during hang-up */
+#ifdef VMS
+extern volatile int exiting;		/* sys/vms/vmsmain.c */
+#endif
 
 #ifdef MULDGN
 #include "quest.h"
@@ -31,7 +34,7 @@ int dotcnt;	/* also used in restore */
 #endif
 
 #ifdef ZEROCOMP
-static void FDECL(bputc, (UCHAR_P));
+static void FDECL(bputc, (int));
 #endif
 static void FDECL(savelevchn, (int, int));
 static void FDECL(savedamage, (int,struct damage *, int));
@@ -61,7 +64,6 @@ dosave()
 	} else {
 		clear_nhwindow(WIN_MESSAGE);
 		pline("Saving...");
-		mark_synch();	/* flush output */
 		hu = FALSE;
 		if(dosave0()) {
 			/* make sure they see the Saving message */
@@ -75,13 +77,17 @@ dosave()
 
 #ifndef NOSAVEONHANGUP
 int
-hangup() {
+hangup()
+{
 	if(!hu) {
 		hu = TRUE;
 		(void) dosave0();
-# ifndef VMS
-		terminate(1);
+# ifdef VMS
+		/* don't call exit when already within an exit handler;
+		   that would cancel any other pending user-mode handlers */
+		if (!exiting)
 # endif
+		terminate(1);
 	}
 	return 0;
 }
@@ -93,6 +99,7 @@ dosave0()
 {
 	register int fd, ofd;
 	xchar ltmp;
+	d_level uz_save;
 #ifdef MFLOPPY
 	long fds, needed;
 #endif
@@ -122,7 +129,8 @@ dosave0()
 	    }
 	}
 #endif
-	
+	if (!hu) mark_synch();	/* flush any buffered screen output */
+
 	fd = create_savefile();
 
 	if(fd < 0) {
@@ -137,12 +145,14 @@ dosave0()
 	if(flags.window_inited)
 	    clear_nhwindow(WIN_MESSAGE);
 
-#ifdef MFLOPPY
+#ifdef MICRO
 	if(!hu) {
 	    dotcnt = 0;
 	    curs(WIN_MAP, 1, 1);
 	    putstr(WIN_MAP, 0, "Saving:");
 	}
+#endif
+#ifdef MFLOPPY
 	/* make sure there is enough disk space */
 	savelev(fd, ledger_no(&u.uz), COUNT_SAVE);
 	savegamestate(fd, COUNT_SAVE);
@@ -170,8 +180,16 @@ dosave0()
 	savelev(fd, ledger_no(&u.uz), WRITE_SAVE | FREE_SAVE);
 	savegamestate(fd, WRITE_SAVE | FREE_SAVE);
 
+	/* While copying level files around, zero out u.uz to keep
+	 * parts of the restore code from completely initializing all
+	 * in-core data structures, since all we're doing is copying.
+	 * This also avoids at least one nasty core dump.
+	 */
+	uz_save = u.uz;
+	u.uz.dnum = u.uz.dlevel = 0;
+
 	for(ltmp = (xchar)1; ltmp <= maxledgerno(); ltmp++) {
-		if (ltmp == ledger_no(&u.uz)) continue;
+		if (ltmp == ledger_no(&uz_save)) continue;
 #ifdef MFLOPPY
 		if (!fileinfo[ltmp].where) continue;
 #else
@@ -179,8 +197,9 @@ dosave0()
 #endif
 #ifdef MICRO
 		if(!hu) {
-		    curs(WIN_MAP, 8 + dotcnt++, 1);
+		    curs(WIN_MAP, 1 + dotcnt++, 2);
 		    putstr(WIN_MAP, 0, ".");
+		    mark_synch();
 		}
 #endif
 		ofd = open_levelfile(ltmp);
@@ -200,6 +219,8 @@ dosave0()
 	}
 	bclose(fd);
 
+	u.uz = uz_save;
+
 	/* get rid of current level --jgm */
 	delete_levelfile(ledger_no(&u.uz));
 	delete_levelfile(0);
@@ -371,7 +392,7 @@ int mode;
 	    struct rm *prm, *rgrm;
 	    int x, y;
 	    uchar match;
-	    
+
 	    rgrm = &levl[0][0];		/* start matching at first rm */
 	    match = 0;
 
@@ -400,7 +421,7 @@ int mode;
 			bwrite(fd, (genericptr_t)rgrm, sizeof(struct rm));
 			/* start encoding again. we have at least 1 rm
 			 * in the next run, viz. this one. */
-			match = 1;	
+			match = 1;
 			rgrm = prm;
 		    }
 		}
@@ -427,6 +448,7 @@ int mode;
 	save_worm(fd, mode);	/* save worm information */
 	savetrapchn(fd, ftrap, mode);
 	saveobjchn(fd, fobj, mode);
+	saveobjchn(fd, level.buriedobjlist, mode);
 	saveobjchn(fd, billobjs, mode);
 
 	save_engravings(fd, mode);
@@ -455,12 +477,12 @@ int mode;
 #define flushoutrun(ln) (bputc(RLESC), bputc(ln), ln = -1)
 
 #ifndef ZEROCOMP_BUFSIZ
-#define ZEROCOMP_BUFSIZ BUFSZ
+# define ZEROCOMP_BUFSIZ BUFSZ
 #endif
-static unsigned char NEARDATA outbuf[ZEROCOMP_BUFSIZ];
-static unsigned short NEARDATA outbufp = 0;
-static short NEARDATA outrunlength = -1;
-static int NEARDATA bwritefd;
+static NEARDATA unsigned char outbuf[ZEROCOMP_BUFSIZ];
+static NEARDATA unsigned short outbufp = 0;
+static NEARDATA short outrunlength = -1;
+static NEARDATA int bwritefd;
 
 /*dbg()
 {
@@ -469,7 +491,7 @@ static int NEARDATA bwritefd;
 
 static void
 bputc(c)
-unsigned char c;
+int c;
 {
 #ifdef MFLOPPY
     bytes_counted++;
@@ -480,13 +502,16 @@ unsigned char c;
 	(void) write(bwritefd, outbuf, sizeof outbuf);
 	outbufp = 0;
     }
-    outbuf[outbufp++] = c;
+    outbuf[outbufp++] = (unsigned char)c;
 }
 
 /*ARGSUSED*/
 void
 bufon(fd)
     int fd;
+#if defined(applec)
+# pragma unused(fd)
+#endif
 {
     return;
 }
@@ -511,23 +536,25 @@ register int fd;
 
 void
 bwrite(fd, loc, num)
-register int fd;
+int fd;
 genericptr_t loc;
 register unsigned num;
 {
-      bwritefd = fd;
-      for (; num; num--, (*(char **)&loc)++) {
-	      if (*((char *)loc) == RLESC) { /* One more char in run */
-		  if (++outrunlength == 0xFF) {
-		      flushoutrun(outrunlength);
-		  }
-	      } else { /* end of run */
-		  if (outrunlength >= 0) {    /* flush run */
-		      flushoutrun(outrunlength);
-		  }
-		  bputc(*((char *)loc));
-	      }
-      }
+	register unsigned char *bp = (unsigned char *)loc;
+
+	bwritefd = fd;
+	for (; num; num--, bp++) {
+	    if (*bp == RLESC) {	/* One more char in run */
+		if (++outrunlength == 0xFF) {
+		    flushoutrun(outrunlength);
+		}
+	    } else {		/* end of run */
+		if (outrunlength >= 0) {	/* flush run */
+		    flushoutrun(outrunlength);
+		}
+		bputc(*bp);
+	    }
+	}
 }
 
 void
@@ -674,10 +701,12 @@ register struct obj *otmp;
 	    bwrite(fd, (genericptr_t) &xl, sizeof(int));
 	    bwrite(fd, (genericptr_t) otmp, xl + sizeof(struct obj));
 
-	    if (Is_container(otmp) || otmp->otyp == STATUE)
+	    if (Has_contents(otmp))
 		saveobjchn(fd,otmp->cobj,mode);
-	    if (mode & FREE_SAVE)
+	    if (mode & FREE_SAVE) {
+		if(otmp->oclass == FOOD_CLASS) food_disappears(otmp);
 		dealloc_obj(otmp);
+	    }
 	    otmp = otmp2;
 	}
 	bwrite(fd, (genericptr_t) &minusone, sizeof(int));
@@ -697,9 +726,6 @@ register struct monst *mtmp;
 
 	while(mtmp) {
 		mtmp2 = mtmp->nmon;
-#ifdef MUSE
-		if (mtmp->mw && mtmp->mw != mtmp->minvent) sort_mwep(mtmp);
-#endif
 		xl = mtmp->mxlth + mtmp->mnamelth;
 		bwrite(fd, (genericptr_t) &xl, sizeof(int));
 		bwrite(fd, (genericptr_t) mtmp, xl + sizeof(struct monst));
