@@ -1,8 +1,8 @@
-/*	SCCS Id: @(#)trap.c	3.1	92/12/10	*/
+/*	SCCS Id: @(#)trap.c	3.1	93/06/20	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
-#include	"hack.h"
+#include "hack.h"
 
 #ifdef OVLB
 const char *traps[TRAPNUM] = {
@@ -57,8 +57,8 @@ register const char *ostr;
 int type;
 boolean print;
 {
-	static const char NEARDATA *action[] = { "smoulder", "rust", "rot", "corrode" };
-	static const char NEARDATA *msg[] =  { "burnt", "rusted", "rotten", "corroded" };
+	static NEARDATA const char *action[] = { "smoulder", "rust", "rot", "corrode" };
+	static NEARDATA const char *msg[] =  { "burnt", "rusted", "rotten", "corroded" };
 	boolean vulnerable = FALSE;
 	boolean plural;
 	boolean grprot = FALSE;
@@ -76,13 +76,13 @@ boolean print;
 
 	plural = is_gloves(otmp) || is_boots(otmp);
 
-	if (!vulnerable)
+	if (!vulnerable) {
 		if (flags.verbose)
 		    Your("%s %s not affected.", ostr, plural ? "are" : "is");
-	else if (otmp->oeroded < MAX_ERODE) {
-		if (grprot && otmp->greased)
+	} else if (otmp->oeroded < MAX_ERODE) {
+		if (grprot && otmp->greased) {
 			grease_protect(otmp,ostr,plural);
-		else if (otmp->oerodeproof || (otmp->blessed && !rnl(4))) {
+		} else if (otmp->oerodeproof || (otmp->blessed && !rnl(4))) {
 			if (flags.verbose)
 				pline("Somehow, your %s %s not affected.",
 					ostr, plural ? "are" : "is");
@@ -93,11 +93,12 @@ boolean print;
 				otmp->oeroded ? " further" : "");
 			otmp->oeroded++;
 		}
-	} else
+	} else {
 		if (flags.verbose)
 			Your("%s %s%s completely %s.", ostr,
 			     Blind ? "feel" : "look",
 			     plural ? "" : "s", msg[type]);
+	}
 	return(TRUE);
 }
 
@@ -124,9 +125,11 @@ maketrap(x,y,typ)
 register int x, y, typ;
 {
 	register struct trap *ttmp;
+	register struct rm *lev;
 	register boolean oldplace;
 
-	if (ttmp = t_at(x,y)) {
+	if ((ttmp = t_at(x,y)) != 0) {
+	    if (ttmp->ttyp == MAGIC_PORTAL) return (struct trap *)0;
 	    oldplace = TRUE;
 	    if (u.utrap && (x == u.ux) && (y == u.uy) && 
 	      ((u.utraptype == TT_BEARTRAP && typ != BEAR_TRAP) ||
@@ -147,9 +150,23 @@ register int x, y, typ;
 	    case PIT:
 	    case SPIKED_PIT:
 	    case TRAPDOOR:
-		levl[x][y].doormask = 0;   /* subsumes altarmask, icedpool... */
-		if (IS_ROOM(levl[x][y].typ))
-			levl[x][y].typ = ROOM;
+		lev = &levl[x][y];
+		lev->doormask = 0;	/* subsumes altarmask, icedpool... */
+		if (IS_ROOM(lev->typ)) /* && !IS_AIR(lev->typ) */
+		    lev->typ = ROOM;
+#if defined(POLYSELF) || defined(MUSE)
+		/*
+		 * some cases which can happen when digging
+		 * down while phazing thru solid areas
+		 */
+		else if (lev->typ == STONE || lev->typ == SCORR)
+		    lev->typ = CORR;
+		else if (IS_WALL(lev->typ) ||
+			 IS_DOOR(lev->typ) || lev->typ == SDOOR)
+		    lev->typ = level.flags.is_maze_lev ? ROOM :
+			       level.flags.is_cavernous_lev ? CORR : DOOR;
+#endif
+		unearth_objs(x, y);
 		break;
 	}
 	ttmp->tseen = 0;
@@ -174,13 +191,13 @@ boolean trapok;
 	boolean	tmp1, tmp2, tmp3, tmp4;
 # ifdef POLYSELF
 	tmp1 = isok(x,y) && (!IS_ROCK(levl[x][y].typ) ||
-		passes_walls(uasmon)) && !MON_AT(x, y);
+		(passes_walls(uasmon) && may_passwall(x,y))) && !MON_AT(x, y);
 # else
 	tmp1 = isok(x,y) && !IS_ROCK(levl[x][y].typ) && !MON_AT(x, y);
 # endif
 	tmp2 = !sobj_at(BOULDER,x,y) && (trapok || !t_at(x,y));
 	tmp3 = !(is_pool(x,y) &&
-	       !(Levitation || Wwalking || Magical_breathing
+	       !(Levitation || Wwalking || Amphibious
 # ifdef POLYSELF
 		 || is_flyer(uasmon) || is_swimmer(uasmon)
 		 || is_clinger(uasmon)
@@ -191,14 +208,15 @@ boolean trapok;
 #else
 	return( isok(x,y) &&
 # ifdef POLYSELF
-		(!IS_ROCK(levl[x][y].typ) || passes_walls(uasmon)) &&
+		(!IS_ROCK(levl[x][y].typ) ||
+		 (passes_walls(uasmon) && may_passwall(x,y))) &&
 # else
 		!IS_ROCK(levl[x][y].typ) &&
 # endif
 		!MON_AT(x, y) &&
 		!sobj_at(BOULDER,x,y) && (trapok || !t_at(x,y)) &&
 		!(is_pool(x,y) &&
-		!(Levitation || Wwalking || Magical_breathing
+		!(Levitation || Wwalking || Amphibious
 # ifdef POLYSELF
 		  || is_flyer(uasmon) || is_swimmer(uasmon)
 		  || is_clinger(uasmon)
@@ -271,7 +289,7 @@ boolean td;	/* td == TRUE : trapdoor */
 	} while(!rn2(4) && newlevel < dunlevs_in_dungeon(&u.uz));
 
 	if(td) pline("A trap door opens up under you!");
-	else pline("The floor opens up under you!");
+	else pline("The %s opens up under you!", surface(u.ux,u.uy));
 
 	if(Levitation || u.ustuck || !Can_fall_thru(&u.uz)
 #ifdef POLYSELF
@@ -317,9 +335,9 @@ register struct trap *trap;
 #else
 	   !(ttype == PIT || ttype == SPIKED_PIT) &&
 #endif
-	   !(ttype == MAGIC_PORTAL || ttype == ANTI_MAGIC) && !rn2(5))
+	   !(ttype == MAGIC_PORTAL || ttype == ANTI_MAGIC) && !rn2(5)) {
 		You("escape a%s.", traps[ttype]);
-	else {
+	} else {
 	    seetrap(trap);
 	    switch(ttype) {
 		case ARROW_TRAP:
@@ -327,9 +345,9 @@ register struct trap *trap;
 		    otmp = mksobj(ARROW, TRUE, FALSE);
 		    otmp->quan = 1L;
 		    otmp->owt = weight(otmp);
-		    if(thitu(8,dmgval(otmp,uasmon),otmp,"arrow"))
+		    if(thitu(8,dmgval(otmp,uasmon),otmp,"arrow")) {
 			obfree(otmp, (struct obj *)0);
-		    else {
+		    } else {
 			place_object(otmp, u.ux, u.uy);
 			otmp->nobj = fobj;
 			fobj = otmp;		
@@ -366,12 +384,14 @@ register struct trap *trap;
 	pline("A trap door in the ceiling opens and a rock falls on your %s!",
 				body_part(HEAD));
 
-			if (uarmh)
+			if (uarmh) {
 			    if(is_metallic(uarmh)) {
 				pline("Fortunately, you are wearing a hard helmet.");
 				dmg = 2;
-			    } else if (flags.verbose)
+			    } else if (flags.verbose) {
  				Your("%s does not protect you.", xname(uarmh));
+			    }
+			}
 
 			stackobj(otmp);
 			newsym(u.ux,u.uy);	/* map the rock */
@@ -447,7 +467,7 @@ register struct trap *trap;
 		    } else
 		    if (u.umonnum == PM_GREMLIN && rn2(3)) {
 			pline("A gush of water hits you!");
-			if(mtmp = cloneu()) {
+			if ((mtmp = cloneu()) != 0) {
 			    mtmp->mhpmax = (u.mhmax /= 2);
 			    You("multiply.");
 			}
@@ -505,9 +525,9 @@ two_hand:		    erode_weapon(FALSE);
 #endif
 			) {
 			if(Blind) break;
-			if(trap->tseen)
+			if(trap->tseen) {
 			    You("see a pit below you.");
-			else {
+			} else {
 			    pline("A pit opens up under you!");
 			    You("don't fall in!");
 			}
@@ -657,10 +677,11 @@ two_hand:		    erode_weapon(FALSE);
 		case STATUE_TRAP:
 		    deltrap(trap);
 		    newsym(u.ux,u.uy);	/* get rid of trap symbol */
-		    for(otmp=level.objects[u.ux][u.uy];
+		    for (otmp = level.objects[u.ux][u.uy];
 						otmp; otmp = otmp->nexthere)
-			if(otmp->otyp == STATUE)
-			    if(mtmp=makemon(&mons[otmp->corpsenm],u.ux,u.uy)) {
+			if (otmp->otyp == STATUE)
+			    if ((mtmp = makemon(&mons[otmp->corpsenm],
+						u.ux, u.uy)) != 0) {
 				pline("The statue comes to life!");
 				/* mimic statues become seen mimics */
 				if(mtmp->m_ap_type) seemimic(mtmp);
@@ -715,6 +736,7 @@ two_hand:		    erode_weapon(FALSE);
 			pline("KAABLAMM!!!  You triggered a land mine!");
 			set_wounded_legs(LEFT_SIDE, rn1(35, 41));
 			set_wounded_legs(RIGHT_SIDE, rn1(35, 41));
+			exercise(A_DEX, FALSE);
 		    }
 		    losehp(rnd(16), "land mine", KILLED_BY_AN);
 		    /* wake everything on the level */
@@ -797,14 +819,14 @@ mintrap(mtmp)
 register struct monst *mtmp;
 {
 	register struct trap *trap = t_at(mtmp->mx, mtmp->my);
-	boolean trapkilled = FALSE, tdoor = FALSE;
+	boolean trapkilled = FALSE;
 	struct permonst *mptr = mtmp->data;
 	struct obj *otmp;
 
 	if(!trap) {
 		mtmp->mtrapped = 0;	/* perhaps teleported? */
 	} else if (mtmp->mtrapped) {	/* was in trap */
-		if(!rn2(40)) 
+		if(!rn2(40)) {
 			if (sobj_at(BOULDER, mtmp->mx, mtmp->my) && 
 			    ((trap->ttyp == PIT) || 
 			     (trap->ttyp == SPIKED_PIT))) {
@@ -814,6 +836,7 @@ register struct monst *mtmp;
 				}
 			} else
 				mtmp->mtrapped = 0;
+		}
 	} else {
 	    register int tt = trap->ttyp;
 
@@ -874,16 +897,17 @@ register struct monst *mtmp;
 		case BEAR_TRAP:
 			if(mptr->msize > MZ_SMALL &&
 			   !amorphous(mptr) && !is_flyer(mptr)) {
-				mtmp->mtrapped = 1;
-				if(in_sight) {
-				  pline("%s is caught in a bear trap!",
-					Monnam(mtmp));
-				  seetrap(trap);
-				} else
-				    if((mptr == &mons[PM_OWLBEAR]
-					|| mptr == &mons[PM_BUGBEAR])
-					&& flags.soundok)
-			    You("hear the roaring of an angry bear!");
+			    mtmp->mtrapped = 1;
+			    if(in_sight) {
+				pline("%s is caught in a bear trap!",
+				      Monnam(mtmp));
+				seetrap(trap);
+			    } else {
+				if((mptr == &mons[PM_OWLBEAR]
+				    || mptr == &mons[PM_BUGBEAR])
+				   && flags.soundok)
+				    You("hear the roaring of an angry bear!");
+			    }
 			}
 			break;
 
@@ -912,7 +936,10 @@ register struct monst *mtmp;
 				    pline("May %s rust in peace.",
 								mon_nam(mtmp));
 				mondied(mtmp);
-				trapkilled = TRUE;
+#ifdef MUSE
+				if (mtmp->mhp <= 0)
+#endif
+					trapkilled = TRUE;
 			} else if (mptr == &mons[PM_GREMLIN] && rn2(3)) {
 				struct monst *mtmp2 = clone_mon(mtmp);
 
@@ -926,8 +953,8 @@ register struct monst *mtmp;
 
 		case FIRE_TRAP:
 			if (in_sight)
-		pline("A tower of flame bursts from the floor under %s!",
-					mon_nam(mtmp));
+			 pline("A tower of flame bursts from the %s under %s!",
+				surface(mtmp->mx,mtmp->my), mon_nam(mtmp));
 			if(resists_fire(mptr)) {
 			    if (in_sight) {
 				shieldeff(mtmp->mx,mtmp->my);
@@ -943,6 +970,8 @@ register struct monst *mtmp;
 			(void) destroy_mitem(mtmp, SCROLL_CLASS, AD_FIRE);
 			(void) destroy_mitem(mtmp, SPBOOK_CLASS, AD_FIRE);
 			(void) destroy_mitem(mtmp, POTION_CLASS, AD_FIRE);
+			if (is_ice(mtmp->mx,mtmp->my))
+			    melt_ice(mtmp->mx,mtmp->my);
 			if (in_sight) seetrap(trap);
 			break;
 
@@ -957,8 +986,15 @@ register struct monst *mtmp;
 				    pline("%s falls into a pit!", Monnam(mtmp));
 				    seetrap(trap);
 				}
+#ifdef MUSE
+				mselftouch(mtmp, "Falling, ", FALSE);
+				if(mtmp->mhp <= 0 ||
+					thitm(0, mtmp, (struct obj *)0,
+					 rnd((tt==PIT) ? 6 : 10)))
+#else
 				if(thitm(0, mtmp, (struct obj *)0,
 					 rnd((tt==PIT) ? 6 : 10)))
+#endif
 				    trapkilled = TRUE;
 			}
 			break;
@@ -967,85 +1003,93 @@ register struct monst *mtmp;
 			if(!Can_fall_thru(&u.uz))
 			    panic("Trapdoors cannot exist on this level.");
 
-			if ( (mptr == &mons[PM_WUMPUS]) ||
-			     (mtmp->wormno && count_wsegs(mtmp) > 5) ) break;
-			tdoor = TRUE;
+			if (is_flyer(mptr) || mptr == &mons[PM_WUMPUS] ||
+			    (mtmp->wormno && count_wsegs(mtmp) > 5)) break;
 			/* Fall through */
 		case LEVEL_TELEP:
-			/* long worms w/tails can now change levels! - Norm */
-			if (!is_flyer(mptr)) {
-			    register int nlev;
+		case MAGIC_PORTAL:
+			if (mtmp == u.ustuck)	/* probably a vortex */
+			    break;		/* temporary? kludge */
+#ifdef WALKIES
+			if (teleport_pet(mtmp))
+#endif
+			{
 			    d_level tolevel;
-#ifdef WALKIES
-			    if(teleport_pet(mtmp)) {
-#endif
-				if(tdoor) {
-				    if(Is_stronghold(&u.uz))
-				        assign_level(&tolevel, &valley_level);
-				    else if(Is_botlevel(&u.uz)) {
-				        pline("%s avoids the trap.", 
-					                  Monnam(mtmp));
-					break;
-				    } else get_level(&tolevel,depth(&u.uz)+1);
+			    int migrate_typ = 0;
+
+			    if (tt == TRAPDOOR) {
+				if (Is_stronghold(&u.uz)) {
+				    assign_level(&tolevel, &valley_level);
+				} else if (Is_botlevel(&u.uz)) {
+				    pline("%s avoids the trap.", Monnam(mtmp));
+				    break;
 				} else {
-#ifdef MULDGN
-				    if(Is_knox(&u.uz)) {
-				        rloc(mtmp);
-					break;
-				    }
-#endif
-				    nlev = rnd(3);
-				    if(!rn2(2)) nlev = -(nlev);
-				    nlev = dunlev(&u.uz) + nlev;
-				    if(nlev > dunlevs_in_dungeon(&u.uz)) {
-				    	nlev = dunlevs_in_dungeon(&u.uz);
-					/* teleport up if already on bottom */
-					if (Is_botlevel(&u.uz)) 
-					    nlev -= rnd(3);
-				    }
-				    if (nlev < 1) {
-					nlev = 1;
-					if (dunlev(&u.uz) == 1) {
-					    nlev += rnd(3);
-					    if (nlev >
-					          dunlevs_in_dungeon(&u.uz)) 
-					        nlev = 
-						  dunlevs_in_dungeon(&u.uz);
-					}
-				    }
-				    /* can't seem to go anywhere    */
-				    /* (possible in short dungeons) */
-				    if (nlev == dunlev(&u.uz)) {
-					rloc(mtmp);
-					break;
-				    }
-				    nlev = dungeons[u.uz.dnum].depth_start +
-				                               nlev;
-				    get_level(&tolevel, nlev);
+				    get_level(&tolevel, depth(&u.uz) + 1);
 				}
-				if(in_sight) {
-		pline("Suddenly, %s disappears out of sight.", mon_nam(mtmp));
-				    seetrap(trap);
+			    } else if (tt == MAGIC_PORTAL) {
+				if (In_endgame(&u.uz) &&
+				    (mon_has_amulet(mtmp) ||
+					is_home_elemental(mptr))) {
+				    if (in_sight && mptr->mlet != S_ELEMENTAL) {
+				     pline("%s seems to shimmer for a moment.",
+					   Monnam(mtmp));
+					seetrap(trap);
+				    }
+				    break;
+				} else {
+				    assign_level(&tolevel, &trap->dst);
+				    migrate_typ = 6; /* see dog.c */
 				}
-				migrate_to_level(mtmp, 
-						 ledger_no(&tolevel), 0);
-				return(3);	/* no longer on this level */
-#ifdef WALKIES
+			    } else { /* (tt == LEVEL_TELEP) */
+				register int nlev;
+
+				if (mon_has_amulet(mtmp)) break;
+				nlev = rnd(3);
+				if (!rn2(2)) nlev = -nlev;
+				nlev += dunlev(&u.uz);
+				if (nlev > dunlevs_in_dungeon(&u.uz)) {
+				    nlev = dunlevs_in_dungeon(&u.uz);
+				    /* teleport up if already on bottom */
+				    if (Is_botlevel(&u.uz)) nlev -= rnd(3);
+				}
+				if (nlev < 1) {
+				    nlev = 1;
+				    if (dunlev(&u.uz) == 1) {
+					nlev += rnd(3);
+					if (nlev > dunlevs_in_dungeon(&u.uz)) 
+					    nlev = dunlevs_in_dungeon(&u.uz);
+				    }
+				}
+				/* can't seem to go anywhere    */
+				/* (possible in short dungeons) */
+				if (nlev == dunlev(&u.uz)) {
+				    rloc(mtmp);
+				    break;
+				}
+				nlev += dungeons[u.uz.dnum].depth_start;
+				get_level(&tolevel, nlev);
 			    }
-#endif
+
+			    if (in_sight) {
+				pline("Suddenly, %s disappears out of sight.",
+				      mon_nam(mtmp));
+				seetrap(trap);
+			    }
+			    migrate_to_level(mtmp, 
+					     ledger_no(&tolevel),
+					     migrate_typ);
+			    return(3);	/* no longer on this level */
 			}
 			break;
 
 		case TELEP_TRAP:
-		case MAGIC_PORTAL:
+			if(tele_restrict(mtmp)) break;
 #ifdef WALKIES
 			if(teleport_pet(mtmp)) {
 #endif
 			    /* Note: don't remove the trap if a vault.  Other-
 			     * wise the monster will be stuck there, since 
 			     * the guard isn't going to come for it...
-			     * Also: don't remove if magic portal.  In short,
-			     * don't remove :-)
 			     */
 			    if (in_sight) {
 				pline("%s suddenly disappears!", 
@@ -1059,7 +1103,7 @@ register struct monst *mtmp;
 #endif
 			break;
 
-	       case WEB:
+		case WEB:
 			/* Monster in a web. */
 			if (mptr->mlet == S_SPIDER) break;
 			if (amorphous(mptr)) {
@@ -1105,8 +1149,8 @@ register struct monst *mtmp;
 		case MAGIC_TRAP:
 			/* A magic trap.  Monsters immune. */
 			break;
-		case ANTI_MAGIC:	
-                        break;
+		case ANTI_MAGIC:
+			break;
 
 		case LANDMINE: {
 			register struct monst *mntmp = fmon;
@@ -1115,7 +1159,7 @@ register struct monst *mtmp;
 				break; /* monsters usually don't set it off */
 			if(is_flyer(mptr)) {
 				if (in_sight) {
-	pline("A trigger appears in a pile of soil below %s.", Monnam(mtmp));
+	pline("A trigger appears in a pile of soil below %s.", mon_nam(mtmp));
 					seetrap(trap);
 				}
 				if (rn2(3)) break;
@@ -1166,7 +1210,7 @@ void
 selftouch(arg)
 const char *arg;
 {
-	if(uwep && (uwep->otyp == CORPSE && uwep->corpsenm == PM_COCKATRICE)
+	if(uwep && uwep->otyp == CORPSE && uwep->corpsenm == PM_COCKATRICE
 #ifdef POLYSELF
 			&& !resists_ston(uasmon)
 #endif
@@ -1183,6 +1227,34 @@ const char *arg;
 	}
 }
 
+#ifdef MUSE
+void
+mselftouch(mon,arg,byplayer)
+struct monst *mon;
+const char *arg;
+boolean byplayer;
+{
+	struct obj *mwep = MON_WEP(mon);
+
+	if (mwep && mwep->otyp == CORPSE && mwep->corpsenm == PM_COCKATRICE
+			&& !resists_ston(mon->data)) {
+		if (cansee(mon->mx, mon->my)) {
+			pline("%s%s touches the cockatrice corpse.",
+			    arg ? arg : "", arg ? mon_nam(mon) : Monnam(mon));
+			pline("%s turns to stone.", Monnam(mon));
+		}
+		if (poly_when_stoned(mon->data)) {
+			mon_to_stone(mon);
+			return;
+		}
+		if (byplayer) {
+			stoned = TRUE;
+			xkilled(mon,0);
+		} else monstone(mon);
+	}
+}
+#endif
+
 void
 float_up()
 {
@@ -1273,17 +1345,17 @@ float_down()
 	if (!trap) {
 		if(Is_airlevel(&u.uz))
 			You("begin to tumble in place.");
-		if(Is_waterlevel(&u.uz) && !no_msg)
+		else if (Is_waterlevel(&u.uz) && !no_msg)
 			You("feel heavier.");
 		/* u.uinwater msgs already in spoteffects()/drown() */
 		else if (!u.uinwater && !no_msg) {
 			if (Hallucination)
 				pline("Bummer!  You've %s.",
-			      	is_pool(u.ux,u.uy) ?
-			      		"splashed down" : "hit the ground");
+				      is_pool(u.ux,u.uy) ?
+					"splashed down" : "hit the ground");
 			else
 				You("float gently to the %s.",
-				    is_pool(u.ux,u.uy) ? "water" : "ground");
+				    surface(u.ux, u.uy));
 		}
 		trap = t_at(u.ux,u.uy);
 	}
@@ -1299,9 +1371,7 @@ float_down()
 		default:
 			dotrap(trap);
 	}
-	if(!flags.nopick && OBJ_AT(u.ux, u.uy) &&
-	   !Is_airlevel(&u.uz) && !Is_waterlevel(&u.uz) &&
-	   (!is_pool(u.ux,u.uy) || Underwater))
+	if(!Is_airlevel(&u.uz) && !Is_waterlevel(&u.uz))
 	    pickup(1);
 	return 0;
 }
@@ -1337,9 +1407,9 @@ tele()
 			    || wizard
 #endif
 					) {
-	    if (unconscious())
+	    if (unconscious()) {
 		pline("Being unconscious, you cannot control your teleport.");
-	    else {
+	    } else {
 		    pline("To what position do you want to be teleported?");
 		    cc.x = u.ux;
 		    cc.y = u.uy;
@@ -1358,6 +1428,19 @@ tele()
 	(void) safe_teleds();
 }
 
+boolean
+tele_restrict(mon)
+struct monst *mon;
+{
+	if(level.flags.noteleport) {
+		if (canseemon(mon))
+		    pline("A mysterious force prevents %s from teleporting!",
+			mon_nam(mon));
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void
 teleds(nux, nuy)
 register int nux,nuy;
@@ -1377,11 +1460,11 @@ register int nux,nuy;
 		u.uundetected = 0;
 	if (u.usym == S_MIMIC_DEF) u.usym = S_MIMIC;
 #endif
-	if(Punished) placebc();
 	if(u.uswallow){
 		u.uswldtim = u.uswallow = 0;
 		docrt();
 	}
+	if(Punished) placebc();
 	initrack(); /* teleports mess up tracking monsters without this */
 	/*
 	 *  Make sure the hero disappears from the old location.  This will
@@ -1400,7 +1483,7 @@ dotele()
 {
 	struct trap *trap;
 	boolean castit = FALSE;
-	register int sp_no = 0;
+	register int sp_no = 0, energy;
 
 	trap = t_at(u.ux, u.uy);
 	if (trap && (!trap->tseen || trap->ttyp != TELEP_TRAP))
@@ -1457,12 +1540,31 @@ dotele()
 	}
 
 	if(!trap && (u.uhunger <= 100 || ACURR(A_STR) < 6)) {
-		You("lack the strength for a teleport spell.");
 #ifdef WIZARD
-		if(!wizard)
+		if (!wizard) {
+#endif
+			You("lack the strength %s.",
+			    castit ? "for a teleport spell" : "to teleport");
+			return 1;
+#ifdef WIZARD
+		}
 #endif
-		return(1);
 	}
+
+	energy = objects[SPE_TELEPORT_AWAY].oc_level * 7 / 2 - 2;
+
+	if(!trap && u.uen <= energy) {
+#ifdef WIZARD
+		if (!wizard) {
+#endif
+			You("lack the energy %s.",
+			    castit ? "for a teleport spell" : "to teleport");
+			return 1;
+#ifdef WIZARD
+		} else u.uen = energy;
+#endif
+	}
+
 	if(!trap &&
 	  check_capacity("Your concentration falters from carrying so much."))
 	    return 1;
@@ -1476,7 +1578,9 @@ dotele()
 		    if (!wizard)
 #endif
 			return(0);
-	}
+	} else if (!trap)
+		u.uen -= energy;
+
 #ifdef WALKIES
 	if(next_to_u()) {
 #endif
@@ -1516,8 +1620,10 @@ level_tele()
 	    char buf[BUFSZ];
 
 	    do {
-	      getlin("To what level do you want to teleport? [type a number]",
+		getlin("To what level do you want to teleport? [type a number]",
 			buf);
+		if (!strcmp(buf,"\033"))	/* cancelled */
+		    return;
 	    } while(!digit(buf[0]) && (buf[0] != '-' || !digit(buf[1])));
 	    newlev = atoi(buf);
 
@@ -1597,7 +1703,11 @@ level_tele()
 		} else
 			You("are now high above the clouds...");
 
-		if(Levitation || is_floater(uasmon)) {
+		if(Levitation
+#ifdef POLYSELF
+		   || is_floater(uasmon)
+#endif
+		   ) {
 		    You("float gently down to earth.");
 		    u.uz.dnum = 0; /* he might have been in another dgn */
 		    newlev = 1;
@@ -1677,7 +1787,7 @@ dofiretrap()
 	 * SCR_FIRE by GAN 11/02/86
 	 */
 
-	pline("A tower of flame bursts from the floor!");
+	pline("A tower of flame bursts from the %s!", surface(u.ux,u.uy));
 	if(Fire_resistance) {
 		shieldeff(u.ux, u.uy);
 		You("are uninjured.");
@@ -1689,6 +1799,8 @@ dofiretrap()
 	destroy_item(SCROLL_CLASS, AD_FIRE);
 	destroy_item(SPBOOK_CLASS, AD_FIRE);
 	destroy_item(POTION_CLASS, AD_FIRE);
+	if (is_ice(u.ux, u.uy))
+		melt_ice(u.ux, u.uy);
 }
 
 static void
@@ -1802,13 +1914,13 @@ domagictrap()
 }
 
 void
-water_damage(obj,force)
+water_damage(obj, force, here)
 register struct obj *obj;
-register boolean force;
+register boolean force, here;
 {
 	/* Scrolls, spellbooks, potions, weapons and
 	   pieces of armor may get affected by the water */
-	for(; obj; obj = obj->nobj) {
+	for (; obj; obj = (here ? obj->nexthere : obj->nobj)) {
 
 		(void) snuff_lit(obj);
 
@@ -1816,7 +1928,7 @@ register boolean force;
 			if (force || !rn2(2)) obj->greased = 0;
 		} else if(Is_container(obj) && !Is_box(obj) &&
 			(obj->otyp != OILSKIN_SACK || (obj->cursed && !rn2(3)))) {
-			water_damage(obj->cobj,force);
+			water_damage(obj->cobj, force, FALSE);
 		} else if(obj->oclass == SCROLL_CLASS && (force || rn2(12) > Luck)
 #ifdef MAIL
 			  && obj->otyp != SCR_MAIL
@@ -1836,8 +1948,8 @@ register boolean force;
 		} else if(is_rustprone(obj) && obj->oeroded < MAX_ERODE &&
 			  !(obj->oerodeproof || (obj->blessed && !rnl(4))) &&
 			  (force || rn2(12) > Luck)) {
-			/* all metal stuff and armor except body armor
-			   protected by oilskin cloak */
+			/* all metal stuff and armor except (body armor
+			   protected by oilskin cloak) */
 			if(obj->oclass != ARMOR_CLASS || obj != uarm ||
 			   !uarmc || uarmc->otyp != OILSKIN_CLOAK ||
  			   (uarmc->cursed && !rn2(3)))
@@ -1909,15 +2021,20 @@ drown()
 
 	/* happily wading in the same contiguous pool */
 	if (u.uinwater && is_pool(u.ux-u.dx,u.uy-u.dy) &&
-	   Magical_breathing) {
+	    (
+#ifdef POLYSELF
+	     is_swimmer(uasmon) ||
+#endif
+	     Amphibious)) {
 		/* water effects on objects every now and then */
 		if (!rn2(5)) inpool_ok = TRUE;
 		else return(FALSE);
 	}
 
 	if (!u.uinwater) {
-	    You("%s into the water!",
-		Is_waterlevel(&u.uz) ? "plunge" : "fall");
+	    You("%s into the water%c",
+		Is_waterlevel(&u.uz) ? "plunge" : "fall",
+		Amphibious ? '.' : '!');
 #ifdef POLYSELF
 	    if(!is_swimmer(uasmon))
 #endif
@@ -1926,18 +2043,16 @@ drown()
 			Hallucination ? "the Titanic" : "a rock");
 	}
 
-	water_damage(invent,FALSE);
+	water_damage(invent, FALSE, FALSE);
 
 #ifdef POLYSELF
 	if(u.umonnum == PM_GREMLIN && rn2(3)) {
 		struct monst *mtmp;
-		if(mtmp = cloneu()) {
+		if ((mtmp = cloneu()) != 0) {
 			mtmp->mhpmax = (u.mhmax /= 2);
 			You("multiply.");
 		}
 	}
-
-	if(is_swimmer(uasmon)) return(FALSE);
 #endif
 	if (inpool_ok) return(FALSE);
 #ifdef WALKIES
@@ -1948,14 +2063,27 @@ drown()
 		unleash_all();
 	}
 #endif
-	if (Magical_breathing) {
-		pline("But wait!");
-		Your("lungs start acting like gills.");
-		if (!Is_waterlevel(&u.uz))
-			Your("%s the bottom.",Hallucination ? "keel hits" : "feet touch");
-		if (Punished) placebc();
+	if (Amphibious
+#ifdef POLYSELF
+			|| is_swimmer(uasmon)
+#endif
+						) {
+		if (Amphibious) {
+			if (flags.verbose)
+				pline("But you aren't drowning.");
+			if (!Is_waterlevel(&u.uz))
+				if (Hallucination) 
+					Your("keel hits the bottom.");
+				else
+					You("touch bottom.");
+		}
+		if (Punished) {
+			unplacebc();
+			placebc();
+		}
 		u.uinwater = 1;
 		under_water(1);
+		vision_full_recalc = 1;
 		return(FALSE);
 	}
 	if((Teleportation || can_teleport(uasmon)) &&
@@ -2013,8 +2141,8 @@ crawl:;
 			done(DROWNING);
 		}
 	u.uinwater = 0;
-	You("find yourself back %s.",Is_waterlevel(&u.uz) ?
-		"in an air bubble" : "on dry land");
+	You("find yourself back %s.", Is_waterlevel(&u.uz) ?
+		"in an air bubble" : "on land");
 	return(TRUE);
 }
 
@@ -2153,7 +2281,7 @@ boolean force;
 		    if(!force && (confused || Fumbling || 
 		                     rnd(75+level_difficulty()/2) > ch)) {
 			    You("set it off!");
-			    b_trapped("door");
+			    b_trapped("door", FINGER);
 		    } else
 			    You("disarm it!");
 		    levl[x][y].doormask &= ~D_TRAPPED;
@@ -2264,7 +2392,7 @@ boolean disarm;
 		case 14:
 		case 13:
 			You("feel a needle prick your %s.",body_part(bodypart));
-			poisoned("needle", A_CON, "poison needle",10);
+			poisoned("needle", A_CON, "poisoned needle",10);
 			exercise(A_CON, FALSE);
 			break;
 		case 12:
@@ -2280,6 +2408,8 @@ boolean disarm;
 			destroy_item(SCROLL_CLASS, AD_FIRE);
 			destroy_item(SPBOOK_CLASS, AD_FIRE);
 			destroy_item(POTION_CLASS, AD_FIRE);
+			if (is_ice(u.ux, u.uy))
+			    melt_ice(u.ux, u.uy);
 			break;
 		case 8:
 		case 7:
@@ -2303,13 +2433,16 @@ boolean disarm;
 		case 2:
 		case 1:
 		case 0:
-			pline("A cloud of %s gas billows from %s",
+			pline("A cloud of %s gas billows from %s.",
 			      hcolor(), the(xname(obj)));
-			if(!Stunned)
+			if(!Stunned) {
 			    if (Hallucination)
 				pline("What a groovy feeling!");
+			    else if (Blind)
+				You("stagger and get dizzy...");
 			    else
 				You("stagger and your vision blurs...");
+			}
 			make_stunned(HStun + rn1(7, 16),FALSE);
 			make_hallucinated(HHallucination + rn1(5, 16),FALSE,0L);
 			break;
@@ -2356,19 +2489,19 @@ register struct trap *trap;
 	dealloc_trap(trap);
 }
 
-/* used for doors.  can be used for anything else that opens. */
+/* used for doors (also tins).  can be used for anything else that opens. */
 void
-b_trapped(item)
+b_trapped(item, bodypart)
 register const char *item;
+register int bodypart;
 {
 	register int lvl = level_difficulty();
-	int dmg = rnd(5 + (lvl<5 ? lvl : 2+lvl/2));
+	int dmg = rnd(5 + (lvl < 5 ? lvl : 2+lvl/2));
 
 	pline("KABOOM!!  %s was booby-trapped!", The(item));
-	if (u.ulevel < 4 && lvl < 3 && !rnl(3))
-		You("are shaken, but luckily unhurt.");		
-	else losehp(dmg, "explosion", KILLED_BY_AN);
+	losehp(dmg, "explosion", KILLED_BY_AN);
 	exercise(A_STR, FALSE);
+	if (bodypart) exercise(A_CON, FALSE);
 	make_stunned(HStun + dmg, TRUE);
 }
 
@@ -2410,8 +2543,14 @@ int d_override;
 			int yy = mon->my;
 
 			monkilled(mon, "", AD_PHYS);
-			newsym(xx, yy);
-			trapkilled = TRUE;
+#ifdef MUSE
+			if (mon->mhp <= 0) {
+#endif
+				newsym(xx, yy);
+				trapkilled = TRUE;
+#ifdef MUSE
+			}
+#endif
 		}
 	}
 	if (obj && (!strike || d_override)) {
@@ -2427,10 +2566,10 @@ int d_override;
 boolean
 unconscious()
 {
-	return (multi < 0 && (!nomovemsg ||
+	return((boolean)(multi < 0 && (!nomovemsg ||
 		u.usleep ||
 		!strncmp(nomovemsg,"You regain con", 15) ||
-		!strncmp(nomovemsg,"You are consci", 15)));
+		!strncmp(nomovemsg,"You are consci", 15))));
 }
 
 static char lava_killer[] = "molten lava";
@@ -2490,7 +2629,7 @@ lava_effects()
 			pline("You're still burning.");
 			done(BURNING);
 		}
-	You("find yourself back on solid ground.");
+	You("find yourself back on solid %s.", surface(u.ux, u.uy));
 	return(TRUE);
     }
 
