@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)u_init.c	3.1	92/11/13	*/
+/*	SCCS Id: @(#)u_init.c	3.1	93/07/07	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -22,6 +22,8 @@ static int FDECL(role_index,(CHAR_P));
 #define	UNDEF_SPE	'\177'
 #define	UNDEF_BLESS	2
 
+static boolean random_role = FALSE;
+
 /* all roles must all have distinct first letter */
 const char *roles[] = {	/* also used in options.c and winxxx.c */
 			/* roles[2] and [6] are changed for females */
@@ -260,6 +262,10 @@ u_init()
 	    terminate(0);
 	}
 	i = role_index(pc);
+	if (random_role) {
+	    pline("This game you will be %s.", an(roles[i]));
+	    display_nhwindow(WIN_MESSAGE, TRUE);
+	}
 
 	(void) strncpy(pl_character, roles[i], PL_CSIZ-1);
 	pl_character[PL_CSIZ-1] = 0;
@@ -268,8 +274,8 @@ u_init()
 	/* zero u, including pointer values --
 	 * necessary when aborting from a failed restore */
 	(void) memset((genericptr_t)&u, 0, sizeof(u));
-	for (i = 0; i < LAST_PROP+1; i++) u.uprops[i].p_tofn = 0;
 	u.ustuck = (struct monst *)0;
+	u.usick_cause = NULL;
 
 #if 0	/* documentation of more zero values as desirable */
 	u.uluck  = u.moreluck = 0;
@@ -298,6 +304,7 @@ u_init()
 
 	u.usym = S_HUMAN;
 	u.umoved = FALSE;
+	u.umortality = 0;
 	u.ugrave_arise = -1;
 
 	u.ulevel = 0;	/* set up some of the initial attributes */
@@ -315,6 +322,12 @@ u_init()
 	set_uasmon();
 #endif
 
+#ifdef BSD
+	(void) time((long *)&u.ubirthday);
+#else
+	(void) time(&u.ubirthday);
+#endif
+
 	/*
 	 *  For now, everyone starts out with a night vision range of 1 and
 	 *  their xray range disabled.
@@ -362,29 +375,21 @@ u_init()
 			 * generators are bad enough to seriously skew the
 			 * results if we use rn2(2)...  --KAA
 			 */
-
+		ini_inv(Elf);
 		/*
 		 * Elves are people of music and song, or they are warriors.
 		 * Warriors get mithril coats; non-warriors MAY get an
 		 * instrument.  We use a kludge to get only non-magic
 		 * instruments.
 		 */
-		if (Elf[E_ARMOR].trotyp == ELVEN_CLOAK) {
-			if (!rn2(5)) {
-		    	static int trotyp[] = {
-					WOODEN_FLUTE, TOOLED_HORN, WOODEN_HARP,
-					BELL, BUGLE, LEATHER_DRUM
-				};
-
-				Instrument[0].trotyp = trotyp[rn2(SIZE(trotyp))];
-#ifdef DEBUG
-				debugpline("Elf got instrument %d",
-					Instrument[0].trotyp);
-#endif
-				ini_inv(Instrument);
-			}
+		if (Elf[E_ARMOR].trotyp == ELVEN_CLOAK && !rn2(5)) {
+		    static int trotyp[] = {
+			WOODEN_FLUTE, TOOLED_HORN, WOODEN_HARP,
+			BELL, BUGLE, LEATHER_DRUM
+		    };
+		    Instrument[0].trotyp = trotyp[rn2(SIZE(trotyp))];
+		    ini_inv(Instrument);
 		}
-		ini_inv(Elf);
 		if(!rn2(5)) ini_inv(Blindfold);
 		else if(!rn2(6)) ini_inv(Lamp);
 		knows_object(ELVEN_SHORT_SWORD);
@@ -475,6 +480,8 @@ u_init()
 	if (discover)
 		ini_inv(Wishing);
 #endif
+	u.ugold0 += hidden_gold();	/* in case sack has gold in it */
+
 	find_ac();			/* get initial ac value */
 	init_attr(75);			/* init attribute values */
 	max_rank_sz();			/* set max str size for class ranks */
@@ -518,10 +525,10 @@ register struct trobj *trop;
 		 */
 		if (undefined) {
 #ifdef POLYSELF
-			static unsigned NEARDATA nocreate = STRANGE_OBJECT;
-			static unsigned NEARDATA nocreate2 = STRANGE_OBJECT;
+			static NEARDATA unsigned nocreate = STRANGE_OBJECT;
+			static NEARDATA unsigned nocreate2 = STRANGE_OBJECT;
 #endif
-			static unsigned NEARDATA nocreate3 = STRANGE_OBJECT;
+			static NEARDATA unsigned nocreate3 = STRANGE_OBJECT;
 
 			while(obj->otyp == WAN_WISHING
 #ifdef POLYSELF
@@ -649,17 +656,25 @@ register struct trobj *trop;
 }
 
 void
-plnamesuffix() {
+plnamesuffix()
+{
 	register char *p;
 	if ((p = rindex(plname, '-')) != 0) {
-		*p = 0;
+		*p = '\0';
 		pl_character[0] = p[1];
-		pl_character[1] = 0;
+		pl_character[1] = '\0';
+		random_role = FALSE;
 		if(!plname[0]) {
 			askname();
 			plnamesuffix();
 		}
 	}
+	if (pl_character[0] == '@') {	/* explicit request for random class */
+		int i = rn2((int)strlen(pl_classes));
+		pl_character[0] = pl_classes[i];
+		pl_character[1] = '\0';
+		random_role = TRUE;
+	}
 }
 
 /*u_init.c*/
