@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)uhitm.c	3.1	92/12/10	*/
+/*	SCCS Id: @(#)uhitm.c	3.1	93/02/18	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -102,7 +102,7 @@ register struct monst *mtmp;
 			else if ((obj = level.objects[mtmp->mx][mtmp->my]) != 0)
 			    pline("Wait!  There's %s hiding under %s!",
 					an(l_monnam(mtmp)), doname(obj));
-			wakeup(mtmp);
+			mtmp->msleep = 0;
 			mtmp->data->mflags3 &= ~M3_WAITMASK;
 			return(TRUE);
 		}
@@ -171,7 +171,7 @@ register struct monst *mtmp;
 	if (is_orc(mtmp->data) && pl_character[0]=='E') tmp++;
 
 /*	with a lot of luggage, your agility diminishes */
-	if(tmp2 = near_capacity()) tmp -= (tmp2*2) - 1;
+	if ((tmp2 = near_capacity()) != 0) tmp -= (tmp2*2) - 1;
 	if(u.utrap) tmp -= 3;
 #ifdef POLYSELF
 /*	Some monsters have a combination of weapon attacks and non-weapon
@@ -312,6 +312,7 @@ register int mhit;
 		/* 1/20 chance of shattering defender's weapon */
 		struct obj *obj = MON_WEP(mon);
 
+		obj->owornmask &= ~W_WEP;
 		MON_NOWEP(mon);
 		m_useup(mon, obj);
 		pline("%s weapon shatters!", s_suffix(Monnam(mon)));
@@ -323,6 +324,8 @@ register int mhit;
 	    }
 #endif
 	} else {
+	    int oldhp = mon->mhp;
+
 	    /* we hit the monster; be careful: it might die! */
 	    notonhead = (mon->mx != u.ux+u.dx || mon->my != u.uy+u.dy);
 	    if((malive = hmon(mon, uwep, 0)) == TRUE) {
@@ -337,7 +340,12 @@ register int mhit;
 								)
 				u.ustuck = 0;
 		}
-		if (mon->wormno) cutworm(mon, u.ux+u.dx, u.uy+u.dy, uwep);
+		/* If no damage was done (Vorpal Blade and not on head)
+		 * do not cut the worm.  We lost the information long ago, so
+		 * we must do this by checking the hit points.
+		 */
+		if (mon->wormno && mon->mhp < oldhp)
+			cutworm(mon, u.ux+u.dx, u.uy+u.dy, uwep);
 	    }
 	    if(mon->ispriest && !rn2(2)) ghod_hitsu(mon);
 	    if(special) (void) angry_guards(!flags.soundok);
@@ -350,7 +358,7 @@ hitum(mon, tmp)		/* returns TRUE if monster still lives */
 struct monst *mon;
 int tmp;
 {
-	static int NEARDATA malive;
+	static NEARDATA boolean malive;
 	boolean mhit = (tmp > (dieroll = rnd(20)) || u.uswallow);
 
 	if(tmp > dieroll) exercise(A_DEX, TRUE);
@@ -426,13 +434,13 @@ register int thrown;
 	       obj->otyp == UNICORN_HORN || obj->oclass == ROCK_CLASS) {
 
 		/* If not a melee weapon, and either not thrown, or thrown */
-		/* and a bow (bows are >BOOMERANG), or thrown and a missile */
-		/* without a propellor (missiles are <DART), do 1-2 points */
-		if((obj->otyp >= BOW || obj->otyp < DART)
+		/* and a bow (bows are >= BOW), or thrown and a missile */
+		/* without a propellor (which means <DART), do 1-2 points */
+		if((obj->otyp >= BOW || obj->otyp < BOOMERANG)
 			&& obj->otyp != PICK_AXE && obj->otyp != UNICORN_HORN
 			&& (!thrown ||
 			    (obj->oclass != ROCK_CLASS &&
-			    (obj->otyp > BOOMERANG ||
+			    (obj->otyp >= BOW ||
 				(obj->otyp < DART &&
 				    (!uwep ||
 				    objects[obj->otyp].w_propellor !=
@@ -448,6 +456,7 @@ register int thrown;
 			artifact_hit(&youmonst, mon, obj, &tmp, dieroll)) {
 			if(mon->mhp <= 0) /* artifact killed monster */
 			    return FALSE;
+			if (tmp == 0) return TRUE;
 			hittxt = TRUE;
 		    }
 		    if (objects[obj->otyp].oc_material == SILVER
@@ -662,12 +671,7 @@ register int thrown;
 	if(mon->mhp < 1)
 		destroyed = TRUE;
 	if(mon->mtame && (!mon->mflee || mon->mfleetim)) {
-#ifdef SOUNDS
-		if (rn2(8)) yelp(mon);
-		else growl(mon); /* give them a moment's worry */
-#endif
-		mon->mtame--;
-		if(!mon->mtame) newsym(mon->mx, mon->my);
+		abuse_dog(mon);
 		mon->mflee = TRUE;		/* Rick Richardson */
 		mon->mfleetim += 10*rnd(tmp);
 	}
@@ -740,7 +744,7 @@ register int thrown;
 	}
 #endif
 
-	return(destroyed ? FALSE : TRUE);
+	return((boolean)(destroyed ? FALSE : TRUE));
 }
 
 #ifdef POLYSELF
@@ -758,7 +762,8 @@ register struct attack *mattk;
 		&& u.umonnum != PM_BALROG) {
 	    struct monst *dtmp;
 	    pline("Some hell-p has arrived!");
-	    if((dtmp = makemon(!rn2(6) ? &mons[ndemon()] : uasmon, u.ux, u.uy)))
+	    if((dtmp = makemon(!rn2(6) ? &mons[ndemon(u.ualign.type)] :
+					 uasmon, u.ux, u.uy)))
 		(void)tamedog(dtmp, (struct obj *)0);
 	    exercise(A_WIS, TRUE);
 	    return(0);
@@ -859,15 +864,13 @@ register struct attack *mattk;
 		    }
 		    if (stealoid) {
 			boolean stolen = FALSE;
-			/* Is "he"/"his" always correct? */
 			if (gender(mdef) == u.mfemale &&
 						uasmon->mlet == S_NYMPH)
 	You("charm %s.  She gladly hands over her possessions.", mon_nam(mdef));
 			else
-		You("seduce %s and %s starts to take off %s clothes.",
-				mon_nam(mdef),
-				gender(mdef) ? "she" : "he",
-				gender(mdef) ? "her" : "his");
+			You("seduce %s and %s starts to take off %s clothes.",
+			    mon_nam(mdef), he[pronoun_gender(mdef)],
+			    his[pronoun_gender(mdef)]);
 			while(mdef->minvent) {
 				otmp = mdef->minvent;
 				mdef->minvent = otmp->nobj;
@@ -892,7 +895,7 @@ register struct attack *mattk;
 				impossible("Player steal fails!");
 			else {
 				pline("%s finishes taking off %s suit.",
-				   Monnam(mdef), gender(mdef) ? "her" : "his");
+				      Monnam(mdef), his[pronoun_gender(mdef)]);
 				You("steal %s!", doname(stealoid));
 # if defined(ARMY) && !defined(MUSE)
 				mdef->data = &mons[PM_UNARMORED_SOLDIER];
@@ -910,6 +913,11 @@ register struct attack *mattk;
 #ifdef MUSE
 			possibly_unwield(mdef);
 			otmp->owornmask = 0L;
+			mselftouch(mdef, (const char *)0, TRUE);
+			if (mdef->mhp <= 0) {
+				tmp = 1; /* avoid early return from damageum */
+				break;
+			}
 #endif
 		   }
 		}
@@ -959,11 +967,12 @@ register struct attack *mattk;
 			int xtmp = d(2,6);
 			pline("%s suddenly seems weaker!", Monnam(mdef));
 			mdef->mhpmax -= xtmp;
-			if ((mdef->mhp -= xtmp) <= 0 || !mdef->m_lev--) {
+			if ((mdef->mhp -= xtmp) <= 0 || !mdef->m_lev) {
 				pline("%s dies!", Monnam(mdef));
 				xkilled(mdef,0);
 				return(2);
 			}
+			mdef->m_lev--;
 		}
 		tmp = 0;
 		break;
@@ -1008,9 +1017,7 @@ register struct attack *mattk;
 #ifdef MUSE
 		if ((mdef->misc_worn_check & W_ARMH) && rn2(8)) {
 		    pline("%s helmet blocks your attack to %s head.",
-			  s_suffix(Monnam(mdef)),
-			  (Blind || !humanoid(mdef->data)) ? "its" :
-				(mdef->female ? "her" : "his"));
+			  s_suffix(Monnam(mdef)), his[pronoun_gender(mdef)]);
 		    break;
 		}
 #endif
@@ -1277,7 +1284,7 @@ use_weapon:
 			/* Enemy dead, before any special abilities used */
 			if (!known_hitum(mon,dhit)) return 0;
 			/* might be a worm that gets cut in half */
-			if (m_at(u.ux+u.dx, u.uy+u.dy) != mon) return(nsum);
+			if (m_at(u.ux+u.dx, u.uy+u.dy) != mon) return((boolean)(nsum != 0));
 			/* Do not print "You hit" message, since known_hitum
 			 * already did it.
 			 */
@@ -1348,13 +1355,13 @@ use_weapon:
 			if (mon->data == &mons[PM_SHADE])
 			    Your("hug passes harmlessly through %s.",
 				mon_nam(mon));
-			else if (!sticks(mon->data))
+			else if (!sticks(mon->data) && !u.uswallow)
 			    if (mon==u.ustuck) {
 				pline("%s is being %s.", Monnam(mon),
 				    u.umonnum==PM_ROPE_GOLEM ? "choked":
 				    "crushed");
 				sum[i] = damageum(mon, mattk);
-			    } else if(sum[i-1] && sum[i-2]) {
+			    } else if(i >= 2 && sum[i-1] && sum[i-2]) {
 				You("grab %s!", mon_nam(mon));
 				u.ustuck = mon;
 				sum[i] = damageum(mon, mattk);
@@ -1404,7 +1411,7 @@ use_weapon:
 	    }
 	    if (dhit == -1)
 		rehumanize();
-	    if(sum[i] == 2) return(passive(mon, 1, 0, (mattk->aatyp==AT_KICK)));
+	    if(sum[i] == 2) return((boolean)passive(mon, 1, 0, (mattk->aatyp==AT_KICK)));
 							/* defender dead */
 	    else {
 		(void) passive(mon, sum[i], 1, (mattk->aatyp==AT_KICK));
@@ -1415,7 +1422,7 @@ use_weapon:
 	    if (multi < 0)
 		break; /* If paralyzed while attacking, i.e. floating eye */
 	}
-	return(nsum);
+	return((boolean)(nsum != 0));
 }
 
 #endif /* POLYSELF */
@@ -1527,7 +1534,7 @@ boolean kicked;
 			} else {
 			    You("are frozen by %s gaze!", 
 				  s_suffix(mon_nam(mon)));
-			    nomul((ACURR(A_WIS) > 12 || rn2(4)) ? -tmp : -120);
+			    nomul((ACURR(A_WIS) > 12 || rn2(4)) ? -tmp : -127);
 			}
 		    } else {
 			pline("%s cannot defend itself.",
