@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)zap.c	3.1	92/10/21	*/
+/*	SCCS Id: @(#)zap.c	3.1	93/06/16	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -11,28 +11,28 @@
  */
 #define MAGIC_COOKIE 1000
 
-static boolean NEARDATA obj_zapped;
-static int NEARDATA poly_zapped;
+static NEARDATA boolean obj_zapped;
+static NEARDATA int poly_zapped;
 
 #ifdef MUSE
 /* kludge to use mondied instead of killed */
 extern boolean m_using;
 #endif
 
-static boolean FDECL(obj_resists, (struct obj *));
 static boolean FDECL(obj_shudders, (struct obj *));
-static void FDECL(polyuse,(struct obj*, int, int));
+static void FDECL(polyuse, (struct obj*, int, int));
 static void FDECL(do_osshock, (struct obj *));
 static void FDECL(create_polymon, (struct obj *));
 static int FDECL(burn_floor_paper,(int,int));
-static void FDECL(cancel_item,(struct obj *));
+static void FDECL(costly_cancel, (struct obj *));
+static void FDECL(cancel_item, (struct obj *));
 static int FDECL(bhitm, (struct monst *,struct obj *));
 #ifndef MUSE
-STATIC_PTR int FDECL(bhito,(struct obj *,struct obj *));
+STATIC_PTR int FDECL(bhito, (struct obj *,struct obj *));
 #endif
-STATIC_PTR int FDECL(bhitpile,(struct obj *,int (*)(OBJ_P,OBJ_P),int,int));
-static void FDECL(backfire,(struct obj *));
-static int FDECL(zhit,(struct monst *,int,int));
+STATIC_PTR int FDECL(bhitpile, (struct obj *,int (*)(OBJ_P,OBJ_P),int,int));
+static void FDECL(backfire, (struct obj *));
+static int FDECL(zhit, (struct monst *,int,int));
 
 #define ZT_MAGIC_MISSILE	(AD_MAGM-1)
 #define ZT_FIRE			(AD_FIRE-1)
@@ -48,7 +48,7 @@ static int FDECL(zhit,(struct monst *,int,int));
 #define ZT_SPELL(x)		(10+(x))
 #define ZT_BREATH(x)	(20+(x))
 
-const char *fl[]= {
+const char *flash_types[] = {		/* also used in buzzmu(mcastu.c) */
 	"magic missile",	/* Wands must be 0-9 */
 	"bolt of fire",
 	"bolt of cold",
@@ -92,12 +92,13 @@ register struct monst *mtmp;
 register struct obj *otmp;
 {
 	register boolean wake = FALSE;
+	register int otyp = otmp->otyp;
 #ifdef MULDGN
 	boolean dbldam = (pl_character[0] == 'K') && u.uhave.questart;
 #endif
 	register int dmg;
 
-	switch(otmp->otyp) {
+	switch(otyp) {
 	case WAN_STRIKING:
 	case SPE_FORCE_BOLT:
 		wake = TRUE;
@@ -106,12 +107,11 @@ register struct obj *otmp;
 #ifdef MULDGN
 			if(dbldam) dmg *= 2;
 #endif
-			hit((otmp->otyp == WAN_STRIKING) ? "wand" :
-						"spell", mtmp, exclam(dmg));
+			hit((otyp == WAN_STRIKING) ? "wand" : "spell",
+			    mtmp, exclam(dmg));
 			(void) resist(mtmp, otmp->oclass, dmg, TELL);
-		} else miss((otmp->otyp == WAN_STRIKING) ? "wand" :
-						"spell", mtmp);
-		makeknown(otmp->otyp);
+		} else miss((otyp == WAN_STRIKING) ? "wand" : "spell", mtmp);
+		makeknown(otyp);
 		break;
 	case WAN_SLOW_MONSTER:
 	case SPE_SLOW_MONSTER:
@@ -152,14 +152,14 @@ register struct obj *otmp;
 		    if (!rn2(25)) {
 			if (canseemon(mtmp)) {
 			    pline("%s shudders!", Monnam(mtmp));
-			    makeknown(otmp->otyp);
+			    makeknown(otyp);
 			}
 			/* no corpse after system shock */
 			xkilled(mtmp, 3);
 		    }
 		    else if (newcham(mtmp, (struct permonst *)0) )
 			if (!Hallucination && (!Blind || sensemon(mtmp)))
-			    makeknown(otmp->otyp);
+			    makeknown(otyp);
 		}
 		break;
 	case WAN_CANCELLATION:
@@ -187,7 +187,7 @@ register struct obj *otmp;
 	case WAN_NOTHING:
 		break;
 	case WAN_PROBING:
-		makeknown(otmp->otyp);
+		makeknown(otyp);
 		mstatusline(mtmp);
 		break;
 	case WAN_OPENING:
@@ -197,14 +197,27 @@ register struct obj *otmp;
 				else pline("%s opens its mouth!", Monnam(mtmp));
 			}
 			expels(mtmp, mtmp->data, TRUE);
-			break;
 		}
+			break;
+	case SPE_HEALING:
+	case SPE_EXTRA_HEALING:
+		mtmp->mhp += (otyp == SPE_HEALING) ? rnd(8) : d(2,8)+2;
+		if (mtmp->mhp > mtmp->mhpmax) 
+		    mtmp->mhp = mtmp->mhpmax;
+		if (canseemon(mtmp))
+		    pline(otyp == SPE_HEALING ? "%s begins to look better." :
+			  "%s looks much better.", Monnam(mtmp));
+		if (mtmp->mtame || mtmp->mpeaceful) {		    
+		    adjalign((pl_character[0] == 'H') ? 1 :
+			     sgn(u.ualign.type));
+		}
+ 		break;
 	case WAN_LOCKING:
 	case SPE_KNOCK:
 	case SPE_WIZARD_LOCK:
 		break;
 	default:
-		impossible("What an interesting effect (%u)", otmp->otyp);
+		impossible("What an interesting effect (%u)", otyp);
 	}
 	if(wake) {
 	    if(mtmp->mhp > 0) {
@@ -267,7 +280,34 @@ boolean ininv;
 	return mtmp;
 }
 
-static const char NEARDATA charged_objs[] = { WAND_CLASS, WEAPON_CLASS, ARMOR_CLASS, 0 };
+static NEARDATA const char charged_objs[] = { WAND_CLASS, WEAPON_CLASS, ARMOR_CLASS, 0 };
+
+static void
+costly_cancel(obj)
+register struct obj *obj;
+{
+	register struct monst *shkp = (struct monst *)0;
+
+	if (carried(obj)) {
+		if (obj->unpaid) {
+	            shkp = shop_keeper(*in_rooms(u.ux, u.uy, SHOPBASE));
+		    if (!shkp) return;
+		    Norep("You cancel an unpaid object, you pay for it!");
+		    bill_dummy_object(obj);
+		}
+	} else {
+	        shkp = shop_keeper(*in_rooms(obj->ox, obj->oy, SHOPBASE));
+		if (!shkp || !inhishop(shkp)) return;
+		if (!costly_spot(obj->ox, obj->oy)) return;
+		if (costly_spot(u.ux, u.uy) && 
+				*in_rooms(u.ux, u.uy, 0) ==
+					*in_rooms(shkp->mx, shkp->my, 0)) {
+		    Norep("You cancel it, you pay for it!");
+		    bill_dummy_object(obj);
+		} else
+		    (void) stolen_value(obj, obj->ox, obj->oy, FALSE, FALSE);
+	}
+}
 
 /* cancel obj, possibly carried by you or a monster */
 static void
@@ -275,6 +315,8 @@ cancel_item(obj)
 register struct obj *obj;
 {
 	boolean	u_ring = (obj == uleft) || (obj == uright);
+	register boolean unpaid = (carried(obj) && obj->unpaid);
+	register boolean holy = (obj->otyp == POT_WATER && obj->blessed);
 
 	switch(obj->otyp) {
 		case RIN_GAIN_STRENGTH:
@@ -320,47 +362,59 @@ register struct obj *obj;
 	    obj->otyp == SLIME_MOLD ||
 #endif
 	    obj->otyp == SKELETON_KEY ||
-	    obj->otyp == LARGE_BOX || obj->otyp == CHEST))
+	    obj->otyp == LARGE_BOX || obj->otyp == CHEST ||
+	    obj->otyp == OIL_LAMP || obj->otyp == BRASS_LANTERN ||
+	    Is_candle(obj) || obj->otyp == CANDELABRUM_OF_INVOCATION)) {
+	        costly_cancel(obj);
 		obj->spe = (obj->oclass == WAND_CLASS) ? -1 : 0;
-
-	if (obj->otyp == OIL_LAMP || obj->otyp == BRASS_LANTERN
-	    || Is_candle(obj) || obj->otyp == CANDELABRUM_OF_INVOCATION) {
-
-	    /* reducing a candle's age to 0 is */
-	    /* the same as destroying it.      */
-	    if (!Is_candle(obj)) obj->age = 0;
-
-	    /* reducing a candelabrum age to 0 */
-	    /* is the same as de-candling it.  */
-	    if(obj->otyp == CANDELABRUM_OF_INVOCATION) obj->spe = 0;
-
-	    obj->lamplit = 0;
-	    check_lamps();
-	}
-
+		if (unpaid) addtobill(obj, TRUE, FALSE, TRUE);
+	    }
 	if (obj->oclass == SCROLL_CLASS
+	    && obj->otyp != SCR_BLANK_PAPER
 #ifdef MAIL
 	    && obj->otyp != SCR_MAIL
 #endif
-	   )
+	   ) {
+	    costly_cancel(obj);
 	    obj->otyp = SCR_BLANK_PAPER;
-
-	if (obj->oclass == SPBOOK_CLASS && obj->otyp != SPE_BOOK_OF_THE_DEAD)
+	    if (unpaid) addtobill(obj, TRUE, FALSE, TRUE);
+        }
+	if (obj->oclass == SPBOOK_CLASS &&
+	                   obj->otyp != SPE_BOOK_OF_THE_DEAD &&
+	                   obj->otyp != SPE_BLANK_PAPER) {
+	    costly_cancel(obj);
 	    obj->otyp = SPE_BLANK_PAPER;
-
-	if (obj->oclass == POTION_CLASS && obj->otyp != POT_BOOZE)
-	    obj->otyp = (obj->otyp==POT_SICKNESS || obj->otyp==POT_SEE_INVISIBLE || obj->otyp==POT_FRUIT_JUICE) ? POT_FRUIT_JUICE : POT_WATER;
+	    if (unpaid) addtobill(obj, TRUE, FALSE, TRUE);
+	}
+	if (obj->oclass == POTION_CLASS && obj->otyp != POT_BOOZE) {
+	    if (obj->otyp==POT_SICKNESS ||
+	                 obj->otyp==POT_SEE_INVISIBLE) {
+	            costly_cancel(obj);
+	            obj->otyp = POT_FRUIT_JUICE;
+		    if (unpaid) addtobill(obj, TRUE, FALSE, TRUE);
+	    } else {
+	            if (obj->otyp != POT_FRUIT_JUICE &&
+			                     obj->otyp != POT_WATER) {
+		        costly_cancel(obj);
+	                obj->otyp = POT_WATER;
+			if (unpaid) addtobill(obj, TRUE, FALSE, TRUE);
+		    }
+	    }
 	    /* sickness is "biologically contaminated" fruit juice; cancel it
 	     * and it just becomes fruit juice... whereas see invisible
 	     * tastes like "enchanted" fruit juice, it similarly cancels.
 	     */
+	}
+	if (holy) costly_cancel(obj);
 	unbless(obj);
+	if (unpaid && holy) addtobill(obj, TRUE, FALSE, TRUE);
 	uncurse(obj);
 }
 
-static boolean
-obj_resists(obj)
+boolean
+obj_resists(obj, ochance, achance)
 struct obj *obj;
+int ochance, achance;	/* percent chance for ordinary objects, artifacts */
 {
 	if (obj->otyp == AMULET_OF_YENDOR ||
 	    obj->otyp == SPE_BOOK_OF_THE_DEAD ||
@@ -369,9 +423,9 @@ struct obj *obj;
 	    (obj->otyp == CORPSE && is_rider(&mons[obj->corpsenm]))) {
 		return TRUE;
 	} else {
-		int chance = rn2(20);
-		/* artifacts resist 95% of the time; normal objects 5% */
-		return (obj->oartifact ? !!chance : !chance);
+		int chance = rn2(100);
+
+		return((boolean)(chance < (obj->oartifact ? achance : ochance)));
 	}
 }
 
@@ -393,7 +447,7 @@ struct obj *obj;
 	/* adjust for "large" quantities of identical things */
 	if(obj->quan > 4L) zap_odds /= 2;
 
-	return (! rn2(zap_odds));
+	return((boolean)(! rn2(zap_odds)));
 }
 
 /* Use up at least minwt number of things made of material mat.
@@ -434,6 +488,7 @@ create_polymon(obj)
 {
 	struct permonst *mdat = (struct permonst *)0;
 	struct monst *mtmp;
+	const char *material;
 	int pm_index;
 
 	/* no golems if you zap only one object -- not enough stuff */
@@ -445,6 +500,7 @@ create_polymon(obj)
 	case METAL:
 	case MITHRIL:
 	    pm_index = PM_IRON_GOLEM;
+	    material = "metal ";
 	    break;
 	case COPPER:
 	case SILVER:
@@ -454,23 +510,29 @@ create_polymon(obj)
 	case GLASS:
 	case MINERAL:
 	    pm_index = rn2(2) ? PM_STONE_GOLEM : PM_CLAY_GOLEM;
+	    material = "lithic ";
 	    break;
 	case 0:
 	    /* there is no flesh type, but all food is type 0, so we use it */
 	    pm_index = PM_FLESH_GOLEM;
+	    material = "organic ";
 	    break;
 	case WOOD:
 	    pm_index = PM_WOOD_GOLEM;
+	    material = "wood ";
 	    break;
 	case LEATHER:
 	    pm_index = PM_LEATHER_GOLEM;
+	    material = "leather ";
 	    break;
 	case CLOTH:
 	    pm_index = PM_ROPE_GOLEM;
+	    material = "cloth ";
 	    break;
 	default:
 	    /* if all else fails... */
 	    pm_index = PM_STRAW_GOLEM;
+	    material = "";
 	    break;
 	}
 
@@ -481,8 +543,8 @@ create_polymon(obj)
 	polyuse(obj, poly_zapped, (int)mons[pm_index].cwt);
 
 	if(!Blind && mtmp) {
-	    pline("Some objects in the pile merge.");
-	    pline("%s rises from the pile!", Amonnam(mtmp));
+	    pline("Some %sobjects meld, and %s arises from the pile!",
+		  material, a_monnam(mtmp));
 	}
 }
 
@@ -532,13 +594,20 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 	register int res = 1;
 	struct obj *otmp2;
 
-	if(obj == uball || obj == uchain)
+	if (obj == uball) {
 		res = 0;
-	else
+	} else if (obj == uchain) {
+		if (otmp->otyp == WAN_OPENING || otmp->otyp == SPE_KNOCK) {
+		    unpunish();
+		    res = 1;
+		    makeknown(otmp->otyp);
+		} else
+		    res = 0;
+	} else
 	switch(otmp->otyp) {
 	case WAN_POLYMORPH:
 	case SPE_POLYMORPH:
-		if (obj_resists(obj)) {
+		if (obj_resists(obj, 5, 95)) {
 		    res = 0;
 		    break;
 		} else if (obj_shudders(obj)) {
@@ -580,6 +649,10 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 		otmp2->oeroded = obj->oeroded;
 		otmp2->oerodeproof = obj->oerodeproof;
 
+		/* reduce spellbook abuse */
+		if (obj->oclass == SPBOOK_CLASS)
+		    otmp2->spestudied = obj->spestudied + 1;
+
 		/* Keep chest/box traps and poisoned ammo if we may */
 		if (obj->otrapped && Is_box(otmp2))
 			otmp2->otrapped = TRUE;
@@ -593,6 +666,7 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 			otmp2->otyp = LOW_BOOTS;
 			otmp2->oclass = ARMOR_CLASS;
 			otmp2->spe = 0;
+			otmp2->oeroded = 0;
 			otmp2->oerodeproof = TRUE;
 			otmp2->quan = 1L;
 			otmp2->cursed = FALSE;
@@ -600,7 +674,7 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 		}
 
 		/* no box contents --KAA */
-		if (Is_container(otmp2) || otmp2->otyp == STATUE)
+		if (Has_contents(otmp2))
 			delete_contents(otmp2);
 
 		/* 'n' merged objects may be fused into 1 object */
@@ -623,7 +697,7 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 		          shop_keeper(*in_rooms(obj->ox, obj->oy, SHOPBASE));
 
 		    if ((!obj->no_charge ||
-			 (Is_container(obj) &&
+			 (Has_contents(obj) &&
 			    (contained_cost(obj, shkp, 0L, FALSE) != 0L)))
 		       && inhishop(shkp)) {
 		        if(shkp->mpeaceful) {
@@ -646,13 +720,26 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 			fracture_rock(obj);
 		else if (obj->otyp == STATUE)
 			(void) break_statue(obj);
-		else
+		else {
+			(void)breaks(obj, FALSE);
 			res = 0;
+		}
 		makeknown(otmp->otyp);
 		break;
+	case WAN_DIGGING:
+	case SPE_DIG:
+		/* vaporize boulders */
+		if (obj->otyp == BOULDER) {
+			delobj(obj);
+			res = 0;
+		}
+		break;
 	case WAN_CANCELLATION:
 	case SPE_CANCELLATION:
 		cancel_item(obj);
+#ifdef TEXTCOLOR
+		newsym(obj->ox,obj->oy);	/* might change color */
+#endif
 		break;
 	case WAN_TELEPORTATION:
 	case SPE_TELEPORT_AWAY:
@@ -668,28 +755,22 @@ register struct obj *obj, *otmp;	/* returns TRUE if sth was done */
 		break;
 	case WAN_OPENING:
 	case SPE_KNOCK:
-		/* Zap it at the chain, not the ball */
-		if (obj == uchain) {
-			unpunish();
-			res = 1;
-			makeknown(obj->otyp);
-			break;
-		}
-		/* fall through */
 	case WAN_LOCKING:
 	case SPE_WIZARD_LOCK:
 		if(Is_box(obj))
 			res = boxlock(obj, otmp);
 		else
 			res = 0;
-		if (res /* && obj->oclass == WAND_CLASS */)
-			makeknown(obj->otyp);
+		if (res /* && otmp->oclass == WAND_CLASS */)
+			makeknown(otmp->otyp);
 		break;
 	case WAN_SLOW_MONSTER:		/* no effect on objects */
 	case SPE_SLOW_MONSTER:
 	case WAN_SPEED_MONSTER:
 	case WAN_NOTHING:
 	case WAN_PROBING:
+	case SPE_HEALING:
+	case SPE_EXTRA_HEALING:
 		res = 0;
 		break;
 	default:
@@ -744,7 +825,7 @@ register struct obj *wand;
 	if(wand->spe < 0 || (wand->spe == 0 && rn2(121)))
 		return 0;
 	if(wand->spe == 0)
-		You("wrest one more spell from the worn-out wand.");
+		You("wrest one last charge from the worn-out wand.");
 	wand->spe--;
 	return 1;
 }
@@ -798,7 +879,7 @@ backfire(otmp)
 	useup(otmp);
 }
 
-static const char NEARDATA zap_syms[] = { WAND_CLASS, 0 };
+static NEARDATA const char zap_syms[] = { WAND_CLASS, 0 };
 
 int
 dozap()
@@ -830,8 +911,7 @@ dozap()
 		weffects(obj);
 	}
 	if (obj->spe < 0) {
-	    pline ("%s %sturns to dust.",
-		   The(xname(obj)), Blind ? "" : "glows violently, then ");
+	    pline("%s turns to dust.", The(xname(obj)));
 	    useup(obj);
 	}
 	return(1);
@@ -850,7 +930,7 @@ zapyourself(obj)
 			shieldeff(u.ux, u.uy);
 			pline("Boing!");
 		    } else {
-			You("magically bash yourself!");
+			You("bash yourself!");
 			damage=d(8,6);
 			exercise(A_STR, FALSE);
 		    }
@@ -859,7 +939,7 @@ zapyourself(obj)
 		case WAN_LIGHTNING:
 		    makeknown(WAN_LIGHTNING);
 		    if (!Shock_resistance) {
-			pline("Idiot!  You've shocked yourself!");
+			You("shock yourself!");
 			damage = d(12,6);
 			exercise(A_CON, FALSE);
 		    } else {
@@ -883,15 +963,16 @@ zapyourself(obj)
 		case WAN_FIRE:
 		    makeknown(WAN_FIRE);
 		case FIRE_HORN:
-		    pline("You've set light to yourself!");
 		    if (Fire_resistance) {
 			shieldeff(u.ux, u.uy);
-			You("feel mildly hot.");
+			You("feel rather warm.");
 #ifdef POLYSELF
 			ugolemeffects(AD_FIRE, d(12,6));
 #endif
-		    } else
+		    } else {
+			pline("You've set yourself afire!");
 			damage = d(12,6);
+		    }
 		    destroy_item(SCROLL_CLASS, AD_FIRE);
 		    destroy_item(POTION_CLASS, AD_FIRE);
 		    destroy_item(SPBOOK_CLASS, AD_FIRE);
@@ -902,7 +983,7 @@ zapyourself(obj)
 		case FROST_HORN:
 		    if (Cold_resistance) {
 			shieldeff(u.ux, u.uy);
-			You("feel mildly chilly.");
+			You("feel a little chill.");
 #ifdef POLYSELF
 			ugolemeffects(AD_COLD, d(12,6));
 #endif
@@ -938,7 +1019,7 @@ zapyourself(obj)
 		case SPE_CANCELLATION:
 		    cancel_monst(&youmonst, obj, TRUE, FALSE, TRUE);
 		    break;
-	       case WAN_MAKE_INVISIBLE: {
+		case WAN_MAKE_INVISIBLE: {
 		    /* have to test before changing HInvis but must change
 		     * HInvis before doing newsym().
 		     */
@@ -954,7 +1035,7 @@ zapyourself(obj)
 		    }
 		    break;
 		}
-	       case WAN_SPEED_MONSTER:
+		case WAN_SPEED_MONSTER:
 		    if (!(Fast & INTRINSIC)) {
 			You("seem to be moving faster.");
 			makeknown(WAN_SPEED_MONSTER);
@@ -962,7 +1043,7 @@ zapyourself(obj)
 		    }
 		    Fast |= FROMOUTSIDE;
 		    break;
-	       case WAN_SLEEP:
+		case WAN_SLEEP:
 		    makeknown(WAN_SLEEP);
 		case SPE_SLEEP:
 		    if(Sleep_resistance) {
@@ -1015,9 +1096,16 @@ zapyourself(obj)
 		    }
 #endif
 		    break;
+		case SPE_HEALING:
+		case SPE_EXTRA_HEALING:
+		    healup(obj->otyp == SPE_HEALING ? rnd(8) : d(2,8)+2,
+			   0, FALSE, FALSE);
+		    You(obj->otyp == SPE_HEALING ? "begin to feel better." :
+			"feel a fair bit better.");
+		    break;
+		case WAN_DIGGING:
 		case SPE_DIG:
 		case SPE_DETECT_UNSEEN:
-		case WAN_DIGGING:
 		case WAN_NOTHING:
 		case WAN_OPENING:
 		case WAN_LOCKING:
@@ -1171,7 +1259,7 @@ register struct	obj	*obj;
 				    stackobj(fobj);
 				    if(Invisible) newsym(u.ux, u.uy);
 				} else {
-				    dighole();
+				    (void) dighole(FALSE);
 				}
 			    }
 			    break;
@@ -1389,6 +1477,15 @@ struct obj *obj;			/* object tossed/used */
 			    break;
 		    }
 
+		if (weapon == THROWN_WEAPON || weapon == KICKED_WEAPON) {
+		    /* can't hit monsters/objects in rock w/solid weapons */
+		    /* but beams/zaps _can_, so we need an extra pre-check */
+		    if(!ZAP_POS(typ) || closed_door(bhitpos.x, bhitpos.y)) {
+			bhitpos.x -= ddx;
+			bhitpos.y -= ddy;
+			break;
+		    }
+		}
 		if ((mtmp = m_at(bhitpos.x, bhitpos.y)) != 0) {
 			if(weapon != ZAPPED_WAND) {
 				if(weapon != INVIS_BEAM) tmp_at(DISP_END, 0);
@@ -1495,7 +1592,7 @@ int dx, dy;
 				break;
 			} else {	/* we catch it */
 				tmp_at(DISP_END, 0);
-				pline("Skillfully, you catch the boomerang.");
+				pline("You skillfully catch the boomerang.");
 				return(&youmonst);
 			}
 		}
@@ -1634,7 +1731,9 @@ int x, y;
 	    /* Bug fix - KAA */
 	    if(obj->oclass == SCROLL_CLASS
 			|| obj->oclass == SPBOOK_CLASS) {
-		if (obj->otyp == SCR_FIRE || obj->otyp == SPE_FIREBALL)
+		if (obj->otyp == SCR_FIRE || obj->otyp == SPE_FIREBALL
+			|| obj->otyp == SPE_BOOK_OF_THE_DEAD
+		        || obj->oartifact)
 		    continue;
 		scrquan = obj->quan;
 		for(i = 1; i <= scrquan ; i++)
@@ -1649,10 +1748,10 @@ int x, y;
 	return(cnt);
 }
 
-/* type == 0 to 9     : you shooting a wand */
-/* type == 10 to 19   : you casting a spell */
-/* type == 20 to 29   : you breathing as a monster */
-/* type == -10 to -19   : monster casting spell */
+/* type ==   0 to   9 : you shooting a wand */
+/* type ==  10 to  19 : you casting a spell */
+/* type ==  20 to  29 : you breathing as a monster */
+/* type == -10 to -19 : monster casting spell */
 /* type == -20 to -29 : monster breathing at you */
 /* type == -30 to -39 : monster shooting a wand (MUSE only) */
 /* called with dx = dy = 0 with vertical bolts */
@@ -1667,10 +1766,12 @@ register int dx,dy;
     register xchar lsx, lsy;
     struct monst *mon;
     boolean bodyhit = FALSE, shopdamage = FALSE;
+    register const char *fltxt;
+
 #ifdef MUSE
-    register const char *fltxt = fl[(type <= -30) ? abstype : abs(type)];
+    fltxt = flash_types[(type <= -30) ? abstype : abs(type)];
 #else
-    register const char *fltxt = fl[abs(type)];
+    fltxt = flash_types[abs(type)];
 #endif
     if(u.uswallow) {
 	register int tmp;
@@ -1711,17 +1812,12 @@ register int dx,dy;
 	    if (type >= 0) mon->data->mflags3 &= ~M3_WAITMASK;
 	    if (rnd(20) < 18 + find_mac(mon)) {
 #ifdef MUSE
-		struct obj *oshld = which_armor(mon, W_ARMS);
-
-		if (oshld && oshld->otyp == SHIELD_OF_REFLECTION) {
+		if (mon_reflects(mon, "")) {
 		    if(cansee(mon->mx,mon->my)) {
 			hit(fltxt, mon, exclam(0));
-			pline("But it reflects from %s shield!",
-			      s_suffix(mon_nam(mon)));
-			makeknown(SHIELD_OF_REFLECTION);
-			shieldeff(sx, sy);
+			shieldeff(mon->mx, mon->my);
+			(void) mon_reflects(mon, "But it reflects from %s %s!");
 		    }
-
 		    dx = -dx;
 		    dy = -dy;
 		} else
@@ -1734,10 +1830,10 @@ register int dx,dy;
 			    hit(fltxt, mon, exclam(tmp));
 			    pline("%s disintegrates.", Monnam(mon));
 			    if(Blind)
-		       You("sense the fragments of %s body reassembling!",
+		       You("hear the fragments of %s body reassembling!",
 			         s_suffix(mon_nam(mon)));
 			    else
-		       pline("%s body fragments reassemble before your %s!",
+		       pline("%s body reintegrates before your %s!",
 			         s_suffix(Monnam(mon)),
 			         makeplural(body_part(EYE)));
 		            pline("%s resurrects!", Monnam(mon));
@@ -1745,8 +1841,7 @@ register int dx,dy;
 		        mon->mhp = mon->mhpmax;
 			break; /* Out of while loop */
 		    }
-		    if(mon->data == &mons[PM_DEATH] &&
-				            abs(type)%10 == ZT_DEATH) {
+		    if(mon->data == &mons[PM_DEATH] && abstype == ZT_DEATH) {
 		        if(cansee(mon->mx,mon->my)) {
 			    hit(fltxt, mon, exclam(tmp));
 		            pline("Death absorbs the deadly %s!",
@@ -1908,8 +2003,9 @@ register int dx,dy;
 		    else if(Antimagic) {
 			shieldeff(sx, sy);
 			You("aren't affected.");
-		    } else
-			u.uhp = -1;
+			break;
+		    }
+		    u.uhp = -1;
 		    break;
 		case ZT_LIGHTNING:
 		    if (Shock_resistance) {
@@ -1969,7 +2065,7 @@ register int dx,dy;
 	    }
 	    bounce = 0;
 	    range--;
-	    if(range && cansee(lsx,lsy))
+	    if(range && isok(lsx, lsy) && cansee(lsx,lsy))
 		pline("%s bounces!", The(fltxt));
 	    if(!dx || !dy || !rn2(20)) {
 		dx = -dx;
@@ -2003,6 +2099,30 @@ register int dx,dy;
 	   	       abstype == ZT_DEATH ? "disintegrate" : "destroy");
 }
 
+void
+melt_ice(x, y)
+xchar x, y;
+{
+	struct rm *lev = &levl[x][y];
+
+	if (lev->typ == DRAWBRIDGE_UP)
+	    lev->drawbridgemask &= ~DB_ICE;	/* revert to DB_MOAT */
+	else {	/* lev->typ == ICE */
+#ifdef STUPID
+	    if (lev->icedpool == ICED_POOL) lev->typ = POOL;
+	    else lev->typ = MOAT;
+#else
+	    lev->typ = (lev->icedpool == ICED_POOL ? POOL : MOAT);
+#endif
+	    lev->icedpool = 0;
+	}
+	unearth_objs(x, y);
+	newsym(x,y);
+	if (cansee(x,y)) Norep("The ice crackles and melts.");
+	if (x == u.ux && y == u.uy)
+		spoteffects();	/* possibly drown, notice objects */
+}
+
 /* Burn floor scrolls, evaporate pools, etc...  in a single square.  Used
  * both for normal bolts of fire, cold, etc... and for fireballs.
  * Sets shopdamage to TRUE if a shop door is destroyed, and returns the 
@@ -2021,23 +2141,9 @@ boolean *shopdamage;
 
 	if(abstype == ZT_FIRE) {
 	    if(is_ice(x, y)) {
-		if (lev->typ == DRAWBRIDGE_UP)
-		    lev->drawbridgemask &= ~DB_ICE;	/* revert to DB_MOAT */
-		else {	/* lev->typ == ICE */
-#ifdef STUPID
-		    if (lev->icedpool == ICED_POOL) lev->typ = POOL;
-		    else lev->typ = MOAT;
-#else
-		    lev->typ = (lev->icedpool == ICED_POOL ? POOL : MOAT);
-#endif
-		    lev->icedpool = 0;
-		}
-		newsym(x,y);
-		Norep("The ice crackles and melts.");
-		if (x == u.ux && y == u.uy)
-			spoteffects();	/* possibly drown */
+		melt_ice(x, y);
 	    } else if(is_pool(x,y)) {
-		const char *msgtxt = "You hear a hissing sound.";
+		const char *msgtxt = "You hear hissing gas.";
 		if(lev->typ != POOL) {	/* MOAT or DRAWBRIDGE_UP */
 		    if (cansee(x,y)) msgtxt = "Some water evaporates.";
 		} else {
@@ -2046,7 +2152,7 @@ boolean *shopdamage;
 		    rangemod -= 3;
 		    lev->typ = ROOM;
 		    ttmp = maketrap(x, y, PIT);
-		    ttmp->tseen = 1;
+		    if (ttmp) ttmp->tseen = 1;
 		    if (cansee(x,y)) msgtxt = "The water evaporates.";
 		}
 		Norep(msgtxt);
@@ -2065,11 +2171,12 @@ boolean *shopdamage;
 		    if (cansee(x,y))
 			pline("The water freezes for a moment.");
 		    else
-			You("hear a soft crackling sound");
+			You("hear a soft crackling.");
 		    rangemod -= 1000;	/* stop */
 		} else {
 		    rangemod -= 3;
 		    if (lev->typ == DRAWBRIDGE_UP) {
+			lev->drawbridgemask &= ~DB_UNDER;  /* clear lava */
 			lev->drawbridgemask |= (lava ? DB_FLOOR : DB_ICE);
 		    } else {
 			if (!lava)
@@ -2077,14 +2184,7 @@ boolean *shopdamage;
 				    (lev->typ == POOL ? ICED_POOL : ICED_MOAT);
 			lev->typ = (lava ? ROOM : ICE);
 		    }
-		    /*
-		     * Not perfect: There could be stuff on the bottom.  If
-		     * we leave it here, it'll pop to the surface which is
-		     * strange, so just destroy all the objects at this location.
-		     * A better solution requires major changes (as usual).
-		     */
-		    if(OBJ_AT(x,y))
-			delallobj(x,y);
+		    bury_objs(x,y);
 		    if(cansee(x,y)) {
 			if(moat)
 				Norep("The moat is bridged with ice!");
@@ -2155,7 +2255,7 @@ boolean *shopdamage;
 			newsym(x,y);
 		    }
 		    else if(flags.soundok)
-			You("hear a crashing sound.");
+			You("hear crashing wood.");
 		    break;
 		case ZT_LIGHTNING:
 		    if(type >= 0 && *in_rooms(x, y, SHOPBASE)) {
@@ -2169,7 +2269,7 @@ boolean *shopdamage;
 			newsym(x,y);
 		    }
 		    else if(flags.soundok)
-			You("hear a crackling sound.");
+			You("hear crackling.");
 		    break;
 		default:
 		def_case:
@@ -2188,6 +2288,8 @@ boolean *shopdamage;
 		    newsym(x,y);
 		    if(!Blind)
 			You("see a puff of smoke.");
+		    else
+			You("smell a whiff of smoke.");
 		}
 	if ((mon = m_at(x,y)) != 0) {
 		/* Cannot use wakeup() which also angers the monster */
@@ -2218,19 +2320,23 @@ register struct obj *obj;
 	} while(!goodpos(tx,ty,(struct monst *)0, (struct permonst *)0));
 	freeobj(obj);
 	if (flooreffects(obj, tx, ty, "fall"))
- 		return;
-	if(costly_spot(otx, oty) && (!costly_spot(tx, ty) ||
-				     !index(in_rooms(tx, ty, 0),
-					    *in_rooms(otx, oty, 0)))) {
+		return;
+	if (otx == 0 && oty == 0) {
+	    ;	/* fell through a trapdoor; no update of old loc needed */
+	} else {
+	    if (costly_spot(otx, oty)
+	      && (!costly_spot(tx, ty) ||
+		  !index(in_rooms(tx, ty, 0), *in_rooms(otx, oty, 0)))) {
 		if(costly_spot(u.ux, u.uy) &&
 			    index(u.urooms, *in_rooms(otx, oty, 0)))
 		    addtobill(obj, FALSE, FALSE, FALSE);
 		else (void)stolen_value(obj, otx, oty, FALSE, FALSE);
+	    }
+	    newsym(otx, oty);	/* update old location */
 	}
 	obj->nobj = fobj;
 	fobj = obj;
 	place_object(obj, tx, ty);
-	newsym(otx, oty);
 	newsym(tx,ty);
 }
 
@@ -2575,11 +2681,12 @@ retry:
 		/* The(aobjnam()) is safe since otmp is unidentified -dlc */
 		(void) hold_another_object(otmp, u.uswallow ?
 				       "Oops!  %s out of your reach!" :
-				       Is_airlevel(&u.uz) ?
-				       "Oops!  %s out of your grasp!" :
+				       Is_airlevel(&u.uz) || u.uinwater ?
+				       "Oops!  %s away from you!" :
 				       "Oops!  %s to the floor!",
-				       The(aobjnam(otmp, Is_airlevel(&u.uz) ?
-					       "slip" : "drop")),
+				       The(aobjnam(otmp,
+					     Is_airlevel(&u.uz) || u.uinwater ?
+						   "slip" : "drop")),
 				       (const char *)0);
 	    u.ublesscnt += rn1(100,50);  /* the gods take notice */
 	}
