@@ -11,7 +11,8 @@
  * interface; not #defining it supports (along with wb.c) the WB interface.
  */
 
-#include "Incl:date.h"          /* this gives us the version string */
+/*#include "Incl:date.h"          /* this gives us the version string */
+#include "Incl:patchlevel.h"	/* and the individual bits */
 
 #ifdef AZTEC_C
 /* Aztec doesn't recognize __chip syntax */
@@ -52,6 +53,9 @@ void s_UnLoadSeg(void);
 #ifdef CLI
 char *cnfsavedir="NetHack:save";    /* unless overridden in cnf file */
 char argline[255];  /* no overflow - bigger than ADOS will pass */
+#ifdef for_later
+int amibbs=0;				/* BBS mode flag */
+#endif
 
 void WaitEOG(GPTR);
 char *eos(char *);
@@ -63,7 +67,6 @@ int running_split=0;        /* if 0, using normal LoadSeg/UnLoadSeg */
 #else
 extern char *options[NUMIDX+1];
 extern GPTR gamehead,gameavail;
-extern DEFAULTS defgame;
 extern struct Window *win;
 #endif  /* CLI */
 
@@ -123,45 +126,74 @@ main( argc, wbs )
     GPTR gptr;
     BPTR lc,lc2;
     struct FileInfoBlock finfo;
-    char *name;
+    char *name=0;
     char namebuf[50];
     char **argv=(char **)wbs;
+    char newcmdline[80]="";
+    char forcenewcmd=0;
 
     ZapOptions( curopts );
     InitWB( argc, (struct WBStartup *)argv );
-    errmsg( NO_FLASH, "Welcome to NetHack Version 3.1!\n" );
+    errmsg( NO_FLASH, "Welcome to NetHack Version %d.%d.%d!\n",
+      VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL );
     CopyRight( );
 
     ReadConfig( );
 
-    /* Wait till user quits */
+#ifdef for_later
+    /* check for BBS mode */
+    if (argc>1 && argv[1][0]==':'){
+	amibbs=1;
+	strcpy(newcmdline,":");
+	amibasename= &argv[1][1];
+    }
+#endif
 
+		/* check/re-assemble initial command line */
+    {
+    int c;
+				/* slow but easy - not a critical path */
+    for(c=1;c<argc;c++){
+	if(strlen(newcmdline)+strlen(argv[c])>=sizeof(newcmdline)){
+	    forcenewcmd=1;
+	    break;
+	}
+	strcpy(eos(newcmdline),argv[c]);
+	if(!strncmp(argv[c],"-u",2)){
+	    if(!strcmp(argv[c],"-u")){
+		name= argv[c+1];
+	    }else{
+		name= &argv[c][2];
+	    }
+	    if(!name){
+		errmsg(NO_FLASH, "No name found.\n");
+		newcmdline[0]='\0';	/* don't leave -u as default */
+		forcenewcmd=1;
+	    }
+	}
+	if(c<argc)strcpy(eos(newcmdline)," ");
+    }
+    eos(newcmdline)[-1]='\0';
+    strcpy(argline,newcmdline);
+    }
+
+    /* Wait till user quits */
     while( !quit )
     {
-	char tbuf[8];
+	char tbuf[80];
 	char *p=tbuf;
 	char *dirname=cnfsavedir;
 /* play a game */
-	{
-	    int c;
-	    argline[0]='\0';
-	    name="NewGame.info";        /* this will fail - it's in NetHack: */
-	    for(c=1;c<argc;c++){
-		/* slow but easy - not a critical path */
-		strcpy(eos(argline),argv[c]);
-		if(!strncmp(argv[c],"-u",2)){
-		    if(!strcmp(argv[c],"-u")){
-			name= argv[c+1];
-		    }else{
-			name= &argv[c][1];
-		    }
-		}
-		if(c<argc)strcpy(eos(argline)," ");
-	    }
-	    eos(argline)[-1]='\0';
-	}
+
+	if(forcenewcmd)
+	    goto build_new_argline;	/* allow initial args to be wrong */
+#undef TESTCMDLINE
+#ifdef TESTCMDLINE
+__builtin_printf("sending '%s'\n",argline);
+#else
 	strcpy(namebuf,cnfsavedir);
 	condaddslash(namebuf);
+	if(!name)name="NewGame.info";
 	strcpy(eos(namebuf),name);
 	lc=Lock(namebuf,ACCESS_READ);
 	if(!lc){
@@ -170,28 +202,52 @@ main( argc, wbs )
 	    strcpy(eos(namebuf),"NewGame.info");
 	    lc=Lock(namebuf,ACCESS_READ);
 	    if(!lc){
-		errmsg(NO_FLASH,"can't find NewGame.info");
+		errmsg(NO_FLASH,"Can't find NewGame.info");
 		cleanup(1);
 	    }
 	}
-	Examine(lc,&finfo);
+	if(!Examine(lc,&finfo)){
+	    errmsg(NO_FLASH,"Can't find info file.\n");
+	    cleanup(1);
+	}
 	lc2=ParentDir(lc);
 	UnLock(lc);
 	gptr=GetWBIcon(lc2,dirname,&finfo);
+	if(!gptr)cleanup(1);
 	UnLock(lc2);
 	run_game(gptr);
 
 /* wait for game to end */
-	WaitEOG(gptr);
+	if(gptr->wbs)
+	    WaitEOG(gptr);
+	/* else load failed */
+
 	FreeGITEM(gptr);
+#endif /* TESTCMDLINE */
 /* ask about another? */
-	printf("Play again? [yn] ");
+build_new_argline:
+	forcenewcmd=0;
+#ifdef for_later
+	if(amibbs) {
+	    quit = 1;		/* bbs mode aborts after one game */
+	} else
+#endif
+	{
+	char *x=argline;
+	while(isspace(*x))x++;
+	if(*x){			/* non-blank argline */
+		printf("%s %s %s",
+		  "Enter options for next game.  Default:\n\t", argline,
+		  "\n(space return to clear) or Q to quit:\n");
+	} else {
+		printf("Enter options for next game or Q to quit:\n");
+	}
 	fgets(tbuf,sizeof(tbuf),stdin);
-	while(*p && isspace(*p))p++;
-	switch(*p){
-	case 'n':
-	case 'N':
+	tbuf[strlen(tbuf)-1]='\0';		/* kill \n */
+	if(strlen(tbuf)==1 && (*p=='q' || *p=='Q')){
 	    quit=1;
+		} else
+	    if(strlen(tbuf))strcpy(argline,tbuf);
 	}
     }
     cleanup(0);
@@ -269,6 +325,7 @@ InitWB( argc, wbs )
     int argc;
     register struct WBStartup *wbs;
 {
+    char **argv=(char **)wbs;
 
     /* Open Libraries */
     GfxBase= (struct GfxBase *) OldOpenLibrary("graphics.library");
@@ -297,6 +354,38 @@ InitWB( argc, wbs )
 	Delay(400);
 	cleanup(1);
     }
+/* we should include hack.h but due to conflicting options to sc
+ * we can't parse tradstdc.h and clib/graphics_protos.h - fake it
+ */
+#define NEWS
+#define WIZARD
+    if (argc>1 && argv[1][0]=='?'){
+	    (void) printf(
+"\nUsage:\n %s [:uname] [-d dir] -s [-[%s]] [maxrank] [name]...",
+		argv[0], classes);
+	    (void) printf("\n or");
+	    (void) printf("\n %s [-d dir] [-u name] [-[%s]]",
+		argv[0], classes);
+#if defined(WIZARD) || defined(EXPLORE_MODE)
+	    (void) printf(" [-[DX]]");
+#endif
+#ifdef NEWS
+	    (void) printf(" [-n]");
+#endif
+#ifndef AMIGA
+	    (void) printf(" [-I] [-i] [-d]");
+#endif
+#ifdef MFLOPPY
+# ifndef AMIGA
+	    (void) printf(" [-r]");
+# endif
+#endif
+#ifdef AMIGA
+	    (void) printf(" [-[lL]]");
+#endif
+	    putchar('\n');
+	    cleanup(1);
+	}
 }
 
 /* CLI */
@@ -408,7 +497,8 @@ run_game( gptr )
     }
 
     /* Set the game name for the status command */
-    sprintf( gptr->gname, "NetHack 3.1 %s", gptr->name );
+    sprintf( gptr->gname, "NetHack %d.%d.%d %s",
+      VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL, gptr->name );
 
     /* Create a process for the game to execute in */
     ctask = FindTask( NULL );
@@ -444,7 +534,12 @@ freemem:
     gptr->wbs->sm_Process = proc;
     gptr->wbs->sm_Segment = gptr->seglist;
     gptr->wbs->sm_NumArgs = 2;
-    gptr->wbs->sm_ToolWindow = "con:0/0/100/300/NetHack 3.1";
+    {
+    static char title[45];	/* some slack */
+    sprintf(title,"con:0/0/100/300/NetHack %d.%d.%d",
+      VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL);
+    gptr->wbs->sm_ToolWindow = title;
+    }
     gptr->wbs->sm_ArgList = gptr->wba;
 
     /* Fill in the args */
@@ -554,24 +649,29 @@ void
 menu_copyopt()
 {
     GPTR gptr;
-    char newname[ 100 ], oldname[ 100 ], cmd[ 200 ];
+    char newname[ 100 ], oldname[ 100 ], cmd[ 300 ], dir[ 100 ];
 
     if( ( gptr = NeedGame() ) == NULL )
 	return;
 
-    if( StrRequest( "Enter new player name", newname, gptr->fname ) == 0 )
+    if( StrRequest( "Enter new player name", newname, gptr->name ) == 0 )
 	return;
 
-    if( strcmp( newname, gptr->fname ) == 0 )
+    if( strcmp( newname, gptr->name ) == 0 )
     {
 	errmsg( FLASH, "Copying aborted, new name same as old" );
 	return;
     }
 
     strcpy( oldname, GameName( gptr, NULL ) );
-    strcpy( newname, GameName( gptr, newname ) );
 
-    sprintf( cmd, "c:copy \"%s\" \"%s\"", oldname, newname );
+    strcpy( dir, options[ SAVE_IDX ] );
+    if( strchr( "/:", dir[strlen(dir)-1] ) == 0 && *dir )
+	strcat( dir, "/" );
+    if( gptr->dobj->do_Gadget.GadgetID == GADNEWGAME )
+	sprintf( cmd, "c:copy \"%s\" \"%s%s.cfg.info\"", oldname, dir, newname );
+    else
+	sprintf( cmd, "c:copy \"%s\" \"%s%s.info\"", oldname, dir, newname );
     Execute( cmd, NULL, NULL );
     MapGadgets( R_DISK, 1 );
 }
@@ -678,6 +778,26 @@ void SafeCloseWindow( window )
     Permit();
 }
 
+void RemoveGITEM( ggptr )
+    register GPTR ggptr;
+{
+    register GPTR gptr, pgptr = NULL;
+
+    for( gptr = gamehead; gptr; pgptr = gptr, gptr = gptr->next )
+    {
+	if( gptr == ggptr )
+	{
+	    if( pgptr )
+		pgptr->next = gptr->next;
+	    else
+		gamehead = gptr->next;
+	    FreeGITEM( gptr );
+	    return;
+	}
+    }
+}
+
+
 #else   /* CLI */
 
 void CloseLibraries( )
@@ -841,6 +961,7 @@ int flash;
     {
 	vprintf( str, vp );
 	va_end( vp );
+	printf("\n");
 	return;
     }
 
@@ -901,7 +1022,7 @@ void menu_scores()
 	}
 
 	/* Add the scores entry */
-	    SetToolLine( gptr, "SCORES", *buf1 ? buf1 : "all" );
+	SetToolLine( gptr, "SCORES", *buf1 ? buf1 : "all" );
 
 	/* Get the scores */
 	run_game( gptr );
@@ -959,8 +1080,6 @@ int IsEditEntry( str, gptr )
     char *str;
     register GPTR gptr;
 {
-    if( gptr->dobj->do_Gadget.GadgetID == GADNEWGAME )
-	return( 1 );
     if( strncmp( str, "CHARACTER=", 10 ) == 0 )
 	return( 0 );
     return( 1 );
@@ -1334,7 +1453,7 @@ void SetBorder( gd, val )
 	sp[1] = gd->Height - 1;
 	sp[2] = -1;
 	sp[3] = -1;
-	sp[4] = gd->Width - 1;
+	sp[4] = gd->Width-1;
 	sp[5] = -1;
 
 	sp[6] = gd->Width + 1;
@@ -1502,23 +1621,12 @@ void CopyOptions( optr, gptr )
     OPTR optr;
     GPTR gptr;
 {
-    char **sp;
+    char *sp;
 
-    for( sp = gptr->dobj->do_ToolTypes; *sp; ++sp )
-    {
-	if( strnicmp( *sp, "options=", 8 ) == 0 )
-	{
-	    break;
-	}
-    }
-
-    if( *sp == NULL )
-    {
-	errmsg( NO_FLASH, "Options not set for %s", gptr->name );
-	return;
-    }
-
-    CopyOptionStr( optr, *sp + 8 );
+    sp = ToolsEntry( gptr, "OPTIONS" );
+    ZapOptions( optr );
+    if( sp && *sp )
+	CopyOptionStr( optr, sp );
 }
 
 /* !CLI */
@@ -1537,7 +1645,7 @@ void CopyOptionStr( optr, str )
     sidx = -1;
 
     /* Start past the 'options=' part */
-    for( t = str; !done; ++t )
+    for( t = str; *str && !done; ++t )
     {
 	if( state == 0 && isspace( *t ) )
 	    continue;
@@ -1584,7 +1692,7 @@ void CopyOptionStr( optr, str )
 	}
 
 	/* If at end of string or comma and we have some text... */
-	if( !*t || *t == ',' && *buf )
+	if( ( !*t || *t == ',' ) && *buf )
 	{
 	    /* Mark end */
 	    *s = 0;
@@ -1629,7 +1737,7 @@ void CopyOptionStr( optr, str )
 		}
 		else
 		{
-		    errmsg( FLASH, "Unrecognized option %s", buf );
+		    errmsg( FLASH, "Unrecognized option `%s'", buf );
 		    return;
 		}
 	    }
@@ -1658,7 +1766,6 @@ void SetOptions( optr, gptr )
 {
     PutOptions( optr );
     SetToolLine( gptr, "OPTIONS", options[ OPTIONS_IDX ] );
-    UpdateGameIcon( gptr );
 }
 
 void
@@ -1699,17 +1806,23 @@ char *ToolsEntry( gptr, name )
  * later to clean up whatever "GetDiskObject" allocated.
  */
 void ReallocTools( gptr, add )
-    GPTR gptr;
-    int add;
+    register GPTR gptr;
+    register int add;
 {
-    int i, cnt;
-    char **sp, **tp;
+    register int i, cnt;
+    register char **sp, **tp;
+
+    /* Already allocated */
+    if( gptr->talloc && add == 0 )
+    	return;
 
     for( cnt = 0, tp = gptr->dobj->do_ToolTypes; tp && *tp ; ++tp )
 	++cnt;
 
-    if( !tp )
+    if( !tp || cnt == 0 )
     {
+	if( gptr->talloc )
+	    free( gptr->dobj->do_ToolTypes );
 	/* If no tooltypes array, fudge something to start with */
 	if( sp = xmalloc( 2 * sizeof( char * ) ) )
 	{
@@ -1725,19 +1838,23 @@ void ReallocTools( gptr, add )
 	    sp[i++] = strdup( *tp );
 	}
 
-	sp[i] = NULL;
+	if( gptr->talloc && gptr->dobj->do_ToolTypes )
+	    free( gptr->dobj->do_ToolTypes );
+	while( i < cnt+add+1 )
+	    sp[ i++ ] = NULL;
     }
-    gptr->otools = gptr->dobj->do_ToolTypes;
+    if( ! gptr->talloc )
+	gptr->otools = gptr->dobj->do_ToolTypes;
     gptr->dobj->do_ToolTypes = sp;
     gptr->toolcnt = cnt + 1;
     gptr->talloc = 1;
 }
 
 void FreeTools( gptr )
-    GPTR gptr;
+    register GPTR gptr;
 {
-    int i;
-    char **sp;
+    register int i;
+    register char **sp;
 
     if( !gptr->talloc )
 	return;
@@ -1761,8 +1878,7 @@ void SetToolLine( gptr, name, value )
 
     /* Realloc ToolTypes to be in memory we know how to manage */
 
-    if( gptr->talloc == 0 )
-	ReallocTools( gptr, 0 );
+    ReallocTools( gptr, 0 );
 
     sp = gptr->dobj->do_ToolTypes;
     len = strlen( name );
