@@ -16,6 +16,11 @@
 #include <sys/wait.h>
 #endif
 
+#ifdef _M_UNIX
+extern void NDECL(sco_mapon);
+extern void NDECL(sco_mapoff);
+#endif
+
 static struct stat buf, hbuf;
 
 void
@@ -155,14 +160,18 @@ getlock()
 	extern int errno;
 	register int i = 0, fd, c;
 
+#ifdef TTY_GRAPHICS
 	/* idea from rpick%ucqais@uccba.uc.edu
 	 * prevent automated rerolling of characters
 	 * test input (fd0) so that tee'ing output to get a screen dump still
 	 * works
 	 * also incidentally prevents development of any hack-o-matic programs
 	 */
-	if (!isatty(0))
+	/* added check for window-system type -dlc */
+	if (!strcmp(windowprocs.name, "tty"))
+	    if (!isatty(0))
 		error("You must play from a terminal.");
+#endif
 
 	/* we ignore QUIT and INT at this point */
 	if (!lock_file(HLOCK, 10)) {
@@ -170,14 +179,22 @@ getlock()
 		error("");
 	}
 
+#ifdef DEBIAN
+	regularize(lock+23);
+#else  /* NOT DEBIAN */
 	regularize(lock);
+#endif /* NOT DEBIAN */
 	set_levelfile_name(lock, 0);
 
 	if(locknum) {
 		if(locknum > 25) locknum = 25;
 
 		do {
+#ifdef DEBIAN
+			lock[23] = 'a' + i++;
+#else  /* NOT DEBIAN */
 			lock[0] = 'a' + i++;
+#endif /* NOT DEBIAN */
 
 			if((fd = open(lock, 0)) == -1) {
 			    if(errno == ENOENT) goto gotlock; /* no such file */
@@ -251,16 +268,24 @@ regularize(s)	/* normalize file name - we don't like .'s, /'s, spaces */
 register char *s;
 {
 	register char *lp;
+#if defined(SYSV) && !defined(AIX_31) && defined(COMPRESS)
+	int i;
+#endif
 
 	while((lp=index(s, '.')) || (lp=index(s, '/')) || (lp=index(s,' ')))
 		*lp = '_';
 #if defined(SYSV) && !defined(AIX_31)
 	/* avoid problems with 14 character file name limit */
 # ifdef COMPRESS
-	if(strlen(s) > 10)
-		/* leave room for .e from error and .Z from compress
-		 * appended to save files */
-		s[10] = '\0';
+	/* leave room for .e from error and .Z from compress appended to
+	 * save files */
+#  ifdef COMPRESS_EXTENSION
+	i = 12 - strlen(COMPRESS_EXTENSION);
+#  else
+	i = 10;		/* should never happen... */
+#  endif
+	if(strlen(s) > i)
+		s[i] = '\0';
 # else
 	if(strlen(s) > 11)
 		/* leave room for .nn appended to level files */
@@ -293,6 +318,9 @@ int wt;
 {
 	register int f;
 	suspend_nhwindows(NULL);	/* also calls end_screen() */
+#ifdef _M_UNIX
+	sco_mapon();
+#endif
 	if((f = fork()) == 0){		/* child */
 		(void) setgid(getgid());
 		(void) setuid(getuid());
@@ -309,6 +337,9 @@ int wt;
 	(void) signal(SIGINT,SIG_IGN);
 	(void) signal(SIGQUIT,SIG_IGN);
 	(void) wait( (int *) 0);
+#ifdef _M_UNIX
+	sco_mapoff();
+#endif
 	(void) signal(SIGINT, (SIG_RET_TYPE) done1);
 #ifdef WIZARD
 	if(wizard) (void) signal(SIGQUIT,SIG_DFL);
