@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)lev_main.c	3.1	92/12/11	*/
+/*	SCCS Id: @(#)lev_main.c	3.1	93/07/10	*/
 /*	Copyright (c) 1989 by Jean-Christophe Collet */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -56,8 +56,8 @@ extern void FDECL (init_yyin, (FILE *));
 extern void FDECL (init_yyout, (FILE *));
 
 int  FDECL (main, (int, char **));
-void FDECL (yyerror, (char *));
-void FDECL (yywarning, (char *));
+void FDECL (yyerror, (const char *));
+void FDECL (yywarning, (const char *));
 int  NDECL (yywrap);
 char *FDECL(dup_string,(const char *));
 int FDECL(get_floor_type, (CHAR_P));
@@ -71,7 +71,7 @@ char FDECL(what_map_char, (CHAR_P));
 void FDECL(scan_map, (char *));
 void NDECL(wallify_map);
 boolean NDECL(check_subrooms);
-void FDECL(check_coord, (int, int, char *));
+void FDECL(check_coord, (int, int, const char *));
 void NDECL(store_part);
 void NDECL(store_room);
 static void FDECL(write_common_data, (int,int,lev_init *,long));
@@ -81,8 +81,8 @@ void FDECL(free_rooms, (room **, int));
 
 static struct {
 	const char *name;
-	short type;
-} trap_types[TRAPNUM] = {
+	int type;
+} trap_types[] = {
 	{ "arrow",	ARROW_TRAP },
 	{ "dart",	DART_TRAP },
 	{ "falling rock", ROCKTRAP },
@@ -102,14 +102,15 @@ static struct {
 	{ "statue",	STATUE_TRAP },
 	{ "magic",	MAGIC_TRAP },
 	{ "anti magic",	ANTI_MAGIC },
-#ifdef POLYSELF
-	{ "polymorph",	POLY_TRAP },
+#ifndef POLYSELF
+#define POLY_TRAP NO_TRAP	/* this will result in a random trap */
 #endif
+	{ "polymorph",	POLY_TRAP },
 	{ 0, 0 }
 };
 
 static struct {
-	char *name;
+	const char *name;
 	int type;
 } room_types[] = {
 	/* for historical reasons, room types are not contiguous numbers */
@@ -140,18 +141,21 @@ static struct {
 	{ 0, 0 }
 };
 
-char *fname = "(stdin)";
+const char *fname = "(stdin)";
 int fatal_error = 0;
 int want_warnings = 0;
 
-/* Flex 2.3 bug work around */
+#ifdef FLEX23_BUG
+/* Flex 2.3 bug work around; not needed for 2.3.6 or later */
 int yy_more_len = 0;
+#endif
 
 extern char tmpmessage[];
 extern altar *tmpaltar[];
 extern lad *tmplad[];
 extern stair *tmpstair[];
 extern digpos *tmpdig[];
+extern digpos *tmppass[];
 extern char *tmpmap[];
 extern region *tmpreg[];
 extern lev_region *tmplreg[];
@@ -174,7 +178,7 @@ extern corridor *tmpcor[];
 extern int n_olist, n_mlist, n_plist;
 
 extern unsigned int nlreg, nreg, ndoor, ntrap, nmons, nobj;
-extern unsigned int ndb, nwalk, npart, ndig, nlad, nstair;
+extern unsigned int ndb, nwalk, npart, ndig, npass, nlad, nstair;
 extern unsigned int naltar, ncorridor, nrooms, ngold, nengraving;
 extern unsigned int nfountain, npool, nsink;
 
@@ -189,7 +193,7 @@ char **argv;
 {
 	FILE *fin;
 	int i;
-#ifdef MAC_THINKC5
+#ifdef THINK_C
 	static char *mac_argv[] = {	"lev_comp",	/* dummy argv[0] */
 				":dat:Arch.des",
 				":dat:Barb.des",
@@ -267,7 +271,7 @@ char **argv;
 
 void
 yyerror(s)
-char *s;
+const char *s;
 {
 	(void) fprintf(stderr, "%s: line %d : %s\n", fname,
 		(*s >= 'A' && *s <= 'Z') ? colon_line_number : line_number, s);
@@ -283,7 +287,7 @@ char *s;
 
 void
 yywarning(s)
-char *s;
+const char *s;
 {
 	(void) fprintf(stderr, "%s: line %d : WARNING : %s\n",
 				fname, colon_line_number, s);
@@ -359,9 +363,9 @@ char *s;
 	register int i;
 
 	SpinCursor(3);
-	for(i=0; i < TRAPNUM - 1; i++)
+	for (i=0; trap_types[i].name; i++)
 	    if(!strcmp(s,trap_types[i].name))
-		return((int)trap_types[i].type);
+		return trap_types[i].type;
 	return ERR;
 }
 
@@ -441,6 +445,7 @@ what_map_char(c)
 		  case 'B'  : return(CROSSWALL); /* hack: boundary location */
 		  case 'C'  : return(CLOUD);
 		  case 'S'  : return(SDOOR);
+		  case 'H'  : return(SCORR);
 		  case '{'  : return(FOUNTAIN);
 		  case '\\' : return(THRONE);
 		  case 'K'  :
@@ -605,7 +610,7 @@ check_subrooms()
 void
 check_coord(x, y, str)
 int x, y;
-char *str;
+const char *str;
 {
     char ebuf[60];
 
@@ -708,6 +713,15 @@ store_part()
 	}
 	ndig = 0;
 
+	/* The non_passwall directives */
+
+	if ((tmppart[npart]->npass = npass) != 0) {
+		tmppart[npart]->passs = NewTab(digpos, npass);
+		for(i=0;i<npass;i++)
+		    tmppart[npart]->passs[i] = tmppass[i];
+	}
+	npass = 0;
+
 	/* The ladders */
 
 	if ((tmppart[npart]->nlad = nlad) != 0) {
@@ -1060,6 +1074,15 @@ specialmaze *maze;
 	    if (pt->ndig > 0)
 		    Free(pt->digs);
 
+	    /* The non_passwall directives */
+	    Write(fd, &(pt->npass), sizeof(pt->npass));
+	    for(j=0;j<pt->npass;j++) {
+		    Write(fd, pt->passs[j], sizeof(digpos));
+		    Free(pt->passs[j]);
+	    }
+	    if (pt->npass > 0)
+		    Free(pt->passs);
+
 	    /* The ladders */
 	    Write(fd, &(pt->nlad), sizeof(pt->nlad));
 	    for(j=0;j<pt->nlad;j++) {
